﻿using System;
using System.Linq;
using UnityEngine;
using System.Text;

using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using MacGruber;
using octopussy;
using Random = UnityEngine.Random;

namespace SPQR
{
    public class AEDoll : MVRScript
    {

        private EventTrigger myEventTrigger;
        protected JSONStorableString myString;

        private LineDrawer walkLineDrawer;
        private LineDrawer walkLineDirectionDrawer;
        private bool clickdown = false;
        private int holdCooldownDefault = 20;
        private int holdCooldown = 20;

        Dictionary<string, string> controlsTypes;
        Dictionary<string, float> controlsJoints;
        Dictionary<string, FreeControllerV3> controls;
        Dictionary<string, Rigidbody> bones;
        Dictionary<string, string> controlsBones;
        Dictionary<string, int> controlsCollision;
        Dictionary<string, string> controlsTired;
        Dictionary<string, string> controlsSway;
        Dictionary<string, string> controlsJitters;
        Dictionary<string, float> controlsHandmade;
        Dictionary<string, bool> controlsHandmadeReset;


        bool previouslyVertical = false;

        Dictionary<string, JSONStorableBool> enableFeatures = new Dictionary<string, JSONStorableBool>();


        bool paused = false;
        bool holdingHeadStarted = false;
        float jointFactorInit = 3f;
        float jointFactor = 3f;
        bool simHold = false;

        JSONStorableFloat tiredHoldJSON;
        JSONStorableFloat flexibilityJSON;
        JSONStorableFloat tiredheadTimeJSON;
        JSONStorableFloat jointFactorJSON;

        string holdControl = "neckControl";

        Dictionary<string, int> actionsHappening;

        Quaternion headRotation;

        int titles = 0;

        public override void Init()
        {
            Utils.OnInitUI(CreateUIElement);

            actionsHappening = new Dictionary<string, int>();


            myEventTrigger = new EventTrigger(this, "OnTrigger");
            // Utils.SetupButton(this, "Actions OnTrigger", myEventTrigger.OpenPanel, false);

            // Utils.SetupButton(this, "Test Trigger", TestTrigger, true);
            // Utils.SetupAction(this, "Trigger", Trigger);
            SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;
            SimpleTriggerHandler.LoadAssets();

            CreateTitle("ÆDoll",60,false);
            CreateTitle("ÆTERNUM TOOLS",30,true);

            Utils.SetupSpacer(this, 10, false);
            Utils.SetupSpacer(this, 10, true);

            bool defaultEnabled = true;

            List<string> choices = new List<string>();
            choices.Add("neckControl");
            choices.Add("headControl");
            choices.Add("pelvisControl");
            choices.Add("lArmControl");
            choices.Add("rArmControl");


            // jointFactorJSON = new JSONStorableFloat("Flexibility", 1f, tiredHoldJSONUpdate, 1f, 300f, true);
            // RegisterFloat(jointFactorJSON);
            // UIDynamicSlider jointFactorJSONSlider = CreateSlider(jointFactorJSON,true);


            

            //CreateScrollablePopup(jchooser, true);

            CreateTitle("Person Grab",30,false);
            JSONStorableString jstring = new JSONStorableString("instructions", "When enabled it stops the resistance of the other body parts when moving the selected grab controller.");
            UIDynamicTextField dtext = CreateTextField(jstring, false);
            dtext.UItext.supportRichText = true;
            dtext.backgroundColor = new Color(0.42f, 0.03f, 0.51f, 0.7f);
            dtext.textColor = new Color(1f, 1f, 1f, 0.9f);
            dtext.height = 140f;

            
         
            CreateTitle("Auto Movements",30,false);
            JSONStorableString jstring2 = new JSONStorableString("instructions2", "These features add micromovement to all body parts. Jitters adds very small jerks to the arms and legs. Jiggles add motion to all body parts. Sway adds movement to hips,chest and shoulders. Work better when Tired mode is turned off.");
            UIDynamicTextField dtext2 = CreateTextField(jstring2, false);
            dtext2.UItext.supportRichText = true;
            dtext2.backgroundColor = new Color(0.42f, 0.03f, 0.51f, 0.7f);
            dtext2.textColor = new Color(1f, 1f, 1f, 0.9f);
            dtext2.height = 250f;
            
          

            CreateTitle("Auto Reposition",30,false);

            JSONStorableString jstring3 = new JSONStorableString("instructions3", "Makes all body parts except the head get tired and reposition based on gravity. Tired Time is how many seconds on average to try to keep the position for a controller until it gets tired. If you move a hand for example and hold it for 2s on a position it will snap there and then will get tired and fall off after at least the set time (can be a few seconds more, auto gravitiy is done with probabilities to avoid it to be too robotic )");
            UIDynamicTextField dtext3 = CreateTextField(jstring3, false);
            dtext3.UItext.supportRichText = true;
            dtext3.backgroundColor = new Color(0.42f, 0.03f, 0.51f, 0.7f);
            dtext3.textColor = new Color(1f, 1f, 1f, 0.9f);
            dtext3.height = 200f;
            
        

             CreateTitle("Other",30,false);

            JSONStorableString jstring4= new JSONStorableString("instructions4", "<b>Flexibilty:</b> changes how flexible the model is. The VAM default for this is 10.\n <b>Auto Head:</b> makes the head automatically go in position off when detected it's close to the selected value in height from the ground compared the hip's height. It automatically reenables it when the position changes.");
            UIDynamicTextField dtext4 = CreateTextField(jstring4, false);
            dtext4.UItext.supportRichText = true;
            dtext4.backgroundColor = new Color(0.42f, 0.03f, 0.51f, 0.7f);
            dtext4.textColor = new Color(1f, 1f, 1f, 0.9f);
            dtext4.height = 200f;

        



            //fields
            Utils.SetupSpacer(this,100, true);

            enableFeatures["Grab"] = new JSONStorableBool("Grab Enabled", defaultEnabled);
            enableFeatures["Grab"].storeType = JSONStorableParam.StoreType.Full;
            RegisterBool(enableFeatures["Grab"]);
            CreateToggle(enableFeatures["Grab"], true);

            JSONStorableStringChooser jchooser = new JSONStorableStringChooser("GrabController", choices, "headControl", "Grab controller", DummyChooserCallback);
            UIDynamicPopup udp = CreatePopup(jchooser, true);
            RegisterStringChooser(jchooser);
            udp.labelWidth = 300f;

            holdControl = jchooser.val;


            Utils.SetupSpacer(this,100, true);

            enableFeatures["Jitters"] = new JSONStorableBool("Jitters", true);
            enableFeatures["Jitters"].storeType = JSONStorableParam.StoreType.Full;
            RegisterBool(enableFeatures["Jitters"]);
            CreateToggle(enableFeatures["Jitters"], true);

            enableFeatures["JiggleSoft"] = new JSONStorableBool("JiggleSoft", defaultEnabled);
            enableFeatures["JiggleSoft"].storeType = JSONStorableParam.StoreType.Full;
            RegisterBool(enableFeatures["JiggleSoft"]);
            CreateToggle(enableFeatures["JiggleSoft"], true);

            enableFeatures["JiggleMedium"] = new JSONStorableBool("JiggleMedium", defaultEnabled);
            enableFeatures["JiggleMedium"].storeType = JSONStorableParam.StoreType.Full;
            RegisterBool(enableFeatures["JiggleMedium"]);
            CreateToggle(enableFeatures["JiggleMedium"], true);

            enableFeatures["Sway"] = new JSONStorableBool("Sway", defaultEnabled);
            enableFeatures["Sway"].storeType = JSONStorableParam.StoreType.Full;
            RegisterBool(enableFeatures["Sway"]);
            CreateToggle(enableFeatures["Sway"], true);


            Utils.SetupSpacer(this,100, true);

            enableFeatures["Tired"] = new JSONStorableBool("Tired Mode", false);
            enableFeatures["Tired"].storeType = JSONStorableParam.StoreType.Full;
            RegisterBool(enableFeatures["Tired"]);
            CreateToggle(enableFeatures["Tired"], true);


            tiredHoldJSON = new JSONStorableFloat("TiredTime", 10f, tiredHoldJSONUpdate, 0f, 300f, true);
            RegisterFloat(tiredHoldJSON);
            UIDynamicSlider tiredHoldJSONSlider = CreateSlider(tiredHoldJSON,true);


            Utils.SetupSpacer(this,100, true);

            flexibilityJSON = new JSONStorableFloat("Flexibility", 2.5f, tiredHoldJSONUpdate, 0f, 20f, true);
            RegisterFloat(flexibilityJSON);
            UIDynamicSlider flexibilityJSONSlider = CreateSlider(flexibilityJSON,true);




            enableFeatures["TiredHead"] = new JSONStorableBool("Auto Head", false);
            enableFeatures["TiredHead"].storeType = JSONStorableParam.StoreType.Full;
            RegisterBool(enableFeatures["TiredHead"]);
            CreateToggle(enableFeatures["TiredHead"], true);
            

            tiredheadTimeJSON = new JSONStorableFloat("Auto Head Distance", 0.3f, tiredHoldJSONUpdate, -1f, 1f, true);
            RegisterFloat(tiredheadTimeJSON);
            UIDynamicSlider tiredheadTimeJSONSlider = CreateSlider(tiredheadTimeJSON,true);

          



            Utils.SetupSpacer(this, 50, false);
            Utils.SetupSpacer(this,50, true);

            // myString = new JSONStorableString("myString", "", UpdateMyString);
            // RegisterString(myString);
            // UIDynamicLabelInput myStringInput = Utils.SetupTextInput(this, "My String", myString, false);

            SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;
            UIUpdate();

            PrepareControllers();
            PrepareJointDrives();

        }
        private void CreateTitle(string text, int fontsize, bool right,int spacing){
            CreateTitle(text, fontsize, right);
            Utils.SetupSpacer(this, spacing, right);
        }
        private void CreateTitle(string text, int fontsize, bool right){
                JSONStorableString title1 = new JSONStorableString("title"+(++titles), "\n<size="+fontsize+"><b>"+text+"</b></size>");
                UIDynamicTextField title1text = CreateTextField(title1, right);
                title1text.UItext.supportRichText = true;
                title1text.backgroundColor = new Color(0f, 0f, 0f, 0f);
                title1text.textColor = new Color(0.42f, 0.03f, 0.51f, 1f);
                title1text.height = 100f;
                title1text.UItext.alignment = right ? TextAnchor.MiddleRight : TextAnchor.MiddleLeft;
        }
        private void OnAtomRename(string oldid, string newid)
        {
            myEventTrigger.SyncAtomNames();
        }
        private void UIUpdate()
        {
        }

        protected void DummyChooserCallback(string s) {
			 holdControl = s;
		}


        void Start()
        {
            paused = false;
            if (containingAtom.type != "Person") return;

            try
            {
                InvokeRepeating("UpdateSecond", 1f, 1f);
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        void UpdateSecond()
        {
            if(paused) return;

            if(IsEnabled("Grab")){
                if(holdingHeadStarted) return;
                controls[holdControl].interactableInPlayMode = true;
            }

           
            try
            {

                foreach (var item in controlsTypes)
                {
                    controls[item.Key].RBMaxVelocity = flexibilityJSON.val;

                    if(controls[item.Key].currentPositionState == FreeControllerV3.PositionState.ParentLink){
                        controlsHandmade[item.Key] = Time.time;
                        controlsHandmadeReset[item.Key] = true;
                    } else{
                        if(controlsHandmadeReset[item.Key]){
                            controls[item.Key].currentPositionState = FreeControllerV3.PositionState.On;
                            controls[item.Key].currentRotationState = FreeControllerV3.RotationState.On;
                            controlsHandmadeReset[item.Key] = false;
                        }
                    }

                     
                     doSoftJiggle(item.Key,2f,1);
                     doMediumJiggle(item.Key,10f,1);

                    if (controlsJitters.ContainsKey(item.Key)){
                        doJitters(item.Key,2f,1);
                    }

                    if (controlsSway.ContainsKey(item.Key)){
                        doSway(item.Key, 5f, 2);
                    }

                    if (controlsTired.ContainsKey(item.Key)){
                        doTired(item.Key, 1f, 2);
                    }
                }



            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }


        }

        void doTired(string s, float chance, int limit)
        {

            
            if(Time.time - controlsHandmade[s] <= tiredHoldJSON.val){
                // SuperController.LogMessage("Skipped!");
                return; 
            }
            if(!IsEnabled("Tired")) return;
            var actionType = "Tired";
            var actionElement = s;
            var actionName = actionType + actionElement;
            if (ActionChanceDoLimit(chance, actionName, limit))
            {
               // SuperController.LogMessage("doing " + actionName + " (" + ActionGetCount(actionName) + ")");
                ActionMovementTired(actionType, actionElement, actionName);
            }
        }

         void doJitters(string s, float chance, int limit)
        {
            if(!IsEnabled("Jitters")) return;
            var actionType = "Jitters";
            var actionElement = s;
            var actionName = actionType + actionElement;
            if (ActionChanceDoLimit(chance, actionName, limit))
            {
                //SuperController.LogMessage("doing " + actionName + " (" + ActionGetCount(actionName) + ")");
                ActionMovementJiggle(actionType, actionElement, actionName);
            }
        }

        void doSoftJiggle(string s, float chance, int limit)
        {
            if(!IsEnabled("JiggleSoft")) return;
            var actionType = "JiggleSoft";
            var actionElement = s;
            var actionName = actionType + actionElement;
            if (ActionChanceDoLimit(chance, actionName, limit))
            {
                //SuperController.LogMessage("doing " + actionName + " (" + ActionGetCount(actionName) + ")");
                ActionMovementJiggle(actionType, actionElement, actionName);
            }
        }

        void doMediumJiggle(string s, float chance, int limit)
        {
            if(!IsEnabled("JiggleMedium")) return;
            var actionType = "JiggleMedium";
            var actionElement = s;
            var actionName = actionType + actionElement;
            if (ActionChanceDoLimit(chance, actionName, limit))
            {
                //SuperController.LogMessage("doing " + actionName + " (" + ActionGetCount(actionName) + ")");
                ActionMovementJiggle(actionType, actionElement, actionName);
            }
        }

        void doSway(string s, float chance, int limit)
        {
            if(!IsEnabled("Sway")) return;
            var actionType = "Sway";
            var actionElement = s;
            var actionName = actionType + actionElement;
            if (ActionChanceDoLimit(chance, actionName, limit))
            {
                //SuperController.LogMessage("doing " + actionName + " (" + ActionGetCount(actionName) + ")");
                ActionMovementSway(actionType, actionElement, actionName);
            }
        }

        bool HoldingHead(){
            if(simHold) return true;
            if(!IsEnabled("Grab")) return false;
            if(controls[holdControl].currentPositionState == FreeControllerV3.PositionState.ParentLink) return true;
            if(controls[holdControl].currentPositionState == FreeControllerV3.PositionState.Hold) return true;
            return false;
        }

        bool HoldingRealHead(){
            if(!IsEnabled("Grab")) return false;
            if(controls["headControl"].currentPositionState == FreeControllerV3.PositionState.ParentLink) return true;
            if(controls["headControl"].currentPositionState == FreeControllerV3.PositionState.Hold) return true;
            return false;
        }

        void holdingHeadStart(){
            if(!IsEnabled("Grab")) return;
            headRotation = controls["headControl"].transform.rotation;
            //SuperController.LogMessage("holdingHeadStart");
            foreach (var item in controlsTypes)
            {
                if(item.Key == holdControl) continue;
               // if(item.Value != "main") continue;
                
                controls[item.Key].currentPositionState = FreeControllerV3.PositionState.Off;
                controls[item.Key].currentRotationState = FreeControllerV3.RotationState.Off;

                // if(controlsJoints.ContainsKey(item.Key)){
                //     controls[item.Key].jointRotationDriveSpring = controlsJoints[item.Key] * jointFactor;
                // }

                
            }

         

        }


       

        void holdingHeadStop(){
            if(!IsEnabled("Grab")) return;
         //   SuperController.LogMessage("holdingHeadStop");
            foreach (var item in controlsTypes)
            {
                if(item.Key == holdControl && item.Value != "main"){
                    controls[item.Key].currentPositionState = FreeControllerV3.PositionState.Off;
                    controls[item.Key].currentRotationState = FreeControllerV3.RotationState.Off;
                    continue;
                }
                if(item.Value != "main") continue;

                // SuperController.LogMessage("holdingHeadStop on :"+item.Key);
                
                controls[item.Key].currentPositionState = FreeControllerV3.PositionState.On;
                controls[item.Key].currentRotationState = FreeControllerV3.RotationState.On;
                
            }
           // controls["headControl"].transform.rotation = headRotation;
            MoveResetControl();
        }

        void UpdateVertical(){
            if(HoldingHead()) return;
            if(!IsEnabled("TiredHead")) return;

            if(controls["headControl"].currentPositionState == FreeControllerV3.PositionState.ParentLink) return;
            if(controls["headControl"].currentPositionState == FreeControllerV3.PositionState.Hold) return;

            //SuperController.LogMessage("vertical"+isVertical().ToString());
            if(isVertical()  ){
                controls["headControl"].currentPositionState = FreeControllerV3.PositionState.On;
                controls["headControl"].currentRotationState = FreeControllerV3.RotationState.On;
            } else{
                controls["headControl"].currentPositionState = FreeControllerV3.PositionState.Off;
                controls["headControl"].currentRotationState = FreeControllerV3.RotationState.Off;
            }
        }

        void Update()
        {
            try
            {
                
                ClickMoveUpdate();
                UpdateVertical();

                if(HoldingHead()){
                    if(!holdingHeadStarted){
                        holdingHeadStart();
                        holdingHeadStarted = true;
                    }
                } else if(holdingHeadStarted){
                    holdingHeadStop();
                    holdingHeadStarted = false;
                }

                if(!HoldingRealHead() && !HoldingHead() && IsEnabled("Tired")){
                    if(isVertical()){
                        controls["headControl"].currentPositionState = FreeControllerV3.PositionState.On;
                        controls["headControl"].currentPositionState = FreeControllerV3.PositionState.On;
                    } else {
                        controls["headControl"].currentPositionState = FreeControllerV3.PositionState.Off;
                        controls["headControl"].currentPositionState = FreeControllerV3.PositionState.Off;
                    }
                }



                myEventTrigger.Update();
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }
        private void TestTrigger()
        {
            // var jc = this.GetJSON(true, true, false);
            // SuperController.LogMessage("current : " + jc.ToString().Replace("\"", "\\\""));
            //SuperController.LogMessage("isVertical:" + isVertical().ToString());

        }
        private void Trigger()
        {
            myEventTrigger.Trigger();
        }

        void FixedUpdate()
        {
            
        }

        void OnDisable(){
            paused = true;
        }
        void OnEnable(){
            paused = false;
        }

        void OnDestroy()
        {
            paused = true;
            SuperController.singleton.onAtomUIDRenameHandlers -= OnAtomRename;
            myEventTrigger.Remove();
            Utils.OnDestroyUI();

            foreach (var item in controlsTypes)
            {
                var prev = bones[item.Key].gameObject.GetComponent<TriggerCollide>();
                if (prev != null) prev.OnCollide -= ObserveCollide;
            }


        }

        public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
        {
            JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
            if (includePhysical || forceStore)
            {
                needsStore = true;
                jc[myEventTrigger.Name] = myEventTrigger.GetJSON(base.subScenePrefix);
            }
            return jc;
        }
        public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
        {
            base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);

            if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
            {
                myEventTrigger.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
            }
        }
        void UpdateMyString(string name)
        {
        }


        //////////////////////////////////////////////

        public bool IsEnabled(string s){
            if(!enableFeatures.ContainsKey(s)) return false;
            return enableFeatures[s].val;
        }

          void PrepareJointDrives(){
            containingAtom.GetStorableByID("headControl").GetFloatJSONParam("jointDriveXTarget").val = -5f;
            containingAtom.GetStorableByID("neckControl").GetFloatJSONParam("jointDriveXTarget").val = -30f;
            containingAtom.GetStorableByID("chestControl").GetFloatJSONParam("jointDriveXTarget").val = 0f;
            containingAtom.GetStorableByID("abdomenControl").GetFloatJSONParam("jointDriveXTarget").val = 20f;
            containingAtom.GetStorableByID("abdomen2Control").GetFloatJSONParam("jointDriveXTarget").val = -15f;
            containingAtom.GetStorableByID("pelvisControl").GetFloatJSONParam("jointDriveXTarget").val = 15f;

            containingAtom.GetStorableByID("lArmControl").GetFloatJSONParam("jointDriveXTarget").val = -75f;
            containingAtom.GetStorableByID("rArmControl").GetFloatJSONParam("jointDriveXTarget").val = 75f;

            containingAtom.GetStorableByID("lHandControl").GetFloatJSONParam("jointDriveXTarget").val = 5f;
            containingAtom.GetStorableByID("rHandControl").GetFloatJSONParam("jointDriveXTarget").val = -5f;

            containingAtom.GetStorableByID("lElbowControl").GetFloatJSONParam("jointDriveXTarget").val = -5f;
            containingAtom.GetStorableByID("lElbowControl").GetFloatJSONParam("jointDriveYTarget").val = -15f;
            containingAtom.GetStorableByID("rElbowControl").GetFloatJSONParam("jointDriveXTarget").val = 5f;
            containingAtom.GetStorableByID("rElbowControl").GetFloatJSONParam("jointDriveYTarget").val = 15f;

            containingAtom.GetStorableByID("lShoulderControl").GetFloatJSONParam("jointDriveXTarget").val = -2f;
            containingAtom.GetStorableByID("lShoulderControl").GetFloatJSONParam("jointDriveYTarget").val = -10f;
            containingAtom.GetStorableByID("lShoulderControl").GetFloatJSONParam("jointDriveZTarget").val = -8f;

            containingAtom.GetStorableByID("rShoulderControl").GetFloatJSONParam("jointDriveXTarget").val = 2f;
            containingAtom.GetStorableByID("rShoulderControl").GetFloatJSONParam("jointDriveYTarget").val = 10f;
            containingAtom.GetStorableByID("rShoulderControl").GetFloatJSONParam("jointDriveZTarget").val = 8f;

        }


        bool isVertical(){
            float diff = controls["headControl"].transform.position.y - controls["hipControl"].transform.position.y;
            if(diff < tiredheadTimeJSON.val) return false;
            return true;
        }


        void PrepareControllers()
        {
           

            controlsTypes = new Dictionary<string, string>(){
                {"headControl", "main"},
                {"chestControl", "main"},
                {"hipControl", "main"},
                {"lElbowControl", "main"},
                {"rElbowControl", "main"},
                {"lHandControl", "main"},
                {"rHandControl", "main"},
                {"lKneeControl", "main"},
                {"rKneeControl", "main"},
                {"lFootControl", "main"},
                {"rFootControl", "main"},
                {"neckControl", "secondary"},
                {"pelvisControl", "secondary"},
                {"abdomenControl", "secondary"},
                {"abdomen2Control", "secondary"},
                {"lShoulderControl", "secondary"},
                {"rShoulderControl", "secondary"},
                {"lArmControl", "secondary"},
                {"rArmControl", "secondary"},
                {"lThighControl", "secondary"},
                {"rThighControl", "secondary"},
                {"lToeControl", "secondary"},
                {"rToeControl", "secondary"},
            };
            
            controlsJoints = new Dictionary<string, float>(){
            {"lKneeControl", 40f},
            {"rKneeControl", 40f},
            {"lThighControl", 50f},
            {"rThighControl", 50f},
            {"lFootControl", 50f},
            {"rFootControl", 50f},
            {"lArmControl", 40f},
            {"rArmControl", 40f},
            {"lHandControl", 40f},
            {"rHandControl", 40f},
            {"lElbowControl", 25f},
            {"rElbowControl", 25f},
            };


            controlsHandmadeReset = new Dictionary<string, bool>() { };
            controlsCollision = new Dictionary<string, int>() { };
            controls = new Dictionary<string, FreeControllerV3>() { };
            bones = new Dictionary<string, Rigidbody>() { };

             controlsTired = new Dictionary<string, string>(){
                // {"headControl", "main"},
                {"chestControl", "main"},
                {"lHandControl", "main"},
                {"rHandControl", "main"},
                {"lElbowControl", "main"},
                {"rElbowControl", "main"},
                {"lShoulderControl", "secondary"},
                {"rShoulderControl", "secondary"},
                {"lArmControl", "main"},
                {"rArmControl", "main"},
                {"hipControl", "main"},
                {"lToeControl", "secondary"},
                {"rToeControl", "secondary"},
                {"lFootControl", "main"},
                {"rFootControl", "main"},
                 {"lKneeControl", "main"},
                {"rKneeControl", "main"},
            };
             controlsHandmade = new Dictionary<string, float>(){
                // {"headControl", "main"},
                {"chestControl", 0f},
                {"lHandControl", 0f},
                {"rHandControl", 0f},
                {"lElbowControl", 0f},
                {"rElbowControl", 0f},
                {"lShoulderControl", 0f},
                {"rShoulderControl", 0f},
                {"lArmControl", 0f},
                {"rArmControl", 0f},
                {"hipControl", 0f},
                {"lToeControl", 0f},
                {"rToeControl", 0f},
                {"lFootControl", 0f},
                {"rFootControl", 0f},
                 {"lKneeControl", 0f},
                {"rKneeControl", 0f},
            };

             controlsSway = new Dictionary<string, string>(){
                {"chestControl", "main"},
                {"hipControl", "main"},
                {"pelvisControl", "secondary"},
                {"lShoulderControl", "secondary"},
                {"rShoulderControl", "secondary"},
                {"lToeControl", "secondary"},
                {"rToeControl", "secondary"},
            };

            controlsJitters = new Dictionary<string, string>(){
                // {"lElbowControl", "main"},
                // {"rElbowControl", "main"},
                {"lHandControl", "main"},
                {"rHandControl", "main"},
                // {"lKneeControl", "main"},
                // {"rKneeControl", "main"},
                {"lFootControl", "main"},
                {"rFootControl", "main"},
                // {"lShoulderControl", "secondary"},
                // {"rShoulderControl", "secondary"},
                {"lArmControl", "secondary"},
                {"rArmControl", "secondary"},
                {"lThighControl", "secondary"},
                {"rThighControl", "secondary"},
                {"lToeControl", "secondary"},
                {"rToeControl", "secondary"},
            };



            controlsBones = new Dictionary<string, string>() {
                { "hipControl", "hip" },
                { "headControl", "head" },
                { "chestControl", "chest" },
                { "lHandControl", "lHand" },
                { "rHandControl", "rHand" },
                { "lFootControl", "lFoot" },
                { "rFootControl", "rFoot" },
                { "lKneeControl", "lShin" },
                { "rKneeControl", "rShin" },
                { "neckControl", "neck" },
                { "lElbowControl", "lForeArm" },
                { "rElbowControl", "rForeArm" },
                { "lArmControl", "lShldr" },
                { "rArmControl", "rShldr" },
                { "lShoulderControl", "lCollar" },
                { "rShoulderControl", "rCollar" },
                { "abdomenControl", "abdomen" },
                { "abdomen2Control", "abdomen2" },
                { "pelvisControl", "pelvis" },
                { "lThighControl", "lThigh" },
                { "rThighControl", "rThigh" },
                { "lToeControl", "lToe" },
                { "rToeControl", "rToe" },
            };

            foreach (var item in controlsTypes)
            {
                controlsHandmadeReset[item.Key] = false;
                controlsCollision[item.Key] = 0;
                controls[item.Key] = containingAtom.GetStorableByID(item.Key) as FreeControllerV3;
                bones[item.Key] = containingAtom.rigidbodies.First(rb => rb.name == controlsBones[item.Key]);
                var prev = bones[item.Key].gameObject.GetComponent<TriggerCollide>();
                if (prev == null) prev = bones[item.Key].gameObject.AddComponent<TriggerCollide>();
                prev.OnCollide += ObserveCollide;
            }
            // rigidbody.gameObject.AddComponent<TriggerCollide>().OnCollide += ObserveSpankTrigger;


        }

        void ObserveCollide(object sender, TriggerEventArgs e)
        {
            string s = sender.ToString().Replace(" (octopussy.TriggerCollide)", "");
          //  SuperController.LogMessage(s + " - " + e.evtType.ToString() + e.evtType.ToString());
        }


        bool ActionChanceDoLimit(float c, string s, int l)
        {
            if (!Chance(c)) return false;
            if (ActionGetCount(s) >= l) return false;
            ActionDo(s);
            return true;
        }

        bool ActionChanceDoOnce(float c, string s)
        {
            if (!Chance(c)) return false;
            if (ActionIsHappening(s)) return false;
            ActionDo(s);
            return true;
        }


        bool Chance(float s)
        {
            return Random.Range(0f, s) <= 1f;
        }

        void ActionDo(string s)
        {
            if (!actionsHappening.ContainsKey(s))
            {
                actionsHappening[s] = 0;
            }
            actionsHappening[s]++;
        }
        int ActionGetCount(string s)
        {
            if (!actionsHappening.ContainsKey(s))
            {
                return 0;
            }
            return actionsHappening[s];
        }

        bool ActionIsHappening(string s)
        {
            if (!actionsHappening.ContainsKey(s))
            {
                return false;
            }
            return actionsHappening[s] > 0;
        }

        void ActionDone(string s)
        {
            if (!actionsHappening.ContainsKey(s))
            {
                return;
            }
            actionsHappening[s]--;
        }
        void ActionDoneAll(string s)
        {
            if (!actionsHappening.ContainsKey(s))
            {
                return;
            }
            actionsHappening[s] = 0;
        }

        void ActionMovementTired(string actionType, string actionElement, string actionName)
        {
            Tired tiredPosition = new Tired(this, actionType, actionElement, actionName);
           StartCoroutine(CoroutineTiredPosition(tiredPosition));

        }


        void ActionMovementJiggle(string actionType, string actionElement, string actionName)
        {
            Jiggle jigglePosition = new Jiggle(this, actionType, actionElement, actionName, false);
            Jiggle jiggleRotation = new Jiggle(this, actionType, actionElement, actionName, true);

           StartCoroutine(CoroutineJigglePosition(jigglePosition));
            StartCoroutine(CoroutineJigglePosition(jiggleRotation));

        }

          void ActionMovementSway(string actionType, string actionElement, string actionName)
        {
            Jiggle jiggleRotation = new Jiggle(this, actionType, actionElement, actionName, true);
            StartCoroutine(CoroutineJigglePosition(jiggleRotation));

        }

        //float rotationFactor, float rotationIn, float rotationOut, float rotationHold


        IEnumerator CoroutineTiredPosition(Tired j)
        {
            FreeControllerV3 atomController = controls[j.actionElement];
            if (atomController == null){
                ActionDone(j.actionName);
                yield break;
            }


            var initialPositionState = atomController.currentPositionState;
            var initialRotationState = atomController.currentRotationState;
            if(initialPositionState == FreeControllerV3.PositionState.Off){
                initialPositionState = FreeControllerV3.PositionState.On;
                initialRotationState = FreeControllerV3.RotationState.On;
            }
            if(initialPositionState == FreeControllerV3.PositionState.Hold){
                ActionDone(j.actionName);
                yield break;
            }
             if(initialPositionState == FreeControllerV3.PositionState.ParentLink){
                 ActionDone(j.actionName);
                yield break;
            }

            float elapsedTime = 0f;
            float totalTime = j.duration;

            //SuperController.LogMessage("TIRED STARTED "+totalTime);

            atomController.currentPositionState = FreeControllerV3.PositionState.Off;
            atomController.currentRotationState = FreeControllerV3.RotationState.Off;

            while (elapsedTime <= totalTime)
            {
                if(holdingHeadStarted){
                    ActionDone(j.actionName);
                    yield break;
                }
                if(atomController.currentPositionState == FreeControllerV3.PositionState.ParentLink){
                    ActionDone(j.actionName);
                    yield break;
                }

                elapsedTime += Time.deltaTime;
                yield return null;
            }
            ActionDone(j.actionName);
             if(holdingHeadStarted)yield break;
            // SuperController.LogMessage("Tired DONE" + "holdingHeadStarted:"+holdingHeadStarted.ToString());
            atomController.currentPositionState = initialPositionState;
            atomController.currentRotationState = initialRotationState;

            yield break;


        }

        IEnumerator CoroutineJigglePosition(Jiggle j)
        {
            FreeControllerV3 atomController = controls[j.actionElement];
            if (atomController == null){
                ActionDone(j.actionName);
                yield break;
            }

            

            var initialPositionState = atomController.currentPositionState;
            if(initialPositionState == FreeControllerV3.PositionState.Hold){
                ActionDone(j.actionName);
                yield break;
            }
             if(initialPositionState == FreeControllerV3.PositionState.ParentLink){
                ActionDone(j.actionName);
                yield break;
            }



            // tval.val = Mathf.Lerp(initialVal, targetVal, elapsedTime / fadeDuration);

            float elapsedTime = 0f;
            float totalTime = j.TotalTime();
            float inTime = j.forceIn;
            float outTime = j.forceIn + j.forceHold;

            bool isRotation = j.isRotation;

            float targetX = j.x * j.forceFactor;
            float targetY = j.y * j.forceFactor;
            float targetZ = j.z * j.forceFactor;

            float prevX = 0f;
            float prevY = 0f;
            float prevZ = 0f;
            float prev2X = 0f;
            float prev2Y = 0f;
            float prev2Z = 0f;

            float deltaX = 0f;
            float deltaY = 0f;
            float deltaZ = 0f;

            float totalAdded = 0f;
            float totalSub = 0f;

            while (elapsedTime <= totalTime + Time.deltaTime)
            {
                if(holdingHeadStarted){
                    ActionDone(j.actionName);
                    yield break;
                }
                var totalOffsetX = 0f;
                var totalOffsetY = 0f;
                var totalOffsetZ = 0f;
                bool isHold = false;

                if (elapsedTime - Time.deltaTime <= inTime)
                {
                    var t = elapsedTime / inTime;
                    t = t > 1 ? 1 : t;
                    totalOffsetX = Mathf.Lerp(0, targetX, t);
                    totalOffsetY = Mathf.Lerp(0, targetY, t);
                    totalOffsetZ = Mathf.Lerp(0, targetZ, t);

                    deltaX = totalOffsetX - prevX;
                    deltaY = totalOffsetY - prevY;
                    deltaZ = totalOffsetZ - prevZ;

                    totalAdded += deltaX;
                    //SuperController.LogMessage("IN (" + deltaX + ") " + totalOffsetX + " of " + targetX + " at "+ t+" , total:"+totalAdded) ;
                    prevX = totalOffsetX;
                    prevY = totalOffsetY;
                    prevZ = totalOffsetZ;


                }
                else if (elapsedTime >= outTime)
                {
                    var t = (elapsedTime - outTime) / (totalTime - outTime);
                    t = t > 1 ? 1 : t;
                    totalOffsetX = Mathf.Lerp(0, targetX, t);
                    totalOffsetY = Mathf.Lerp(0, targetY, t);
                    totalOffsetZ = Mathf.Lerp(0, targetZ, t);
                    deltaX = -totalOffsetX + prev2X;
                    deltaY = -totalOffsetY + prev2Y;
                    deltaZ = -totalOffsetZ + prev2Z;

                    totalSub += deltaX;
                    //SuperController.LogMessage("OUT (" + deltaX + ") " + totalOffsetX + " of " + targetX + " at "+ t+" , total:"+totalSub) ;

                    prev2X = totalOffsetX;
                    prev2Y = totalOffsetY;
                    prev2Z = totalOffsetZ;

                }
                else
                {
                    isHold = true;

                }

                if (!isHold)
                {
                    if (isRotation) {
                        atomController.transform.Rotate(new Vector3(deltaX,deltaY,deltaZ),Space.Self);
                    } else {
                        Vector3 atomDestination = new Vector3(deltaX, deltaY, deltaZ);
                        atomController.transform.Translate(atomDestination);

                    }
                }

                elapsedTime += Time.deltaTime;
                yield return null;
            }
            ActionDone(j.actionName);
            // SuperController.LogMessage("DONE");

            yield break;


        }

        public void MoveResetControl(){
            FreeControllerV3 personController = containingAtom.GetStorableByID("control") as FreeControllerV3;
            Vector3 newControllerPos = MoveGetPosition();
            Vector3 newControllerRot = MoveGetRotation();

            JSONStorable t = containingAtom.GetStorableByID("control");
            JSONStorableBool tval = t.GetBoolJSONParam("detachControl");
            tval.val = true;
            personController.transform.position = newControllerPos;
           // personController.transform.rotation = Quaternion.Euler(newControllerRot);
            tval.val = false;
        }

        public Vector3 MoveGetPosition(){
            FreeControllerV3 personController = containingAtom.GetStorableByID("control") as FreeControllerV3;

            var headController = containingAtom.GetStorableByID("headControl") as FreeControllerV3;
            var neckController = containingAtom.GetStorableByID("neckControl") as FreeControllerV3;
            var chestController = containingAtom.GetStorableByID("chestControl") as FreeControllerV3;
            var pelvisController = containingAtom.GetStorableByID("pelvisControl") as FreeControllerV3;
            var hipController = containingAtom.GetStorableByID("hipControl") as FreeControllerV3;

            var x = (headController.transform.position.x + neckController.transform.position.x + chestController.transform.position.x + pelvisController.transform.position.x + hipController.transform.position.x)/5;
            var z = (headController.transform.position.z + neckController.transform.position.z + chestController.transform.position.z + pelvisController.transform.position.z + hipController.transform.position.z)/5;

            Vector3 position = new Vector3(x, 0f, z);
            return position;
        }

        public Vector3 MoveGetRotation(){
            FreeControllerV3 personController = containingAtom.GetStorableByID("control") as FreeControllerV3;

            // var headController = containingAtom.GetStorableByID("headControl") as FreeControllerV3;
            // var neckController = containingAtom.GetStorableByID("neckControl") as FreeControllerV3;
            var chestController = containingAtom.GetStorableByID("chestControl") as FreeControllerV3;
            var pelvisController = containingAtom.GetStorableByID("pelvisControl") as FreeControllerV3;
            var hipController = containingAtom.GetStorableByID("hipControl") as FreeControllerV3;

            return chestController.transform.rotation.eulerAngles;

            var x = ( chestController.transform.rotation.eulerAngles.x + pelvisController.transform.rotation.eulerAngles.x + hipController.transform.rotation.eulerAngles.x)/3;
            var y = ( chestController.transform.rotation.eulerAngles.y + pelvisController.transform.rotation.eulerAngles.y + hipController.transform.rotation.eulerAngles.y)/3;
            var z = ( chestController.transform.rotation.eulerAngles.z + pelvisController.transform.rotation.eulerAngles.z + hipController.transform.rotation.eulerAngles.z)/3;

            Vector3 rotation = new Vector3(0f, y, z);
            return rotation;
        }

        protected void tiredHoldJSONUpdate(JSONStorableFloat jf) {
		
		}








        void ClickMoveUpdate()
        {
            try
            {
                myEventTrigger.Update();

                if (Input.GetKey(KeyCode.LeftControl)){
                    if(SuperController.singleton.GetSelectedAtom() != containingAtom) return;

                    Plane plane = new Plane(Vector3.up, 0);
                    if (Input.GetMouseButtonDown(1))
                    {
                        clickdown = true;
                        simHold = true;
                        holdCooldown = holdCooldownDefault;
                        // SuperController.LogMessage("DOWN");
                        float distance;
                        RaycastHit raycastHit;
                        Vector3 worldPosition;
                        Ray ray = Camera.main.ScreenPointToRay(Input.mousePosition);
                        if (plane.Raycast(ray, out distance))
                        {
                            worldPosition = ray.GetPoint(distance);
                            //SuperController.LogMessage("p"+ worldPosition.ToString());
                            if (Physics.Raycast(ray, out raycastHit, 100f))
                            {
                                if (raycastHit.transform != null)
                                {
                                    CurrentClickedGameObject(worldPosition, ray, raycastHit.transform.gameObject);
                                    MoveTowardsDestination();

                                }
                            }
                        }
                        
                
                    }

                    else if (Input.GetMouseButton(1))
                    {
                        if(holdCooldown > 0){
                            holdCooldown--;
                            return;
                        }
                        MoveTowardsDestinationFinish();
                        //SuperController.LogMessage("HOLD");
                        if(walkLineDrawer.IsVisible()){
                            Ray ray2 = Camera.main.ScreenPointToRay(Input.mousePosition);
                            float distance2;
                            Vector3 worldPosition2;
                            if (plane.Raycast(ray2, out distance2)){
                                worldPosition2 = ray2.GetPoint(distance2);


                                Vector3 from2 = new Vector3(worldPosition2.x, 0f, worldPosition2.z);
                                Vector3 to2 = new Vector3(worldPosition2.x, 0.5f, worldPosition2.z);
                                walkLineDirectionDrawer.DrawLineInGameView(from2, to2, Color.blue);

                                RotateTowardsDestination();
                            }
                        }

                    }
                    else if (Input.GetMouseButtonUp(1))
                    {
                       
                    }
                } else {
                    if(clickdown){
                        clickdown = false;
                        simHold = false;
                        //SuperController.LogMessage("UP");
                        //SuperController.singleton.SelectController(containingAtom.mainController);
                        GuideFloorMoveHide();
                    }

                }
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        public void CurrentClickedGameObject(Vector3 click, Ray ray, GameObject gameObject)
        {
          
            foreach (var component in gameObject.GetComponentsInParent<Component>())
            {
                var atom = component as Atom;
                if (atom != null)
                {
                   // SuperController.LogMessage("name:"+atom.name);
                    if(atom.name == "Slate" || atom.name == "Floor" || atom.name == "PlayerNavigationPanel"){
                       GuideFloorMoveShow(atom, gameObject, click);
                       break;
                    } 
                    GuideFloorMoveHide();
                }
            }
           
        }

        public void GuideFloorMoveShow(Atom atom, GameObject gameObject, Vector3 click){
            Vector3 from = new Vector3(click.x, 0f, click.z);
            Vector3 to = new Vector3(click.x, 2f, click.z);
            walkLineDrawer.DrawLineInGameView(from, to, Color.green);
            
            //copy joints 
            //set joints to controll all
            //rotate person towards

        }
        public void GuideFloorMoveHide(){
            walkLineDrawer.Destroy(); 
            walkLineDirectionDrawer.Destroy(); 

        }

        
        public void MoveTowardsDestination(){
            if(!walkLineDrawer.IsVisible()) return;
            var pos = walkLineDrawer.lineStart;
           // SuperController.LogMessage(pos.ToString());

  

            // containingAtom.SetFreezePhysics(true);
          //  containingAtom.SetOn(false);
            
            // containingAtom.freezePhysicsJSON.val = true;

            // var initPos = containingAtom.mainController.currentPositionState;
            // var initRot = containingAtom.mainController.currentRotationState;

            // containingAtom.mainController.currentPositionState = FreeControllerV3.PositionState.Hold;
            // containingAtom.mainController.currentRotationState = FreeControllerV3.RotationState.Hold;
            
            // containingAtom.transform.position =  pos;

            //SuperController.LogMessage("p: " + containingAtom.transform.position.ToString());
           // SuperController.LogMessage("m: " + containingAtom.mainController.transform.position.ToString());

            // containingAtom.mainController.transform.position =  pos;
            // containingAtom.mainController.transform.position =  new Vector3(pos.x, pos.y+0.1f, pos.z);
            containingAtom.mainController.transform.position =  new Vector3(pos.x, pos.y, pos.z);
         //   containingAtom.SetOn(true);
            //containingAtom.ResetPhysics(false);

            //SuperController.LogMessage("p: " + containingAtom.transform.position.ToString());
           // SuperController.LogMessage("m: " + containingAtom.mainController.transform.position.ToString());

            // containingAtom.mainController.currentPositionState  = initPos;
            // containingAtom.mainController.currentRotationState  = initRot;
            
            

           // SPQRUtils.Invoke(this,() => MoveTowardsDestinationFinish(pos), 0.5f);
           
            Invoke("MoveTowardsDestinationFinish", 1f);
           

        }

        void MoveTowardsDestinationFinish(){
         //   containingAtom.SetOn(true);
            // containingAtom.SetFreezePhysics(false);
            // containingAtom.mainController.transform.position =  pos;
            // containingAtom.freezePhysicsJSON.val = false;
            // containingAtom.SetFreezePhysics(false);
            //  containingAtom.SetOn(true);
        }

        

        public void RotateTowardsDestination(){
            if(!walkLineDrawer.IsVisible()) return;

            float delta = Vector3.Distance(walkLineDrawer.lineStart, walkLineDirectionDrawer.lineStart);
            if(delta < 0.25f) return;
            // SuperController.LogMessage("delta:"+delta);

            Vector3 personPosition = MoveGetPosition();
            Vector2 a = new Vector2(personPosition.x, personPosition.z);
            Vector2 b = new Vector2(walkLineDirectionDrawer.lineStart.x, walkLineDirectionDrawer.lineStart.z);
            Vector2 diference = a - b;
            float sign = (b.y < a.y) ? -1.0f : 1.0f;
            float angle = -90 + Vector2.Angle(Vector2.right, diference) * sign;

           // SuperController.LogMessage("angle:"+angle);

            Vector3 currentRotation = containingAtom.mainController.transform.rotation.ToEulerAngles();
            currentRotation.y = angle;
            containingAtom.mainController.transform.rotation = Quaternion.Euler(currentRotation);
            // SuperController.LogMessage("angles:"+currentRotation.ToString());

        }

      

    }


    class Jiggle
    {

        AEDoll myScript;

        public string actionType;
        public string actionElement;
        public string actionName;

        public float x;
        public float y;
        public float z;

        public bool isRotation;

        public float forceFactor;
        public float forceIn;
        public float forceOut;
        public float forceHold;

        public Jiggle(AEDoll script, string actionType, string actionElement, string actionName, bool isRotation)
        {
            this.myScript = script;
            this.actionType = actionType;
            this.actionElement = actionElement;
            this.actionName = actionName;
            this.isRotation = isRotation;
            this.Randomize();
        }

        void Randomize()
        {


            // SuperController.LogMessage("xyz:"+x.ToString());

            if(this.actionType == "Jitters") RandomizeJitters();
            if(this.actionType == "JiggleSoft") RandomizeJiggleSoft();
            if(this.actionType == "JiggleMedium") RandomizeJiggleMedium();
            if(this.actionType == "Sway") RandomizeSway();

           
        }

        void RandomizeSway(){
            this.x = Random.Range(-1f, 1f);
            this.y = Random.Range(-1f, 1f);
            this.z = Random.Range(-1f, 1f);
            this.forceFactor = Random.Range(5f, 25f);
            this.forceIn = Random.Range(0.5f, 2f);
            this.forceOut = Random.Range(0.5f, 2f);
            this.forceHold = Random.Range(0.00001f, 2f);
        }

         void RandomizeJitters(){
            this.x = Random.Range(-1f, 1f);
            this.y = Random.Range(-1f, 1f);
            this.z = Random.Range(-1f, 1f);
            if(this.isRotation){
                this.forceFactor = Random.Range(0.1f, 0.5f);
                this.forceIn = Random.Range(0f, 0.2f);
                this.forceOut = Random.Range(0f, 0.2f);
                this.forceHold = 0.001f;
            } else {
                this.forceFactor = Random.Range(0.0005f, 0.0025f);
                this.forceIn = Random.Range(0f, 0.2f);
                this.forceOut = Random.Range(0f, 0.2f);
                this.forceHold = 0.001f;
            }
        }
        void RandomizeJiggleSoft(){
            this.x = Random.Range(-1f, 1f);
            this.y = Random.Range(-1f, 1f);
            this.z = Random.Range(-1f, 1f);
            if(this.isRotation){
                this.forceFactor = 0.5f;
                this.forceIn = 0.5f;
                this.forceOut = 0.5f;
                this.forceHold = 1f;
            } else {
                this.forceFactor = 0.002f;
                this.forceIn = 0.2f;
                this.forceOut = 0.6f;
                this.forceHold = 0.001f;
            }
        }

        void RandomizeJiggleMedium(){
            this.x = Random.Range(-1f, 1f);
            this.y = Random.Range(-1f, 1f);
            this.z = Random.Range(-1f, 1f);
            if(this.isRotation){
                this.forceFactor = Random.Range(1f, 3f);
                this.forceIn = Random.Range(0.5f, 3f);
                this.forceOut = Random.Range(0.5f, 3f);
                this.forceHold = Random.Range(0.5f, 2f);
            } else {
                this.forceFactor = Random.Range(0.005f, 0.05f);
                this.forceIn = Random.Range(1f, 4f);
                this.forceOut = Random.Range(1f, 4f);
                this.forceHold = Random.Range(0.5f, 2f);
            }
        }

        public float TotalTime()
        {
            return this.forceIn + this.forceHold + this.forceOut;
        }

      
    }


    class Tired
    {

        AEDoll myScript;

        public string actionType;
        public string actionElement;
        public string actionName;

        public float duration;

        public Tired(AEDoll script, string actionType, string actionElement, string actionName)
        {
            this.myScript = script;
            this.actionType = actionType;
            this.actionElement = actionElement;
            this.actionName = actionName;
            this.Randomize();
        }

        void Randomize()
        {
            this.duration = Random.Range(0.5f, 1f);
        }

    }

}