﻿using UnityEngine;
using System;  
using System.Collections.Generic;  
using System.Linq;  
using SimpleJSON;
using System.Net;
using System.Text; 
using SimpleJSON; 
using MacGruber;


namespace SPQR
{
    public class SPQR_APICheck : MVRScript
    {
		private JSONStorableBool myAcceptTrigger;
		private JSONStorableBool stopOnTrigger;
		private JSONStorableFloat myCount;
		private JSONStorableFloat myMinTime;
		private JSONStorableFloat myMaxTime;
		private JSONStorableString jstring;
		private EventTrigger myEventTrigger;
		private JSONStorableString debugText;
		private float myClock = -1.0f;
		private int myIndex = 0;

        public override void Init()
		{
			Utils.SetupInfoText(this,
				"<color=#606060><size=40><b>SPQR_APICheck</b></size>\nLogicBrick that checks a URL for a json response. If it has ok:1, the trigger gets called otherwise it does not. You can set for the request to repeat n times and with a delay that can be random.</color>",
				250.0f, true
			);
			Utils.OnInitUI(CreateUIElement);
			
			myAcceptTrigger = Utils.SetupToggle(this, "AcceptTrigger", true, false);
			stopOnTrigger = Utils.SetupToggle(this, "stopOnTrigger", true, false);
		
			jstring = new JSONStorableString("URL", "", UpdateJstring);
			RegisterString(jstring);
			UIDynamicLabelInput myStringInput = Utils.SetupTextInput(this, "URL", jstring, false);
			
			myCount = Utils.SetupSliderInt(this, "Ping Count", 3, 1, 999999999, false);
			

			myMinTime = Utils.SetupSliderFloatWithRange(this, "Min Ping Delay (sec)", 1, 0,3600, false);
			myMinTime.constrained = false;
			myMinTime.setCallbackFunction += (float v) => {
				Utils.AdjustMaxSliderFromMin(v, myMaxTime);
			};

			myMaxTime = Utils.SetupSliderFloatWithRange(this, "Max Ping Delay (sec)", 1, 0, 3600, false);
			myMaxTime.constrained = false;
			myMaxTime.setCallbackFunction += (float v) => {
				myMinTime.valNoCallback = Mathf.Min(v, myMinTime.val);
			};

			myEventTrigger = new EventTrigger(this, "OnTrigger");
			Utils.SetupButton(this, "Actions OnTrigger", myEventTrigger.OpenPanel, false);

			Utils.SetupButton(this, "Test Trigger", Trigger, false);
			Utils.SetupAction(this, "Trigger", Trigger);
			
			
			debugText = new JSONStorableString("Info", "");
            UIDynamicTextField dtext3 = CreateTextField(debugText,true);
            dtext3.height = 450f;
		

			SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;
			SimpleTriggerHandler.LoadAssets();
		}

		private void OnAtomRename(string oldid, string newid)
		{
			myEventTrigger.SyncAtomNames();
		}

		public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
		{
			JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
			if (includePhysical || forceStore)
			{
				needsStore = true;
				jc[myEventTrigger.Name] = myEventTrigger.GetJSON(base.subScenePrefix);
			}
			return jc;
		}

		public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
		{
			base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
			Utils.AdjustSliderRange(myMinTime);
			Utils.AdjustSliderRange(myMaxTime);
			myClock = -1.0f;

			if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
			{
				myEventTrigger.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
			}
		}
		
		private void UpdateJstring(string s){
			
		}
		
		private void OnDisable()
		{
			myClock = -1.0f;
		}

		private void OnDestroy()
		{
			SuperController.singleton.onAtomUIDRenameHandlers -= OnAtomRename;
			myEventTrigger.Remove();
		}

		private void Update()
		{
			myEventTrigger.Update();
			
			if (myClock < 0.0f || myIndex >= myCount.val){
				myClock = -1;
				return;
			}
			myClock -= Time.deltaTime;
			if (myClock <= 0.0f)
			{
				myClock = UnityEngine.Random.Range(myMinTime.val, myMaxTime.val);
				++myIndex;
				if (debugText.val.Length > 500) debugText.val = "";
				debugText.val+= "Making URL call #:"+myIndex+"\r\n";
				if(CheckApi()){
					debugText.val+= "API OK response received!"+"\r\n";
					myEventTrigger.Trigger();
					if(stopOnTrigger.val){
						myClock = -1;
					}
				} else {
				}
			}
		}
		
		private bool CheckApi(){
			var url = jstring.val;
			
			WebClient client = new WebClient();
			string downloadString = client.DownloadString(url);
			debugText.val+= "response:"+downloadString+"\r\n";
			if(downloadString == null) return false;
			if(downloadString.Length < 8) return false;
			JSONNode js = new JSONNode();
			try{
				js = JSON.Parse(downloadString);
			} catch(Exception e){
				return false;
			}
			if(js == null) return false;
			if(js["ok"] == null) return false;
			if(js["ok"].Value == "1") return true;
			return false;
		}
		
		private void Trigger()
		{
			debugText.val = "";
			if (myAcceptTrigger.val){
				myClock = UnityEngine.Random.Range(myMinTime.val, myMaxTime.val);
				myIndex = 0;
			}
		}
    }
}
