﻿using System;
using UnityEngine;
using System.Text;

using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using MacGruber;


namespace SPQR
{
    public class SPQRDynamicConditions : MVRScript {

        private EventTrigger myEventTrigger;

        protected JSONStorableString varValue;
        protected JSONStorableString varNotes;

        
        JSONStorableStringChooser jchooser;
        int titles = 0;
    
        public override void Init()
        {
            if(containingAtom.type != "UIText") return;
            Utils.OnInitUI(CreateUIElement);


            CreateTitle("DYNAMIC Conditions",45,false);
            CreateTitle("AETERNUM TOOLS",30,true);
            
            


            JSONStorableString jstring = new JSONStorableString("instructions2", "\n This plugin must be added to a text atom. It allows setting up a callback and a condition. When the callback is triggerd, it will run only if the condition is true. T = Text value, V = the variable set in the condition.");
            UIDynamicTextField dtext = CreateTextField(jstring, true);
            dtext.UItext.supportRichText = true;
            dtext.backgroundColor = new Color(0.42f, 0.03f, 0.51f, 0.7f);
            dtext.textColor = new Color(1f, 1f, 1f, 0.9f);
            dtext.height = 350f;

            JSONStorableString jstring2 = new JSONStorableString("instructions1", "\n 1. set a value in the Variable field and pick a Condition \n 2. set the callbacks that you want to happen when the condition is true. \n\n  You can now call the condition trigger with callbacks to the Atom > plugin#SPQRDynamicConditions > TriggerCondition.");
            UIDynamicTextField dtext2 = CreateTextField(jstring2);
            dtext2.UItext.supportRichText = true;
            dtext2.backgroundColor = new Color(0.42f, 0.03f, 0.51f, 0.7f);
            dtext2.textColor = new Color(1f, 1f, 1f, 0.9f);
            dtext2.height = 350f;
            

            Utils.SetupSpacer(this, 70, false);
            Utils.SetupSpacer(this, 65, true);


            myEventTrigger = new EventTrigger(this, "TriggerCondition");
            Utils.SetupButton(this, "Trigger", myEventTrigger.OpenPanel, false);

            Utils.SetupButton(this, "Test Trigger", TestTrigger, true);
            Utils.SetupAction(this, "TriggerCondition", Trigger);
            SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;
            SimpleTriggerHandler.LoadAssets();

           



            varNotes = new JSONStorableString("Notes", "", UpdateVarNotes);
            RegisterString(varNotes);
            UIDynamicLabelInput myStringInput2 = Utils.SetupTextInput(this, "Notes", varNotes, true);
    

            varValue = new JSONStorableString("Value", "", UpdateVarValue);
            RegisterString(varValue);
            UIDynamicLabelInput myStringInput = Utils.SetupTextInput(this, "Variable", varValue, false);

    
            List<string> choices = new List<string>();
            choices.Add("None");
            choices.Add("T=V");
            choices.Add("T<V");
            choices.Add("T<=V");
            choices.Add("T>V");
            choices.Add("T>=V");
            jchooser = new JSONStorableStringChooser("OperationType", choices, "None", "Condition", OperationUpdate);
            RegisterStringChooser(jchooser);
            UIDynamicPopup udp = CreatePopup(jchooser, false);
            //CreateScrollablePopup(jchooser, true);
            udp.labelWidth = 300f;



            SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;
            UIUpdate();
        }
        private void OnAtomRename(string oldid, string newid)
        {
            myEventTrigger.SyncAtomNames();
        }
        private void UIUpdate()
        {
        }


        void Start()
        {
            try
            {
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        void Update()
        {
            try
            {
            //    myEventTrigger.Update();
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }
        private void TestTrigger()
        {
           Trigger();

        }
        private void Trigger()
        {

            string currentValue = GetText();
            string newValue = varValue.val;
            int currentValueInt = ToInt(currentValue);
            int newValueInt = ToInt(newValue);
            if(jchooser.val == "None") return;
            bool truth = false;
            
            

            switch(jchooser.val){
                case "T=V":
                    truth  = currentValue == newValue;
                    break;
                case "T<V":
                    truth  = currentValueInt < newValueInt;
                    break;
                case "T<=V":
                    truth  = currentValueInt <= newValueInt;
                    break;
                case "T>V":
                    truth  = currentValueInt > newValueInt;
                    break;
                case "T>=V":
                    truth  = currentValueInt >= newValueInt;
                    break;

            }
            if(currentValue == "" || newValue == "") truth = false;     
            if(truth){
                myEventTrigger.Trigger();
                // SuperController.LogMessage("triggered");
            }
            
        }

        void FixedUpdate()
        {
            try
            {
                // put code in here
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        void OnDestroy()
        {
            SuperController.singleton.onAtomUIDRenameHandlers -= OnAtomRename;
            myEventTrigger.Remove();
            Utils.OnDestroyUI();
        }



        public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
        {
            JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
            if (includePhysical || forceStore)
            {
                needsStore = true;
                jc[myEventTrigger.Name] = myEventTrigger.GetJSON(base.subScenePrefix);
            }
            return jc;
        }

        public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
        {
            base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);

            if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
            {
                myEventTrigger.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
            }
        }



        //////////////////////////////////////////////

        void UpdateVarValue(string name)
        {
            

        }
         void UpdateVarNotes(string name)
        {
            

        }
         void UpdateVarMin(string name)
        {
            

        }
         void UpdateVarMax(string name)
        {
            

        }
          void UpdateVarDefault(string name)
        {
            

        }

        
		protected void OperationUpdate(string s) {
            

		}

        public static int ToInt(string s){
            if(s=="" || s== null) s = "0";
            return Int32.Parse(s);
        }


        public string GetText()
        {
            JSONStorable f = containingAtom.GetStorableByID("Text");
            JSONStorableString fs = f.GetStringJSONParam("text");
            return fs.val;
        }

        void SetAtomText(string t){ 
            JSONStorable f = containingAtom.GetStorableByID("Text");
            JSONStorableString fs = f.GetStringJSONParam("text");
            fs.val = t;
         }


        private void CreateTitle(string text, int fontsize, bool right,int spacing){
            CreateTitle(text, fontsize, right);
            Utils.SetupSpacer(this, spacing, right);
        }
        private void CreateTitle(string text, int fontsize, bool right){
                JSONStorableString title1 = new JSONStorableString("title"+(++titles), "\n<size="+fontsize+"><b>"+text+"</b></size>");
                UIDynamicTextField title1text = CreateTextField(title1, right);
                title1text.UItext.supportRichText = true;
                title1text.backgroundColor = new Color(0f, 0f, 0f, 0f);
                title1text.textColor = new Color(0.42f, 0.03f, 0.51f, 1f);
                title1text.height = 100f;
                title1text.UItext.alignment = right ? TextAnchor.MiddleRight : TextAnchor.MiddleLeft;
        }

	}
}