﻿using System;
using System.Linq;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Net.Mail;

namespace SPQR {
	public class CyberDJ : MVRScript {

		bool isLoaded = false;
		AudioSourceControl audioSourceControl;
		int _prevLoop = -1;

		List<int> _oneOffs = new List<int> { 2 };
		public override void Init() {

			InvokeRepeating("UpdateSecond", 1f, 1f);
			InvokeRepeating("UpdateConstant", 1f, 1f / 30f);
			InvokeRepeating("UpdateConstant5", 1f, 1f / 5f);
			audioSourceControl = containingAtom.GetComponentInChildren<AudioSourceControl>();
            if (audioSourceControl == null) { 
                SuperController.LogMessage("CyberDJ: audio source not found. Plugin needs to be added on an AudioSource atom.");
				return;
			}
            isLoaded = true;
			StopMusic();

            var nac = EmbeddedAudioClipManager.singleton.GetClip("CyberStabs6");
            audioSourceControl.PlayNext(nac);
            StartCoroutine(InvokeRoutine_p67(StopMusic, 3f));
        }


	
		void Start() {
		
		}

	
		void Update() {

            if (IsDisabled()) return;
			if (!audioSourceControl.audioSource.isPlaying)
			{
				PlayNext();
			}
		}

	
		void FixedUpdate() {
            if (IsDisabled()) return;

        }
		void UpdateSecond()
		{
			if (IsDisabled()) return;

		}
		void UpdateConstant()
		{
			if (IsDisabled()) return;

		}
		void UpdateConstant5()
		{
			if (IsDisabled()) return;

		}
		void OnDestroy()
		{
			CancelInvoke();
			StopMusic();
		}

		void OnDisable()
		{
			StopMusic();

        }
	
		bool IsDisabled()
        {
			if (!enabledJSON.val)
			{
				return true;
			}
			if (SuperController.singleton.freezeAnimation) return true;

            if (audioSourceControl == null) return true;
			return !isLoaded;
		}

		void PlayNext()
		{
			var last = 17;
			var bars = UnityEngine.Random.Range(1, 5)*2;
			var nextLoop = UnityEngine.Random.Range(1, last+1);
			while (nextLoop == _prevLoop || _oneOffs.Contains(nextLoop))
			{
				nextLoop = UnityEngine.Random.Range(1, last + 1);
			}
			nextLoop = AddNeighbor(_prevLoop, nextLoop);
			if (_prevLoop == -1) nextLoop = 1;
			bars = AlterBars(nextLoop, bars);


            for (var i = 1; i<= bars; i++) {
				if(i==bars) nextLoop = AlterLastLoop(nextLoop);
                var nac = EmbeddedAudioClipManager.singleton.GetClip("CyberTakenLoop" + nextLoop);
				//SuperController.LogMessage("play " + Time.time + ": " + nextLoop + " " + bars);
                audioSourceControl.PlayNext(nac);
			}
            

            _prevLoop = nextLoop;

        }

		void StopMusic()
		{
			if (audioSourceControl == null) return;
			audioSourceControl.StopAndClearQueue();
		}

        private static IEnumerator InvokeRoutine_p67(System.Action f, float delay)
        {
            yield return new WaitForSeconds(delay);
            f();
        }

		int AddNeighbor(int prev, int loop)
		{
			var rand = UnityEngine.Random.Range(0, 101);
            if (rand < 50) return loop;


			switch (prev)
			{
                case 6:
                case 7:
                case 11:
                case 12:
					if (rand % 4 == 0) loop = 6;
                    if (rand % 4 == 1) loop = 7;
                    if (rand % 4 == 2) loop = 11;
                    if (rand % 4 == 3) loop = 12;
                    break;

                case 8:
                case 9:
                case 14:
                case 15:
                    if (rand % 4 == 0) loop = 8;
                    if (rand % 4 == 1) loop = 9;
                    if (rand % 4 == 2) loop = 14;
                    if (rand % 4 == 3) loop = 15;
                    break;
				
				default:
					prev = -1;
					break;

            }
			if (prev == loop) return AddNeighbor(prev, loop);
			return loop;
        }
        int AlterBars(int loop, int bars)
		{
			switch (loop)
			{
                case 2:
                case 10:
                case 4:
                case 5:
                case 6:
                    bars = 1;
                    break;

                case 1:
                case 13:
                    bars = UnityEngine.Random.Range(1, 2) * 2;
					break;
                case 3:


                
                case 7:
                case 11:
                case 12:

                case 8:
				case 9:
				case 14:
                case 15:

                case 16:
                    bars = UnityEngine.Random.Range(1, 2) * 4;
                    break;
            }
			return bars;
		}


        int AlterLastLoop(int loop)
		{
			switch (loop)
			{
				case 3:
					return 2;
					
			}
			return loop;

        }
    }
}