﻿using System;
using System.Linq;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace SPQR {
	public class CyberHair : MVRScript {
		
		bool sceneLoaded = false;
		string colorStorableId = "";
		public Dictionary<string, JSONStorableFloat> sliders = new Dictionary<string, JSONStorableFloat>();

		private float minSpeedRoot = 0.06f;  // Minimum rotation speed
		private float maxSpeedRoot = 0.08f; // Maximum rotation speed
		JSONStorableColor hairRootStorable;
		JSONStorableColor hairScalpStorable;
		JSONStorableColor hairScalpStorable2;
		Color hairRootColor = new Color(125f / 255f, 115f / 255f, 65f / 255f);
		float hairRootHue;
		float hairRootRotationSpeed;
		bool initiatedHairRoot = false;


		private float minSpeedTip = 0.05f;  // Minimum rotation speed
		private float maxSpeedTip = 0.1f; // Maximum rotation speed
		JSONStorableColor hairTipStorable;
		Color hairTipColor = new Color(48f / 255f, 243f / 255f, 0f / 255f);
		float hairTipHue;
		float hairTipRotationSpeed;
		bool initiatedHairTip = false;

		private float minSpeedIris = 0.1f;  // Minimum rotation speed
		private float maxSpeedIris= 0.15f; // Maximum rotation speed
		JSONStorableColor irisStorable;
		Color irisTipColor = new Color(212f / 255f, 69f / 255f, 69f / 255f);
		float irisTipHue;
		float irisTipRotationSpeed;
		bool initiatediris = false;

		Dictionary<string, JSONStorableString> titles = new Dictionary<string, JSONStorableString>();
		bool prepared = false;
		bool ok = false;

		string currentHairsKey = "";

		Dictionary<string, JSONStorableColor> hairs = new Dictionary<string, JSONStorableColor>() { };

		DAZCharacterSelector dcs;

		public override void Init() {

			if (containingAtom.type != "Person")
			{
				SuperController.LogError("CyberHair must be added to a Person atom");
				return;
			}
			ok = true;

			float t1, t2, t3, t4;
			Color.RGBToHSV(hairRootColor, out hairRootHue, out t1, out t2);
			Color.RGBToHSV(hairTipColor, out hairTipHue, out t3, out t4);
			Color.RGBToHSV(irisTipColor, out irisTipHue, out t3, out t4);


			CreateTitle_p67("titlepluginname", "CyberHair v1", 40, false);
			CreateTitle_p67("titlepluginname2", "patreon.com/\nSPQR_AETERNUM", 20, true);


			sliders.Add("Color speed", new JSONStorableFloat("Color speed", 1f, 0.01f, 10f, false));
			RegisterFloat(sliders["Color speed"]);
			CreateSlider(sliders["Color speed"], false);


			InvokeRepeating("UpdateSecond", 1f, 1f);
			InvokeRepeating("UpdateConstant", 1f, 1f / 30f);
			InvokeRepeating("UpdateConstant5", 1f, 1f / 5f);
			SuperController.singleton.onSceneLoadedHandlers += OnSceneLoaded;
		}

	
		void Start() {
			dcs = containingAtom.GetComponentInChildren<DAZCharacterSelector>();
		}

	
		void Update() {
			if (SuperController.singleton.isLoading) return;
            if (!prepared)
            {
				prepared = true;
				PrepareColorItem();
			}
			
		}

	
		void FixedUpdate() {
		
		}
		void UpdateSecond()
		{
			if (IsDisabled()) return;

            if (!IsHairsChange())
            {
				prepared = true;

			}


		}
		void UpdateConstant()
		{
			if (IsDisabled()) return;

		}
		void UpdateConstant5()
		{
			if (IsDisabled()) return;
			try
			{
				UpdateColor();
			} catch(Exception e)
            {
				SuperController.LogMessage("error: " + e);
            }

		}
		void OnDestroy()
		{
			CancelInvoke();
			SuperController.singleton.onSceneLoadedHandlers -= OnSceneLoaded;
		}

		void OnSceneLoaded()
		{
			sceneLoaded = true;
		}
		bool IsDisabled()
        {
			if (!enabledJSON.val)
			{
				return true;
			}
			if (!ok) return true;
			return false;
		}


		bool IsHairsChange()
        {
			var newKey = GetLiveKey();
			if (newKey != currentHairsKey)
            {
				PrepareColorItem();
            }
			return false;
        }


		string GetLiveKey()
        {
			if (dcs == null) return "";
			var key = "";
			var pjson = dcs.GetJSON(true, true);
			for (int i = 0; i < pjson["hair"].Count; i++)
			{
				var storable = pjson["hair"][i];
				if (!IsHair_p67(storable["internalId"])) continue;
				colorStorableId = storable["internalId"];
				key += colorStorableId + ";";
			}
			return key;
		}

		bool IsHair_p67(string s)
		{
			if (s == null) return false;
			s = s.ToLower();
			if (s.Contains("lashes")) return false;
			if (s.Contains("pubic")) return false;
			return true;
		}
		void PrepareColorItem(){
			try
			{
				hairs = new Dictionary<string, JSONStorableColor>() { };
				var key = "";
				if (dcs == null) return;
				var pjson = dcs.GetJSON(true, true);
				for (int i = 0; i < pjson["hair"].Count; i++)
				{
					var storable = pjson["hair"][i];
					if (!IsHair_p67(storable["internalId"])) continue;
					colorStorableId = storable["internalId"];
					JSONStorable t = containingAtom.GetStorableByID(colorStorableId + "Sim");
					hairTipStorable = t.GetColorJSONParam("tipColor");
					hairs[colorStorableId] = hairTipStorable;
					key += colorStorableId + ";";

				}

				currentHairsKey = key;
			}
			catch (Exception e)
			{
				SuperController.LogMessage("error: " + e);
			}
		}

		void UpdateColor()
        {

			//hairRootHue += hairRootRotationSpeed * Time.deltaTime;
			//hairRootHue = Mathf.Repeat(hairRootHue, 1f);
			//if (UnityEngine.Random.value < Time.deltaTime / 10f || !initiatedHairRoot)
			//{
			//	initiatedHairRoot = true;
			//	hairRootRotationSpeed = UnityEngine.Random.Range(minSpeedRoot, maxSpeedRoot) * (UnityEngine.Random.value > 0.5f ? 1f : -1f);
			//}
			//hairRootColor = Color.HSVToRGB(hairRootHue, 0.75f, 0.4f);
			//hairRootStorable.val = HSVColorPicker.RGBToHSV(hairRootColor.r, hairRootColor.g, hairRootColor.b);

			//var hairScalpColor = Color.HSVToRGB(hairRootHue, 0.75f, 0.4f);
			//hairScalpStorable.val = HSVColorPicker.RGBToHSV(hairScalpColor.r, hairScalpColor.g, hairScalpColor.b);
			//hairScalpStorable2.val = HSVColorPicker.RGBToHSV(hairScalpColor.r, hairScalpColor.g, hairScalpColor.b);




			float hairTipHue = -2f;
			List<string> keys = new List<string>(hairs.Keys);
			if (keys.Count() < 1) return;
			foreach (var k in keys)
			{
				var hairTipStorable = hairs[k];
				if (hairTipStorable == null) return;
				if (hairTipHue <= -2f) {
					hairTipHue = hairTipStorable.val.H + hairTipRotationSpeed * Time.deltaTime * sliders["Color speed"].val;
					hairTipHue = Mathf.Repeat(hairTipHue, 1f);
				}
				if (UnityEngine.Random.value < Time.deltaTime / 10f || !initiatedHairTip)
				{
					initiatedHairTip = true;
					hairTipRotationSpeed = UnityEngine.Random.Range(minSpeedTip, maxSpeedTip) * (UnityEngine.Random.value > 0.5f ? 1f : -1f);
				}
				hairTipColor = Color.HSVToRGB(hairTipHue, 1f, 0.8f);
				hairTipStorable.val = HSVColorPicker.RGBToHSV(hairTipColor.r, hairTipColor.g, hairTipColor.b);
			}




		}


		public void CreateTitle_p67(string id, string text, int fontsize, bool right, int spacing)
		{
			CreateTitle_p67(id, text, fontsize, right);
			CreateSpacer_p67(spacing, right);
		}

		public void CreateTitle_p67(string id, string text, int fontsize, bool right)
		{
			if (!titles.ContainsKey(id))
				titles.Add(id, new JSONStorableString("title " + id, "\n<size=" + fontsize + "><b>" + text + "</b></size>"));
			UIDynamicTextField title1text = CreateTextField(titles[id], right);
			title1text.UItext.supportRichText = true;
			title1text.backgroundColor = new Color(0f, 0f, 0f, 0f);
			title1text.textColor = new Color(0.42f, 0.03f, 0.51f, 1f);
			title1text.height = 100f;
			title1text.UItext.alignment = right ? TextAnchor.MiddleRight : TextAnchor.MiddleLeft;
		}


		public UIDynamic CreateSpacer_p67(float height, bool rightSide)
		{
			UIDynamic spacer = CreateSpacer(rightSide);
			spacer.height = height;
			return spacer;
		}


	}
}