﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace SPQR
{


    public class CycleForceExtra : MVRScript
    {

        protected Rigidbody RB;
        protected void SyncReceiver(string receiver)
        {
            if (receiver != null)
            {
                ForceReceiver fr;
                if (receiverNameToForceReceiver.TryGetValue(receiver, out fr))
                {
                    RB = fr.GetComponent<Rigidbody>();
                }
                else
                {
                    RB = null;
                }
            }
            else
            {
                RB = null;
            }
        }
        protected JSONStorableStringChooser receiverChoiceJSON;

        protected string _forceAxis = "Z";
        protected void SyncForceAxis(string axisName)
        {
            try
            {
                string an = axisName;
                _forceAxis = an;
            }
            catch (System.ArgumentException)
            {
                Debug.LogError("Attempt to set axis to " + axisName + " which is not a valid axis name");
            }
        }
        protected JSONStorableStringChooser forceAxisJSON;


        protected string _torqueAxis = "X";
        protected void SyncTorqueAxis(string axisName)
        {
            try
            {
                string an = axisName;
                _torqueAxis = an;
            }
            catch (System.ArgumentException)
            {
                Debug.LogError("Attempt to set axis to " + axisName + " which is not a valid axis name");
            }
        }
        protected JSONStorableStringChooser torqueAxisJSON;


        protected JSONStorableFloat intervalJSON, intervalMinJSON, intervalMaxJSON;
        protected JSONStorableFloat periodJSON, periodMinJSON, periodMaxJSON;
        protected JSONStorableFloat periodRatioJSON, periodRatioMinJSON, periodRatioMaxJSON;
        protected JSONStorableFloat forceDurationJSON, forceDurationMinJSON, forceDurationMaxJSON;
        protected JSONStorableFloat forceFactorJSON, forceFactorMinJSON, forceFactorMaxJSON;
        protected JSONStorableFloat forceQuicknessJSON, forceQuicknessMinJSON, forceQuicknessMaxJSON;
        protected JSONStorableFloat torqueFactorJSON, torqueFactorMinJSON, torqueFactorMaxJSON;
        protected JSONStorableFloat torqueQuicknessJSON, torqueQuicknessMinJSON, torqueQuicknessMaxJSON;
        protected JSONStorableBool applyForceOnReturnJSON;

        protected List<string> receiverChoices;
        protected Dictionary<string, ForceReceiver> receiverNameToForceReceiver;

        string currentRandPos = "X";
        string currentRandRot = "X";

        List<UIDynamicSlider> disabledSliders = new List<UIDynamicSlider>();

        public override void Init()
        {
            try
            {
                receiverChoices = new List<string>();
                receiverNameToForceReceiver = new Dictionary<string, ForceReceiver>();
                foreach (ForceReceiver fr in containingAtom.forceReceivers)
                {
                    receiverChoices.Add(fr.name);
                    receiverNameToForceReceiver.Add(fr.name, fr);
                }
                receiverChoiceJSON = new JSONStorableStringChooser("receiver", receiverChoices, null, "Receiver", SyncReceiver);
                receiverChoiceJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterStringChooser(receiverChoiceJSON);
                UIDynamicPopup dp = CreateScrollablePopup(receiverChoiceJSON);
                dp.popupPanelHeight = 1100f;
                dp.popup.alwaysOpen = false;

                string[] axisChoices = new string[] { "X", "Y", "Z", "XY", "XZ", "YZ", "NegX", "NegY", "NegZ", "NegXNegY", "NegXNegZ", "NegYNegZ", "RandXYZ", "RandXY", "RandXZ", "RandYZ", "RandNegXYZ", "RandNegXY", "RandNegXZ", "RandNegYZ" };
                List<string> axisChoicesList = new List<string>(axisChoices);

                forceAxisJSON = new JSONStorableStringChooser("forceAxis", axisChoicesList, _forceAxis.ToString(), "Force Axis", SyncForceAxis);
                forceAxisJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterStringChooser(forceAxisJSON);
                CreatePopup(forceAxisJSON, true);

                torqueAxisJSON = new JSONStorableStringChooser("torqueAxis", axisChoicesList, _torqueAxis.ToString(), "Torque Axis", SyncTorqueAxis);
                torqueAxisJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterStringChooser(torqueAxisJSON);
                CreatePopup(torqueAxisJSON, true);

                CreateSpacer_p67(60f, false);
                intervalJSON = new JSONStorableFloat("interval", 1f, 0.1f, 60f, false, false);
                intervalJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterFloat(intervalJSON);
                disabledSliders.Add(CreateSlider(intervalJSON));

                intervalMinJSON = new JSONStorableFloat("intervalMin", 1f, RefreshMins, 0.1f, 60f, false);
                intervalMinJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterFloat(intervalMinJSON);
                CreateSlider(intervalMinJSON, true);

                intervalMaxJSON = new JSONStorableFloat("intervalMax", 3f, RefreshMins, 0.1f, 60f, false);
                intervalMaxJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterFloat(intervalMaxJSON);
                CreateSlider(intervalMaxJSON, true);

                CreateSpacer_p67(120f, false);
                periodJSON = new JSONStorableFloat("period", 0.5f, 0f, 10f, false, false);
                periodJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterFloat(periodJSON);
                disabledSliders.Add(CreateSlider(periodJSON));

                periodMinJSON = new JSONStorableFloat("periodMin", 0.5f, RefreshMins, 0f, 10f, false);
                periodMinJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterFloat(periodMinJSON);
                CreateSlider(periodMinJSON, true);

                periodMaxJSON = new JSONStorableFloat("periodMax", 0.5f, RefreshMins, 0f, 10f, false);
                periodMaxJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterFloat(periodMaxJSON);
                CreateSlider(periodMaxJSON, true);

                CreateSpacer_p67(120f, false);
                periodRatioJSON = new JSONStorableFloat("periodRatio", 0.5f, 0f, 1f, true, false);
                periodRatioJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterFloat(periodRatioJSON);
                disabledSliders.Add(CreateSlider(periodRatioJSON));

                periodRatioMinJSON = new JSONStorableFloat("periodRatioMin", 0.5f, RefreshMins, 0f, 1f, true);
                periodRatioMinJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterFloat(periodRatioMinJSON);
                CreateSlider(periodRatioMinJSON, true);

                periodRatioMaxJSON = new JSONStorableFloat("periodRatioMax", 0.5f, RefreshMins, 0f, 1f, true);
                periodRatioMaxJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterFloat(periodRatioMaxJSON);
                CreateSlider(periodRatioMaxJSON, true);


                CreateSpacer_p67(120f, false);
                forceDurationJSON = new JSONStorableFloat("forceDuration", 1f, 0f, 1f, true, false);
                forceDurationJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterFloat(forceDurationJSON);
                disabledSliders.Add(CreateSlider(forceDurationJSON));

                forceDurationMinJSON = new JSONStorableFloat("forceDurationMin", 1f, RefreshMins, 0f, 1f, true);
                forceDurationMinJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterFloat(forceDurationMinJSON);
                CreateSlider(forceDurationMinJSON, true);

                forceDurationMaxJSON = new JSONStorableFloat("forceDurationMax", 1f, RefreshMins, 0f, 1f, true);
                forceDurationMaxJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterFloat(forceDurationMaxJSON);
                CreateSlider(forceDurationMaxJSON, true);

                CreateSpacer_p67(120f, false);
                forceFactorJSON = new JSONStorableFloat("forceFactor", 0f, 0f, 1000f, false, false);
                forceFactorJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterFloat(forceFactorJSON);
                disabledSliders.Add(CreateSlider(forceFactorJSON));

                forceFactorMinJSON = new JSONStorableFloat("forceFactorMin", 20f, RefreshMins, 0f, 1000f, false, true);
                forceFactorMinJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterFloat(forceFactorMinJSON);
                CreateSlider(forceFactorMinJSON, true);

                forceFactorMaxJSON = new JSONStorableFloat("forceFactorMax", 30f, RefreshMins, 0f, 1000f, false, true);
                forceFactorMaxJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterFloat(forceFactorMaxJSON);
                CreateSlider(forceFactorMaxJSON, true);

                CreateSpacer_p67(120f, false);
                forceQuicknessJSON = new JSONStorableFloat("forceQuickness", 10f, 0f, 50f, false, false);
                forceQuicknessJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterFloat(forceQuicknessJSON);
                disabledSliders.Add(CreateSlider(forceQuicknessJSON));

                forceQuicknessMinJSON = new JSONStorableFloat("forceQuicknessMin", 10f, RefreshMins, 0f, 50f, false, true);
                forceQuicknessMinJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterFloat(forceQuicknessMinJSON);
                CreateSlider(forceQuicknessMinJSON, true);

                forceQuicknessMaxJSON = new JSONStorableFloat("forceQuicknessMax", 10f, RefreshMins, 0f, 50f, false, true);
                forceQuicknessMaxJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterFloat(forceQuicknessMaxJSON);
                CreateSlider(forceQuicknessMaxJSON, true);

                CreateSpacer_p67(120f, false);
                torqueFactorJSON = new JSONStorableFloat("torqueFactor", 5f, 0f, 100f, false, false);
                torqueFactorJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterFloat(torqueFactorJSON);
                disabledSliders.Add(CreateSlider(torqueFactorJSON));

                torqueFactorMinJSON = new JSONStorableFloat("torqueFactorMin", 5f, RefreshMins, 0f, 100f, false, true);
                torqueFactorMinJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterFloat(torqueFactorMinJSON);
                CreateSlider(torqueFactorMinJSON, true);

                torqueFactorMaxJSON = new JSONStorableFloat("torqueFactorMax", 5f, RefreshMins, 0f, 100f, false, true);
                torqueFactorMaxJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterFloat(torqueFactorMaxJSON);
                CreateSlider(torqueFactorMaxJSON, true);

                CreateSpacer_p67(120f, false);
                torqueQuicknessJSON = new JSONStorableFloat("torqueQuickness", 10f, 0f, 50f, false, false);
                torqueQuicknessJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterFloat(torqueQuicknessJSON);
                disabledSliders.Add(CreateSlider(torqueQuicknessJSON));

                torqueQuicknessMinJSON = new JSONStorableFloat("torqueQuicknessMin", 10f, RefreshMins, 0f, 50f, false, true);
                torqueQuicknessMinJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterFloat(torqueQuicknessMinJSON);
                CreateSlider(torqueQuicknessMinJSON, true);

                torqueQuicknessMaxJSON = new JSONStorableFloat("torqueQuicknessMax", 10f, RefreshMins, 0f, 50f, false, true);
                torqueQuicknessMaxJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterFloat(torqueQuicknessMaxJSON);
                CreateSlider(torqueQuicknessMaxJSON, true);

                applyForceOnReturnJSON = new JSONStorableBool("applyForceOnReturn", true);
                applyForceOnReturnJSON.storeType = JSONStorableParam.StoreType.Full;
                RegisterBool(applyForceOnReturnJSON);
                CreateToggle(applyForceOnReturnJSON, true);


            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        protected float timer;
        protected float forceTimer;
        protected float flip;
        protected Vector3 targetForce;
        protected Vector3 targetTorque;
        protected Vector3 currentForce;
        protected Vector3 currentTorque;

        float timeSinceLastInterval = 0f;

        string[] stringXYZ = new string[] { "X", "Y", "Z" };
        string[] stringXY = new string[] { "X", "Y" };
        string[] stringXZ = new string[] { "X", "Z" };
        string[] stringYZ = new string[] { "Y", "Z" };



        protected void Start()
        {
            timer = 0f;
            forceTimer = 0f;
            flip = 1f;
        }

        protected virtual Vector3 AxisToVector(string axis)
        {
            Vector3 result;
            if (RB != null)
            {
                switch (axis)
                {
                    case "X":
                        result = RB.transform.right;
                        break;
                    case "NegX":
                        result = -RB.transform.right;
                        break;
                    case "Y":
                        result = RB.transform.up;
                        break;
                    case "NegY":
                        result = -RB.transform.up;
                        break;
                    case "Z":
                        result = RB.transform.forward;
                        break;
                    case "NegZ":
                        result = -RB.transform.forward;
                        break;

                    case "XY":
                        result = RB.transform.right + RB.transform.up;
                        break;
                    case "XZ":
                        result = RB.transform.right + RB.transform.forward;
                        break;
                    case "YZ":
                        result = RB.transform.up + RB.transform.forward;
                        break;
                    case "NegXNegY":
                        result = -RB.transform.right - RB.transform.up;
                        break;
                    case "NegXNegZ":
                        result = -RB.transform.right - RB.transform.forward;
                        break;
                    case "NegYNegZ":
                        result = -RB.transform.up - RB.transform.forward;
                        break;

                    default:
                        result = Vector3.zero;
                        break;
                }
            }
            else
            {
                result = Vector3.zero;
            }
            return (result);
        }

        protected void SetTargets(float percent)
        {
            targetForce = AxisToVector(currentRandPos) * percent * forceFactorJSON.val;
            targetTorque = AxisToVector(currentRandRot) * percent * torqueFactorJSON.val;
        }

        protected void UpdateIntervals()
        {
            intervalJSON.val = UnityEngine.Random.Range(intervalMinJSON.val, intervalMaxJSON.val);
            timeSinceLastInterval = intervalJSON.val;
            periodJSON.val = UnityEngine.Random.Range(periodMinJSON.val, periodMaxJSON.val);
            periodRatioJSON.val = UnityEngine.Random.Range(periodRatioMinJSON.val, periodRatioMaxJSON.val);
            forceDurationJSON.val = UnityEngine.Random.Range(forceDurationMinJSON.val, forceDurationMaxJSON.val);
            forceFactorJSON.val = UnityEngine.Random.Range(forceFactorMinJSON.val, forceFactorMaxJSON.val);
            forceQuicknessJSON.val = UnityEngine.Random.Range(forceQuicknessMinJSON.val, forceQuicknessMaxJSON.val);
            torqueFactorJSON.val = UnityEngine.Random.Range(torqueFactorMinJSON.val, torqueFactorMaxJSON.val);
            torqueQuicknessJSON.val = UnityEngine.Random.Range(torqueQuicknessMinJSON.val, torqueQuicknessMaxJSON.val);

            var forceNeg = forceAxisJSON.val.Contains("Neg") ? "Neg" : "";

            if (forceAxisJSON.val == "Rand" + forceNeg + "XYZ")
            {
                currentRandPos = forceNeg + stringXYZ[UnityEngine.Random.Range(0, stringXYZ.Length)];
            }
            else if (forceAxisJSON.val == "Rand" + forceNeg + "XY")
            {
                currentRandPos = forceNeg + stringXY[UnityEngine.Random.Range(0, stringXY.Length)];
            }
            else if (forceAxisJSON.val == "Rand" + forceNeg + "XZ")
            {
                currentRandPos = forceNeg + stringXZ[UnityEngine.Random.Range(0, stringXZ.Length)];
            }
            else if (forceAxisJSON.val == "Rand" + forceNeg + "YZ")
            {
                currentRandPos = forceNeg + stringYZ[UnityEngine.Random.Range(0, stringYZ.Length)];
            }
            else
            {
                currentRandPos = forceAxisJSON.val;
            }

            forceNeg = torqueAxisJSON.val.Contains("Neg") ? "Neg" : "";

            if (torqueAxisJSON.val == "Rand" + forceNeg + "XYZ")
            {
                currentRandRot = forceNeg + stringXYZ[UnityEngine.Random.Range(0, stringXYZ.Length)];
            }
            else if (torqueAxisJSON.val == "Rand" + forceNeg + "XY")
            {
                currentRandRot = forceNeg + stringXY[UnityEngine.Random.Range(0, stringXY.Length)];
            }
            else if (torqueAxisJSON.val == "Rand" + forceNeg + "XZ")
            {
                currentRandRot = forceNeg + stringXZ[UnityEngine.Random.Range(0, stringXZ.Length)];
            }
            else if (torqueAxisJSON.val == "Rand" + forceNeg + "YZ")
            {
                currentRandRot = forceNeg + stringYZ[UnityEngine.Random.Range(0, stringYZ.Length)];
            }
            else
            {
                currentRandRot = torqueAxisJSON.val;

            }

        }

        protected void Update()
        {
            if (timeSinceLastInterval >= 0f)
            {
                timeSinceLastInterval -= Time.deltaTime;
                if (timeSinceLastInterval < 0f)
                {
                    UpdateIntervals();
                }
            }

            timer -= Time.deltaTime;
            forceTimer -= Time.deltaTime;
            if (timer < 0.0f)
            {
                if ((flip > 0f && periodRatioJSON.val != 1f) || periodRatioJSON.val == 0f)
                {
                    if (applyForceOnReturnJSON.val)
                    {
                        flip = -1f;
                    }
                    else
                    {
                        flip = 0f;
                    }
                    timer = periodJSON.val * (1f - periodRatioJSON.val);
                    forceTimer = forceDurationJSON.val * periodJSON.val;
                }
                else
                {
                    flip = 1f;
                    timer = periodJSON.val * periodRatioJSON.val;
                    forceTimer = forceDurationJSON.val * periodJSON.val;
                }
                SetTargets(flip);
            }
            else if (forceTimer < 0.0f)
            {
                SetTargets(0f);
            }
        }

        // FixedUpdate is called with each physics simulation frame by Unity
        void FixedUpdate()
        {
            try
            {
                // apply forces here
                float timeFactor = Time.fixedDeltaTime;
                currentForce = Vector3.Lerp(currentForce, targetForce, timeFactor * forceQuicknessJSON.val);
                currentTorque = Vector3.Lerp(currentTorque, targetTorque, timeFactor * torqueQuicknessJSON.val);
                if (RB && (!SuperController.singleton || !SuperController.singleton.freezeAnimation))
                {
                    RB.AddForce(currentForce, ForceMode.Force);
                    RB.AddTorque(currentTorque, ForceMode.Force);
                }
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        UIDynamic CreateSpacer_p67(float height, bool rightSide)
        {
            UIDynamic spacer = CreateSpacer(rightSide);
            spacer.height = height;
            return spacer;
        }

        
        void RefreshMins(float v)
        {
            if (intervalMinJSON.val > intervalMaxJSON.val) intervalMinJSON.val = intervalMaxJSON.val;
            if (periodMinJSON.val > periodMaxJSON.val) periodMinJSON.val = periodMaxJSON.val;
            if (periodRatioMinJSON.val > periodRatioMaxJSON.val) periodRatioMinJSON.val = periodRatioMaxJSON.val;
            if (forceDurationMinJSON.val > forceDurationMaxJSON.val) forceDurationMinJSON.val = forceDurationMaxJSON.val;
            if (forceFactorMinJSON.val > forceFactorMaxJSON.val) forceFactorMinJSON.val = forceFactorMaxJSON.val;
            if (forceQuicknessMinJSON.val > forceQuicknessMaxJSON.val) forceQuicknessMinJSON.val = forceQuicknessMaxJSON.val;
            if (torqueFactorMinJSON.val > torqueFactorMaxJSON.val) torqueFactorMinJSON.val = torqueFactorMaxJSON.val;
            if (torqueQuicknessMinJSON.val > torqueQuicknessMaxJSON.val) torqueQuicknessMinJSON.val = torqueQuicknessMaxJSON.val;
        }

    }
}