﻿using System;
using System.Globalization;
using UnityEngine;
using UnityEngine.XR;
using System.Text;
using System.Linq;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using UnityEngine.XR;
using MacGruber;
using customevent;

namespace SPQR
{
    public class Inventory : MVRScript
    {
		

        private bool gameIsVR;
		
		bool setParentLink = false;
		string setParentSender;
		Atom setParentAtom;
		
        GameObject playerHands;
        float collisionThreshold = 0.05f;
		
		Rigidbody lhandRB;
		Rigidbody rhandRB;

        protected JSONStorableString myString;

		Dictionary<string, JSONStorableBool> enableFeatures = new Dictionary<string, JSONStorableBool>();
		
		JSONStorableFloat generalForceScaleJSON;
		JSONStorableFloat generalTorqueScaleJSON;
		JSONStorableFloat generalLengthScaleJSON;
		JSONStorableFloat generalIntensityScaleJSON;

        public override void Init()
        {
            Utils.OnInitUI(CreateUIElement);
            gameIsVR = XRSettings.isDeviceActive && !string.IsNullOrEmpty(XRSettings.loadedDeviceName);
			
			var bones = containingAtom.transform.Find("rescale2").GetComponentsInChildren<DAZBone>();
			var lhandT = bones.FirstOrDefault(v => v.name == "lHand")?.transform;
			var rhandT = bones.FirstOrDefault(v => v.name == "rHand")?.transform;
			lhandRB = lhandT.GetComponent<Rigidbody>();
			rhandRB = rhandT.GetComponent<Rigidbody>();
			
			bool showUI = false;

            SimpleTriggerHandler.LoadAssets();
            UIUpdate();
            
        }

        private void UIUpdate()
        {
        }


        void Start()
        {	
		
			RefreshColliders();
        }

        void Update()
        {
        }

        void FixedUpdate()
        {
            try
            {
				if(setParentLink){
					if(!isBeingHeld(setParentAtom)){
						setParentLink = false;
						InventoryLink(setParentAtom,setParentSender);
					}
				}
				
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }
		
		void OnEnable(){
			RefreshColliders();
		}

        void OnDisable()
        {
            containingAtom.GetComponentsInChildren<Rigidbody>().Where(f => f.name == "rHand" || f.name == "lHand" || f.name == "rThumb3" || f.name == "lThumb3").ToList().ForEach(f =>
			{
				try{
				 f.gameObject.GetComponent<TriggerCollide>().OnCollide -= ObserveTrigger;
				} catch(Exception e){
					
				}

			});
        }
		void OnReload()
        {
            OnDisable();
        }

        void OnDestroy()
        {
            Utils.OnDestroyUI();
            OnDisable();
        }



        public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
        {
            JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
            if (includePhysical || forceStore)
            {
                needsStore = true;
            }
            return jc;
        }

        public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
        {
            base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);


        }

		protected void generalForceScaleJSONUpdate(JSONStorableFloat jf) {
		
		}

        public void RefreshColliders()
        {
			OnDisable();
			containingAtom.GetComponentsInChildren<Rigidbody>().Where(f => f.name == "rHand" || f.name == "lHand" || f.name == "rThumb3" || f.name == "lThumb3").ToList().ForEach(f =>
			{
				try{
				 f.gameObject.GetComponent<TriggerCollide>().OnCollide += ObserveTrigger;
				} catch (Exception e){
					f.gameObject.AddComponent<TriggerCollide>().OnCollide += ObserveTrigger;
				}
			});
	
        }

        void ObserveTrigger(object sender, TriggerEventArgs e)
        {
			if (!enabledJSON.val) return;

            Atom parent = null;
            foreach (var component in e.collider.gameObject.GetComponentsInParent<Component>())
            {

                var atom = component as Atom;
                if (atom != null)
                {
                    parent = atom;
                    break;
                }
            }
          



            var found = "";
            var components = e.collider.transform.parent.GetComponents<Component>();

            var RB = e.collider.attachedRigidbody;

            foreach (var xatom in SuperController.singleton.GetAtoms())
            {
                var RB2 = xatom.rigidbodies.First();
                if (RB == RB2)
                {
                    found = xatom.name;
                    break;
                }

            }
			


            if (e.evtType == customevent.EventType.HIT)
            {
                float level = e.collision.relativeVelocity.magnitude / 3.0f;
                if (level > collisionThreshold)
                {
                    if (level > 3) level = 3;

                    if (e.insideTrigger)
                    { }
                    else
                    {
                        var xname = found.Length > 0 ? found : parent.name;
                        var xcollider = e.collider.name;
                        RaeClickTouch(sender.ToString(), xname, xcollider, RB);

                    }
                }
                else
                { 
					
				}
            }
            else{}
			
        }


        //////////////////////////////////////////////



		void RaeClickTouch(string sender, string name, string collider, Rigidbody RB){
			sender = sender.Replace(" (customevent.TriggerCollide)","");
			if(name==containingAtom.name) return;
			Atom a = SPQRUtils.getAtomById(name);
			if(a == null) return;
			//if(!a.IsBoolJSONParam("Grabbable")) return;
			
			var storables = a.GetStorableIDs();
			if(storables.Count == 0) return;
			var ok = false;
			for (var i=0; i<storables.Count; i++){
				if(!storables[i].Contains("SPQR.Grabbable")) continue;
				if(!SPQRUtils.getAtomStorableBool(name, storables[i],"Grabbable").val) return;
				ok = true;
			}
			if(!ok) return;
		//	SuperController.LogMessage("Detected touch from "+sender+" with "+name+" - "+collider);
			
			setParentLink = true;
			setParentAtom = a;
			setParentSender = sender;
			
		}
		
		void InventoryLink(Atom a, string sender){
			var fc = a.GetStorableByID("control") as FreeControllerV3;
			var RB = (sender == "lHand" || sender == "lThumb3") ? lhandRB : rhandRB;

			//fc.SetLinkToRigidbody(name+":"+collider);
			fc.SelectLinkToRigidbody(RB);
			//a.ClearParentAtom();
			
		}
		
		

        void UpdateMyString(string name)
        {
        }
		
		bool isBeingHeld(Atom a){
			var _fc = a.GetStorableByID("control") as FreeControllerV3;
			if(_fc.currentPositionState == FreeControllerV3.PositionState.Hold) return true;
			if(_fc.linkToRB != null){
				if(_fc.linkToRB.name == "SelectionHandles")  return true;
				if(_fc.linkToRB.name == "MouseGrab")  return true;
			}
			return false;
		}



     
        


    }



}
