﻿using System;
using System.Linq;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace SPQR {
	public class LightPulsate : MVRScript {
		
		bool sceneLoaded = false;

		JSONStorableFloat valMin;
        JSONStorableFloat valMax;
        JSONStorableFloat valSpeed;
		
		JSONStorable jsonStorableLight;
		JSONStorableFloat jsonStorableIntensity;

        Dictionary<string, JSONStorableString> titles = new Dictionary<string, JSONStorableString>();

        float currentVal;
        public override void Init() {

			InvokeRepeating("UpdateSecond", 1f, 1f);
			InvokeRepeating("UpdateConstant", 1f, 1f / 30f);
			InvokeRepeating("UpdateConstant5", 1f, 1f / 5f);
			SuperController.singleton.onSceneLoadedHandlers += OnSceneLoaded;

			valMin = new JSONStorableFloat("Light Min", 0.8f, 0f, 10f, true, true);
            valMax = new JSONStorableFloat("Light Max", 1.1f, 0f, 10f, true, true);
            valSpeed = new JSONStorableFloat("Pulsate Speed", 1f, 0f, 120f, true, true);

			if(containingAtom.type != "InvisibleLight")
			{
				SuperController.LogMessage("LightPulsate Plugin must be added to Invisible Light");
				return;
			}
            CreateTitle_p67("titlepluginname", "LightPulsate", 40, false);
            CreateTitle_p67("titlepluginname2", "patreon.com/SPQR_AETERNUM", 20, true);

            RegisterFloat(valMin);
            RegisterFloat(valMax);
            RegisterFloat(valSpeed);

			CreateSlider(valMin);
            CreateSlider(valMax);
            CreateSlider(valSpeed);

            jsonStorableLight = containingAtom.GetStorableByID("Light");
            if (jsonStorableLight == null) return;
            jsonStorableIntensity = jsonStorableLight.GetFloatJSONParam("intensity");
            if (jsonStorableIntensity == null) return;


        }

	
		void Start() {
		
		}

	
		void Update() {
            float t = (Mathf.Sin(Time.time * valSpeed.val) + 1f) / 2f; 
            currentVal = Mathf.Lerp(valMin.val, valMax.val, t);
			jsonStorableIntensity.val = currentVal;


        }

	
		void FixedUpdate() {
		
		}
		void UpdateSecond()
		{
			if (IsDisabled()) return;

		}
		void UpdateConstant()
		{
			if (IsDisabled()) return;

		}
		void UpdateConstant5()
		{
			if (IsDisabled()) return;

		}
		void OnDestroy()
		{
			CancelInvoke();
			SuperController.singleton.onSceneLoadedHandlers -= OnSceneLoaded;
		}

		void OnSceneLoaded()
		{
			sceneLoaded = true;
		}
		bool IsDisabled()
        {
			if (!enabledJSON.val)
			{
				return true;
			}
			return false;
		}


        void CreateTitle_p67(string id, string text, int fontsize, bool right, int spacing)
        {
            CreateTitle_p67(id, text, fontsize, right);
            CreateSpacer_p67(this, spacing, right);
        }


        UIDynamic CreateSpacer_p67(MVRScript X, float height, bool rightSide)
        {
            UIDynamic spacer = X.CreateSpacer(rightSide);
            spacer.height = height;
            return spacer;
        }

        void CreateTitle_p67(string id, string text, int fontsize, bool right)
        {
            if (!titles.ContainsKey(id))
                titles.Add(id, new JSONStorableString("title " + id, "\n<size=" + fontsize + "><b>" + text + "</b></size>"));
            UIDynamicTextField title1text = CreateTextField(titles[id], right);
            title1text.UItext.supportRichText = true;
            title1text.backgroundColor = new Color(0f, 0f, 0f, 0f);
            title1text.textColor = new Color(0.42f, 0.03f, 0.51f, 1f);
            title1text.height = 100f;
            title1text.UItext.alignment = right ? TextAnchor.MiddleRight : TextAnchor.MiddleLeft;
        }

    }
}