﻿using System;
using System.Linq;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace SPQR
{
	public class MagicAtomsBondageTrigger : MVRScript
	{

		CollisionTrigger ct;
		CollisionTriggerEventHandler cteh;
		Rigidbody containingRB;

		Dictionary<string, bool> personToggles = new Dictionary<string, bool>() { };
		Dictionary<string, string> personTogglesUid = new Dictionary<string, string>() { };
		Dictionary<string, string> personTogglesRB = new Dictionary<string, string>() { };
		Dictionary<string, FreeControllerV3.PositionState> personTogglesBefore = new Dictionary<string, FreeControllerV3.PositionState>() { };
		bool hasCollisions = false;

		JSONStorableStringChooser unsetType;

		public override void Init()
		{

			if (containingAtom == null) return;
			if (containingAtom.type != "CollisionTrigger") return;
			ct = containingAtom.GetComponentInChildren<CollisionTrigger>();
			if (ct == null) return;
			cteh = containingAtom.GetComponentInChildren<CollisionTriggerEventHandler>();
			if (cteh == null) return;

			containingRB = SPQRUtils.GetAtomRigidbody_p67(containingAtom);

			List<string> actions = new List<string>(new string[] { "Auto", "On", "Off", "Comply" });
			unsetType = new JSONStorableStringChooser("Unset Type", actions, "Auto", "Unset Type");
			CreatePopup(unsetType, false);
			RegisterStringChooser(unsetType);

			//InvokeRepeating("UpdateSecond_p67", 1f, 1f);
			//	InvokeRepeating("UpdateConstant_p67", 1f, 1f / 30f);
			InvokeRepeating("UpdateConstant5", 1f, 1f / 5f);

			SuperController.singleton.onSceneLoadedHandlers += OnSceneLoaded;

			


		}
		void OnSceneLoaded()
        {
            if (containingAtom.on)
            {
                Invoke("EnableAtom", 0.7f);
                Invoke("DisableAtom", 0.5f);
            }
        }
		void EnableAtom()
        {
			containingAtom.SetOn(true);
        }
		void DisableAtom()
		{
			containingAtom.SetOn(false);
		}
		void Start()
		{

		}


		void Update()
		{

		}


		void FixedUpdate()
		{

		}
		void UpdateSecond()
		{
			if (IsDisabled()) return;

		}
		void UpdateConstant()
		{
			if (IsDisabled()) return;

		}
		void UpdateConstant5()
		{
			if (!containingAtom.on) return;
			try
			{
				if (IsDisabled()) return;
				if (cteh == null) return;
				if (cteh.collidingWithDictionary == null) return;
				if (cteh.collidingWithDictionary.Count == 0 && hasCollisions == false)
				{
					return;
				}

				Dictionary<string, bool> currentOn = new Dictionary<string, bool>() { };


				for (var i = 0; i < cteh.collidingWithDictionary.Count; i++)
				{
					var collider = cteh.collidingWithDictionary.ElementAt(i).Key;
					if (collider == null) return;
					if (collider.attachedRigidbody == null) continue;
					if (collider.transform == null) continue;
					var parent = SPQRUtils.getAtomFromTransform_p67(collider.transform);
					if (parent == null) continue;
					if (parent.type != "Person") continue;

					//if (collider.attachedRigidbody.name == "lHand")
					//	SuperController.LogMessage(collider.attachedRigidbody.name + " - in - " + parent.uid);


					if (
						collider.attachedRigidbody.name == "rFoot" ||
						collider.attachedRigidbody.name == "lFoot" ||
						collider.attachedRigidbody.name == "rHand" ||
						collider.attachedRigidbody.name == "lHand"
					)
					{
						var key = parent.uid + ":" + collider.attachedRigidbody.name;
						SetControlSticky(key, parent, collider.attachedRigidbody);
						if (!currentOn.ContainsKey(key)) currentOn.Add(key, true);
						hasCollisions = true;
					}

					//if (collider.name == "Collider1")
					//{
					//	SuperController.LogMessage("legs");
					//}
					//if (collider.name == "Collider1")
					//{
					//	SuperController.LogMessage(legs);
					//}
				}
				bool hasc = false;
				for (var i = 0; i < personToggles.Count; i++)
				{
					var item = personToggles.ElementAt(i);
					if (!currentOn.ContainsKey(item.Key) && personToggles[item.Key])
					{
						UnsetItem(item.Key);
					}
					if (item.Value) hasc = true;
				}
				hasCollisions = hasc;


			}
			catch (Exception e)
			{
				//SuperController.LogError("err: " + e);
			}
		}
		void OnDestroy()
		{
			CancelInvoke();
			SuperController.singleton.onSceneLoadedHandlers -= OnSceneLoaded;
		}

		bool IsDisabled()
		{
			if (!enabledJSON.val)
			{
				return true;
			}
			return false;
		}

		void SetControlSticky(string key, Atom parent, Rigidbody rb)
		{

			if (!personToggles.ContainsKey(key))
			{
				personToggles.Add(key, false);
				personTogglesUid.Add(key, parent.uid);
				personTogglesRB.Add(key, rb.name);
				personTogglesBefore.Add(key, FreeControllerV3.PositionState.On);
			}

			if (personToggles[key] == true) return;
			var ctrl = parent.GetStorableByID(rb.name + "Control") as FreeControllerV3;
			if (SPQRUtils.isBeingHeld_p67(ctrl)) return;
			if (ctrl == null) return;
			ctrl.SetLinkToAtom(containingAtom.uid);

			SetBeforeKeys(key, ctrl);

			ctrl.currentPositionState = FreeControllerV3.PositionState.On;
			ctrl.currentRotationState = FreeControllerV3.RotationState.On;
			ctrl.SelectLinkToRigidbody(null, FreeControllerV3.SelectLinkState.PositionAndRotation);


			ctrl.currentPositionState = FreeControllerV3.PositionState.PhysicsLink;
			ctrl.currentRotationState = FreeControllerV3.RotationState.PhysicsLink;
			//ctrl.SetLinkToRigidbodyObject(containingRB.name);
			
			ctrl.SelectLinkToRigidbody(containingRB, FreeControllerV3.SelectLinkState.PositionAndRotation);
			personToggles[key] = true;

			//SuperController.LogMessage("set " + key);

		}
		void SetBeforeKeys(string key, FreeControllerV3 ctrl)
		{
			if (ctrl.currentPositionState == FreeControllerV3.PositionState.On)
			{
				personTogglesBefore[key] = FreeControllerV3.PositionState.On;
			}
			else if (ctrl.currentPositionState == FreeControllerV3.PositionState.Comply)
			{
				personTogglesBefore[key] = FreeControllerV3.PositionState.Comply;
			}
			else if (ctrl.currentPositionState == FreeControllerV3.PositionState.Off)
			{
				personTogglesBefore[key] = FreeControllerV3.PositionState.Off;
			}
			else
			{
				personTogglesBefore[key] = FreeControllerV3.PositionState.On;
			}
		}

		void SetFromBefore(string key, FreeControllerV3 ctrl)
		{
			if (unsetType.val == "On") personTogglesBefore[key] = FreeControllerV3.PositionState.On;
			if (unsetType.val == "Off") personTogglesBefore[key] = FreeControllerV3.PositionState.Off;
			if (unsetType.val == "Comply") personTogglesBefore[key] = FreeControllerV3.PositionState.Comply;
			SetFromBeforeKeys(key, ctrl);
		}

		void SetFromBeforeKeys(string key, FreeControllerV3 ctrl)
		{
			if (personTogglesBefore[key] == FreeControllerV3.PositionState.On)
			{
				ctrl.currentPositionState = FreeControllerV3.PositionState.On;
				ctrl.currentRotationState = FreeControllerV3.RotationState.On;
			}
			if (personTogglesBefore[key] == FreeControllerV3.PositionState.Comply)
			{
				ctrl.currentPositionState = FreeControllerV3.PositionState.Comply;
				ctrl.currentRotationState = FreeControllerV3.RotationState.Comply;
			}
			if (personTogglesBefore[key] == FreeControllerV3.PositionState.Off)
			{
				ctrl.currentPositionState = FreeControllerV3.PositionState.Off;
				ctrl.currentRotationState = FreeControllerV3.RotationState.Off;
			}
		}
		void UnsetAll()
		{
			//	SuperController.LogMessage("UnsetALl");

			for (var i = 0; i < personToggles.Count; i++)
			{
				var item = personToggles.ElementAt(i);
				if (item.Value == false) continue;
				UnsetItem(item.Key);
			}
		}
		void UnsetItem(string k)
		{

			if (!personTogglesRB.ContainsKey(k)) return;
			var rbName = personTogglesRB[k];
			var parent = SPQRUtils.GetAtomById_p67(personTogglesUid[k]);
			if (rbName == null) return;
			if (parent == null) return;

			var ctrl = parent.GetStorableByID(rbName + "Control") as FreeControllerV3;
			if (ctrl == null) return;
			if (SPQRUtils.isBeingHeld_p67(ctrl)) return;

			var distance = Vector3.Distance(ctrl.transform.position, containingAtom.mainController.transform.position);
			if (distance < 0.15f) return;

			ctrl.SetLinkToAtom(null);
			ctrl.SetLinkToRigidbody(null);
			ctrl.SelectLinkToRigidbody(null, FreeControllerV3.SelectLinkState.PositionAndRotation);
			SetFromBefore(k, ctrl);


			personToggles[k] = false;

		}
	}
}