﻿using System;
using System.Linq;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using MVR.FileManagementSecure;

namespace SPQR {


	public class CommandPanel : MVRScript {
		
		bool sceneLoaded = false;
		public string SavePath_p67 = "Saves\\PluginData\\MagicAtomsCommandPanel";

		public Dictionary<string, EventTrigger> myEventTriggers_p67 = new Dictionary<string, EventTrigger>();
		public Dictionary<string, UIDynamicButton> buttons_p67 = new Dictionary<string, UIDynamicButton>();
		public Dictionary<string, JSONStorableString> strings_p67 = new Dictionary<string, JSONStorableString>();


		List<string> actions = new List<string>(new string[] { "...", "TriggerAll", "TriggerRandom", "Trigger0", "Trigger1", "Trigger2", "Trigger3", "Trigger4", "Trigger5", "Trigger6", "Trigger7", "Trigger8", "Trigger9" });
		List<string> keysProfile = new List<string>(new string[] {"Numpad","Ctrl+Numpad", "Alt+Numpad", "#", "Ctrl+#", "Alt+#", "Shift+#" });

		JSONStorableStringChooser actionType;
		JSONStorableStringChooser keysProfileType;

		public override void Init() {
			try
			{
				InvokeRepeating("UpdateSecond", 1f, 1f);
				InvokeRepeating("UpdateConstant", 1f, 1f / 30f);
				InvokeRepeating("UpdateConstant5", 1f, 1f / 5f);
				SuperController.singleton.onSceneLoadedHandlers += OnSceneLoaded;
				SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;


				
			

				for (var i = 0; i <= 9; i++)
				{
					strings_p67.Add("CMD " + i, new JSONStorableString("Description " + i, "Description " + i));
					RegisterString(strings_p67["CMD " + i]);
					SPQRUtils.CreateTextInput_p67(this, null, strings_p67["CMD " + i], true);
					myEventTriggers_p67.Add("CMD " + i, new EventTrigger(this, "CMD " + i));
					buttons_p67.Add("CMD " + i, SPQRUtils.CreateButton_p67(this, "CMD " + i, myEventTriggers_p67["CMD " + i].OpenPanel, true));
				}


				buttons_p67["load_preset"] = CreateButton("Load Preset", false);
				buttons_p67["load_preset"].button.onClick.AddListener(() =>
				{
					FileManagerSecure.CreateDirectory(SavePath_p67);
					var shortcuts = FileManagerSecure.GetShortCutsForDirectory(SavePath_p67);
					SuperController.singleton.GetMediaPathDialog(HandleLoadPreset_p67, "json", SavePath_p67, false, true, false, null, false, shortcuts);
				});

				buttons_p67["save_preset"] = CreateButton("Save Preset", false);
				buttons_p67["save_preset"].button.onClick.AddListener(() =>
				{
					FileManagerSecure.CreateDirectory(SavePath_p67);
					var fileBrowserUI = SuperController.singleton.fileBrowserUI;
					fileBrowserUI.SetTitle("Save preset");
					fileBrowserUI.fileRemovePrefix = null;
					fileBrowserUI.hideExtension = true;
					fileBrowserUI.keepOpen = false;
					fileBrowserUI.canCancel = true;
					fileBrowserUI.fileFormat = "json";
					fileBrowserUI.defaultPath = SavePath_p67;
					fileBrowserUI.showDirs = true;
					fileBrowserUI.showFiles = true;
					fileBrowserUI.shortCuts = null;
					fileBrowserUI.browseVarFilesAsDirectories = false;
					fileBrowserUI.SetTextEntry(true);
					fileBrowserUI.Show(HandleSavePreset_p67);
					fileBrowserUI.fileEntryField.text = (int)(DateTime.UtcNow - new DateTime(1970, 1, 1)).TotalSeconds + ".json";
					fileBrowserUI.ActivateFileNameField();

				});
				keysProfileType = new JSONStorableStringChooser("Keys", keysProfile, "Numpad", "Keys");
				CreatePopup(keysProfileType, false);
				RegisterStringChooser(keysProfileType);

				actionType = new JSONStorableStringChooser("Actions", actions, "...", "Actions", callbackUpdateAction);
				CreatePopup(actionType, false);
				RegisterStringChooser(actionType);


				SimpleTriggerHandler.LoadAssets_p67();

				

			} 
			catch(Exception e)
            {
				SuperController.LogError("err " + e);
            }
		}

	
		void Start() {
		
		}

	
		void Update() {

			foreach (var item in myEventTriggers_p67)
			{
				item.Value.Update();
			}

			if (LookInputModule.singleton.inputFieldActive) return;
			var allow = true; 
			if(keysProfileType.val.Contains("Ctrl") && !Input.GetKey(KeyCode.LeftControl))
            {
				allow = false;
            }
			if (keysProfileType.val.Contains("Alt") && !Input.GetKey(KeyCode.LeftAlt))
			{
				allow = false;
			}
			if (keysProfileType.val.Contains("Shift") && !Input.GetKey(KeyCode.LeftShift))
			{
				allow = false;
			}


			if (allow)
			{
				if (keysProfileType.val.Contains("Numpad"))
				{
					if (Input.GetKeyDown(KeyCode.Keypad0)) TriggerCustom(0);
					if (Input.GetKeyDown(KeyCode.Keypad1)) TriggerCustom(1);
					if (Input.GetKeyDown(KeyCode.Keypad2)) TriggerCustom(2);
					if (Input.GetKeyDown(KeyCode.Keypad3)) TriggerCustom(3);
					if (Input.GetKeyDown(KeyCode.Keypad4)) TriggerCustom(4);
					if (Input.GetKeyDown(KeyCode.Keypad5)) TriggerCustom(5);
					if (Input.GetKeyDown(KeyCode.Keypad6)) TriggerCustom(6);
					if (Input.GetKeyDown(KeyCode.Keypad7)) TriggerCustom(7);
					if (Input.GetKeyDown(KeyCode.Keypad8)) TriggerCustom(8);
					if (Input.GetKeyDown(KeyCode.Keypad9)) TriggerCustom(9);
				}
				else if (keysProfileType.val.Contains("#"))
				{
					if (Input.GetKeyDown(KeyCode.Alpha0)) TriggerCustom(0);
					if (Input.GetKeyDown(KeyCode.Alpha1)) TriggerCustom(1);
					if (Input.GetKeyDown(KeyCode.Alpha2)) TriggerCustom(2);
					if (Input.GetKeyDown(KeyCode.Alpha3)) TriggerCustom(3);
					if (Input.GetKeyDown(KeyCode.Alpha4)) TriggerCustom(4);
					if (Input.GetKeyDown(KeyCode.Alpha5)) TriggerCustom(5);
					if (Input.GetKeyDown(KeyCode.Alpha6)) TriggerCustom(6);
					if (Input.GetKeyDown(KeyCode.Alpha7)) TriggerCustom(7);
					if (Input.GetKeyDown(KeyCode.Alpha8)) TriggerCustom(8);
					if (Input.GetKeyDown(KeyCode.Alpha9)) TriggerCustom(9);
				}
			}
		}

	
		void FixedUpdate() {
		
		}
		void UpdateSecond()
		{
			if (IsDisabled()) return;

		}
		void UpdateConstant()
		{
			if (IsDisabled()) return;

		}
		void UpdateConstant5()
		{
			if (IsDisabled()) return;

		}
		void OnDestroy()
		{
			CancelInvoke();
			SuperController.singleton.onSceneLoadedHandlers -= OnSceneLoaded;
			SuperController.singleton.onAtomUIDRenameHandlers -= OnAtomRename;
			foreach (var item in myEventTriggers_p67)
			{
				item.Value.Remove();
			}
		}
		void OnAtomRename(string oldName, string neNname)
        {
			foreach (var item in myEventTriggers_p67)
				item.Value.SyncAtomNames();
		}
		public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
		{
			
			JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
			if (includePhysical || forceStore)
			{
				needsStore = true;
				foreach (var item in myEventTriggers_p67)
				{
					jc[item.Value.Name] = item.Value.GetJSON(base.subScenePrefix);
				}
			}
			

			return jc;
		}

		public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
		{
			base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
			if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
			{
				foreach (var item in myEventTriggers_p67)
				{
					myEventTriggers_p67[item.Key].RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
				}
			}


		}


		void OnSceneLoaded()
		{
			sceneLoaded = true;
		}
		bool IsDisabled()
        {
			if (!enabledJSON.val)
			{
				return true;
			}
			return false;
		}

		void TriggerCustom(int i)
        {
			string s = "CMD " + i;
			myEventTriggers_p67[s].Trigger();
			FlashButton(s);
		}

		void callbackUpdateAction(string action)
        {
			var rand = UnityEngine.Random.Range(0, 10);
			for(var i=0; i<=9; i++)
            {
				if (action == "Trigger" + i) TriggerCustom(i);
				if (action == "TriggerAll") TriggerCustom(i);
				if (action =="TriggerRandom" && i==rand) TriggerCustom(i); 
			}
			
			actionType.val = "...";

		}

		void FlashButton(string s)
        {
			buttons_p67[s].buttonColor = new Color(0.2f, 0.9f, 0.2f);
			StartCoroutine(CoroutineFlashButton_p67(s));
		}

		private IEnumerator Wait_p67(float seconds)
		{
			yield return new WaitForSeconds(seconds);
		}

		private IEnumerator CoroutineFlashButton_p67(string s)
		{
			yield return StartCoroutine(Wait_p67(0.5f));
			buttons_p67[s].buttonColor = new Color(0.839f, 0.839f, 0.839f);
		}


		public void HandleLoadPreset_p67(string path)
		{
			if (path == null) return;
			var jc = (JSONClass)LoadJSON(path);
			base.RestoreFromJSON(jc);

			for (var i = 0; i <= 9; i++)
            {
				myEventTriggers_p67["CMD " + i].RestoreFromJSON(jc, base.subScenePrefix, false, true);
			}

		}

		public void HandleSavePreset_p67(string path)
		{
			if (path == null || path == "") return;
			if (!path.EndsWith(".json"))
			{
				path += ".json";
			}
			var jc = GetJSON(true, true, true);
			SaveJSON(jc, path);
			SuperController.singleton.DoSaveScreenshot(path);
		}
	}
}