﻿using UnityEngine;
using UnityEngine.UI;
using System;
using System.Linq;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using SimpleJSON;
using Random2 = System.Random;
using Random = UnityEngine.Random;


namespace SPQR
{
    public class DancingAtom : MVRScript
    {
        LineDrawer[] lds;
        int b = 64;
        int customb = 16;

        int ratio = 10;

        float min_delta = 0.0002f;

        Dictionary<int, float> history;

        JSONStorableStringChooser jScaleX;
        JSONStorableStringChooser jScaleY;
        JSONStorableStringChooser jScaleZ;
        JSONStorableStringChooser jScale;

        JSONStorableStringChooser jRotX;
        JSONStorableStringChooser jRotY;
        JSONStorableStringChooser jRotZ;

        public override void Init()
        {
            history = new Dictionary<int, float> { };
            for (var i = 1; i <= customb; i++)
            {
                history[i] = 0f;
            }


            List<string> choicesActions = new List<string>();
            choicesActions.Add("None");
            for (var i=1;i<=16;i++) choicesActions.Add(i.ToString());


            jScaleX = new JSONStorableStringChooser("Scale X Channel", choicesActions, "4", "Scale X Channel");
            RegisterStringChooser(jScaleX);
            UIDynamicPopup udp = CreateScrollablePopup(jScaleX, false);
            udp.labelWidth = 300f;

            jScaleY = new JSONStorableStringChooser("Scale Y Channel", choicesActions, "2", "Scale Y Channel");
            RegisterStringChooser(jScaleY);
            UIDynamicPopup udp2 = CreateScrollablePopup(jScaleY, false);
            udp2.labelWidth = 300f;

            jScaleZ = new JSONStorableStringChooser("Scale Z Channel", choicesActions, "None", "Scale Z Channel");
            RegisterStringChooser(jScaleZ);
            UIDynamicPopup udp3 = CreateScrollablePopup(jScaleZ, false);
            udp3.labelWidth = 300f;

            jScale = new JSONStorableStringChooser("Scale All Channel", choicesActions, "None", "Scale All Channel");
            RegisterStringChooser(jScale);
            UIDynamicPopup udp7 = CreateScrollablePopup(jScale, false);
            udp7.labelWidth = 300f;

            jRotX = new JSONStorableStringChooser("Rot X Channel", choicesActions, "13", "Rot X Channel");
            RegisterStringChooser(jRotX);
            UIDynamicPopup udp4 = CreateScrollablePopup(jRotX, true);
            udp4.labelWidth = 300f;

            jRotY = new JSONStorableStringChooser("Rot Y Channel", choicesActions, "12", "Rot Y Channel");
            RegisterStringChooser(jRotY);
            UIDynamicPopup udp5 = CreateScrollablePopup(jRotY, true);
            udp5.labelWidth = 300f;


            jRotZ = new JSONStorableStringChooser("Rot Z Channel", choicesActions, "14", "Rot Z Channel");
            RegisterStringChooser(jRotZ);
            UIDynamicPopup udp6 = CreateScrollablePopup(jRotZ, true);
            udp6.labelWidth = 300f;

            UIDynamicButton dbutton = CreateButton("Shuffle", true);
            if (dbutton != null)
            {
                dbutton.button.onClick.AddListener(ShuffleAll);
            }


            InvokeRepeating("UpdateConstant", 1f, 1f / 30f);
        }




        void Start()
        {

        }
		void Update(){

		}
		void OnDestroy()
		{
			CancelInvoke();
		}

        void UpdateConstant()
        {
			if (!enabledJSON.val) return;
			if (SuperController.singleton.freezeAnimationToggle.isOn) return;


            float[] spectrum = new float[b];
            AudioListener.GetSpectrumData(spectrum, 0, FFTWindow.Hanning);

            var avg = 0f;
            var ite = 1;
            var average = 0f; 
            for (int i = 1; i < spectrum.Length - 1; i++)
            {
                var scale = i * 20;

                avg += spectrum[i];


                if (i % (b / customb) == 0)
                {
                    var val = history[ite];
                    avg /= (b / customb);
                    var newval = ((ratio - 1) * val + avg * scale) / ratio;

                    newval = Clamp((newval - 0.03f), 0f, 1f);
                    
                  

                    history[ite] = newval;
                    doOp(ite, newval);
                    average+=newval;


                    avg = 0;
                    ite++;
                }

            }
            average/=customb;
             doOp(-1, average);


        }


        void FixedUpdate()
        {

        }

        string GetShuffleVal()
        {
            var rand = Random.Range(-4, 17);
            if (rand <= 0) return "None";
            return rand.ToString();
        }
        void ShuffleAll()
        {
            jRotX.val = GetShuffleVal();
            jRotY.val = GetShuffleVal();
            jRotZ.val = GetShuffleVal();
            jScale.val = GetShuffleVal();
            jScaleX.val = GetShuffleVal();
            jScaleY.val = GetShuffleVal();
            jScaleZ.val = GetShuffleVal();
        }



        public static float Clamp(float value, float min, float max)
        {
            return (value < min) ? min : (value > max) ? max : value;
        }

        void doOp(int ite, float newval)
        {
            var damping = 50f;
            var scalefactor = 0.5f;

            if (ite.ToString() == jScale.val)
            {
                SPQRUtils.setAtomStorableFloat_p67(containingAtom.uid, "scale", "scale", 1f + 0.25f * newval);
                SPQRUtils.setAtomStorableFloat_p67(containingAtom.uid, "scale", "Scale", 1f + 0.25f * newval);
            }
            try
            {
                if (ite.ToString() == jScaleX.val) SPQRUtils.setAtomStorableFloat_p67(containingAtom.uid, "scale", "scaleX", 1f + scalefactor * newval);
                if (ite.ToString() == jScaleY.val) SPQRUtils.setAtomStorableFloat_p67(containingAtom.uid, "scale", "scaleY", 1f + scalefactor * newval);
                if (ite.ToString() == jScaleZ.val) SPQRUtils.setAtomStorableFloat_p67(containingAtom.uid, "scale", "scaleZ", 1f + scalefactor * newval);
            } catch(Exception e)
            {
                SuperController.LogError("err:" + e);
            }
           
            if (ite.ToString() == jRotX.val)
            {
                var desiredRotQ = Quaternion.Euler(45f * newval, 0f, 0f);
                containingAtom.mainController.transform.rotation = Quaternion.Lerp(containingAtom.mainController.transform.rotation, desiredRotQ, Time.deltaTime * damping);
            }
            if (ite.ToString() == jRotY.val)
            {
                var desiredRotQ = Quaternion.Euler( 0f, 45f * newval, 0f);
                containingAtom.mainController.transform.rotation = Quaternion.Lerp(containingAtom.mainController.transform.rotation, desiredRotQ, Time.deltaTime * damping);
            }
            if (ite.ToString() == jRotZ.val)
            {
                var desiredRotQ = Quaternion.Euler(0f, 0f, 45f * newval);
                containingAtom.mainController.transform.rotation = Quaternion.Lerp(containingAtom.mainController.transform.rotation, desiredRotQ, Time.deltaTime * damping);
            }
        }
    }









}