﻿using System;
using System.Linq;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using Random = UnityEngine.Random;

namespace SPQR {
	public class ShakyCam : MVRScript {
		int forcesNumber = 3;
		Dictionary<string, CycleforceGeneral> cycleForces = new Dictionary<string, CycleforceGeneral>() { };
		public Dictionary<string, JSONStorableFloat> sliders = new Dictionary<string, JSONStorableFloat>();

		public override void Init() {

			if (containingAtom == null) return;
			if (containingAtom.type != "WindowCamera") return;

			string[] axisChoices = System.Enum.GetNames(typeof(ForceProducerV2.AxisName));
			List<string> axisChoicesList = new List<string>(axisChoices);


			containingAtom.mainController.physicsEnabled = true;




			sliders.Add("Shakiness", new JSONStorableFloat("Shakiness", 1f, UpdateShakiness, 0f, 10f, false));
			RegisterFloat(sliders["Shakiness"]);
			CreateSlider(sliders["Shakiness"], false);


			for (var i = 1; i <= forcesNumber; i++) {
				cycleForces["force"+i] = new CycleforceGeneral(this);
				
				sliders.Add("force"+i+"_factor", new JSONStorableFloat("force" + i + "_factor", 1f, UpdateShakiness, 0f, 10f, false));
				sliders.Add("force"+i+"_force", new JSONStorableFloat("force" + i + "_force", 1f, UpdateShakiness, 0f, 10f, false));
				
				sliders["force" + i + "_factor"].storeType = JSONStorableParam.StoreType.Full;
				sliders["force" + i + "_force"].storeType = JSONStorableParam.StoreType.Full;

				RegisterFloat(sliders["force" + i + "_factor"]);
				RegisterFloat(sliders["force" + i + "_force"]);
								
				//CreateSlider(sliders_p67["force" + i + "_factor"], false);
				//CreateSlider(sliders_p67["force" + i + "_force"], false);



				var ctitle = "Force" + i;
				var cycleforcestype = "force" + i;
				cycleForces[cycleforcestype].generalForceScaleJSON = new JSONStorableFloat(ctitle + " Force", 1f, 0f, 50f, true);
				cycleForces[cycleforcestype].generalForceScaleJSON.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(cycleForces[cycleforcestype].generalForceScaleJSON);
				//   CreateSlider(generalForceScaleJSON_p67, true);

				var defaultTorqueScale = 1f;
				if (cycleforcestype == "reaction_spank") defaultTorqueScale = 3f;
				if (cycleforcestype == "reaction_tease") defaultTorqueScale = 0.25f;
				cycleForces[cycleforcestype].generalTorqueScaleJSON = new JSONStorableFloat(ctitle + " Torque", defaultTorqueScale, 0f, 50f, true);
				cycleForces[cycleforcestype].generalTorqueScaleJSON.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(cycleForces[cycleforcestype].generalTorqueScaleJSON);
				// CreateSlider(generalTorqueScaleJSON_p67, true);

				cycleForces[cycleforcestype].generalForceQuicknessScaleJSON = new JSONStorableFloat(ctitle + " F Quickness", 1f, 0f, 50f, true);
				cycleForces[cycleforcestype].generalForceQuicknessScaleJSON.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(cycleForces[cycleforcestype].generalForceQuicknessScaleJSON);
				//  CreateSlider(generalForceQuicknessScaleJSON_p67, true);

				var defaultgeneralTorqueQuicknessScaleJSON = 1f;
				if (cycleforcestype == "reaction_tease") defaultgeneralTorqueQuicknessScaleJSON = 50f;
				cycleForces[cycleforcestype].generalTorqueQuicknessScaleJSON = new JSONStorableFloat(ctitle + " T Quickness", defaultgeneralTorqueQuicknessScaleJSON, 0f, 50f, true);
				cycleForces[cycleforcestype].generalTorqueQuicknessScaleJSON.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(cycleForces[cycleforcestype].generalTorqueQuicknessScaleJSON);
				//  CreateSlider(generalTorqueQuicknessScaleJSON_p67, true);

				var defaultgeneralLengthScale = 1f;
				if (cycleforcestype == "reaction_tease") defaultgeneralLengthScale = 0.5f;
				cycleForces[cycleforcestype].generalLengthScaleJSON = new JSONStorableFloat(ctitle + " Duration", defaultgeneralLengthScale, 0f, 50f, true);
				cycleForces[cycleforcestype].generalLengthScaleJSON.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(cycleForces[cycleforcestype].generalLengthScaleJSON);
				//  CreateSlider(generalLengthScaleJSON_p67, true);

				var defaultgeneralIntensityScaleJSON = 1f;
				if (cycleforcestype == "reaction_tease") defaultgeneralIntensityScaleJSON = 0.36f;
				cycleForces[cycleforcestype].generalIntensityScaleJSON = new JSONStorableFloat(ctitle + " Intensity", defaultgeneralIntensityScaleJSON, 0f, 10f, true);
				cycleForces[cycleforcestype].generalIntensityScaleJSON.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(cycleForces[cycleforcestype].generalIntensityScaleJSON);
				// CreateSlider(generalIntensityScaleJSON_p67, true);




				cycleForces[cycleforcestype].receiverChoices = new List<string>();
				cycleForces[cycleforcestype].receiverNameToForceReceiver = new Dictionary<string, ForceReceiver>();
				cycleForces[cycleforcestype].receiverChoices.Add("none");
				foreach (ForceReceiver fr in containingAtom.forceReceivers)
				{
					cycleForces[cycleforcestype].receiverChoices.Add(fr.name);
					cycleForces[cycleforcestype].receiverNameToForceReceiver.Add(fr.name, fr);
				}
				cycleForces[cycleforcestype].receiverChoiceJSON = new JSONStorableStringChooser(ctitle + " receiver", cycleForces[cycleforcestype].receiverChoices, null, "Receiver", cycleForces[cycleforcestype].SyncReceiver);
				cycleForces[cycleforcestype].receiverChoiceJSON.storeType = JSONStorableParam.StoreType.Full;
				RegisterStringChooser(cycleForces[cycleforcestype].receiverChoiceJSON);


				cycleForces[cycleforcestype].forceAxisJSON = new JSONStorableStringChooser(ctitle + " forceAxis", axisChoicesList, cycleForces[cycleforcestype]._forceAxis.ToString(), "Force Axis", cycleForces[cycleforcestype].SyncForceAxis);
				cycleForces[cycleforcestype].forceAxisJSON.storeType = JSONStorableParam.StoreType.Full;
				RegisterStringChooser(cycleForces[cycleforcestype].forceAxisJSON);

				cycleForces[cycleforcestype].torqueAxisJSON = new JSONStorableStringChooser(ctitle + " torqueAxis", axisChoicesList, cycleForces[cycleforcestype]._torqueAxis.ToString(), "Torque Axis", cycleForces[cycleforcestype].SyncTorqueAxis);
				cycleForces[cycleforcestype].torqueAxisJSON.storeType = JSONStorableParam.StoreType.Full;
				RegisterStringChooser(cycleForces[cycleforcestype].torqueAxisJSON);

				cycleForces[cycleforcestype].periodJSON = new JSONStorableFloat(ctitle + " period", 0.2f, 0f, 10f, false);
				cycleForces[cycleforcestype].periodJSON.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(cycleForces[cycleforcestype].periodJSON);

				cycleForces[cycleforcestype].periodRatioJSON = new JSONStorableFloat(ctitle + " periodRatio", 0.5f, 0f, 1f, true);
				cycleForces[cycleforcestype].periodRatioJSON.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(cycleForces[cycleforcestype].periodRatioJSON);

				cycleForces[cycleforcestype].forceDurationJSON = new JSONStorableFloat(ctitle + " forceDuration", 1f, 0f, 1f, true);
				cycleForces[cycleforcestype].forceDurationJSON.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(cycleForces[cycleforcestype].forceDurationJSON);


				cycleForces[cycleforcestype].forceFactorJSON = new JSONStorableFloat(ctitle + " forceFactor", 20f, 0f, 1000f, false, true);
				cycleForces[cycleforcestype].forceFactorJSON.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(cycleForces[cycleforcestype].forceFactorJSON);

				cycleForces[cycleforcestype].forceQuicknessJSON = new JSONStorableFloat(ctitle + " forceQuickness", 10f, 0f, 50f, false, true);
				cycleForces[cycleforcestype].forceQuicknessJSON.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(cycleForces[cycleforcestype].forceQuicknessJSON);

				cycleForces[cycleforcestype].torqueFactorJSON = new JSONStorableFloat(ctitle + " torqueFactor", 5f, 0f, 100f, false, true);
				cycleForces[cycleforcestype].torqueFactorJSON.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(cycleForces[cycleforcestype].torqueFactorJSON);


				cycleForces[cycleforcestype].torqueQuicknessJSON = new JSONStorableFloat(ctitle + " torqueQuickness", 10f, 0f, 50f, false, true);
				cycleForces[cycleforcestype].torqueQuicknessJSON.storeType = JSONStorableParam.StoreType.Full;
				RegisterFloat(cycleForces[cycleforcestype].torqueQuicknessJSON);

				cycleForces[cycleforcestype].applyForceOnReturnJSON = new JSONStorableBool(ctitle + " applyForceOnReturn", true);
				cycleForces[cycleforcestype].applyForceOnReturnJSON.storeType = JSONStorableParam.StoreType.Full;
				RegisterBool(cycleForces[cycleforcestype].applyForceOnReturnJSON);


			}

			SetupForces();

			InvokeRepeating("UpdateSecond", 1f, 1f);
			//InvokeRepeating("UpdateConstant_p67", 1f, 1f / 30f);
			//InvokeRepeating("UpdateConstant5_p67", 1f, 1f / 5f);
		}

		
		void Start() {
		
		}

	
		void Update() {

			for(var i=0; i<cycleForces.Count; i++)
			{
				cycleForces.ElementAt(i).Value.Update();
			}
		}

	
		void FixedUpdate() {
			for (var i = 0; i < cycleForces.Count; i++)
			{
				cycleForces.ElementAt(i).Value.FixedUpdate(cycleForces.ElementAt(i).Key);
			}
		}
		void UpdateSecond()
		{
			if (IsDisabled()) return;

			if (containingAtom.mainController.physicsEnabled == false) containingAtom.mainController.physicsEnabled = true;

			SetupForcesRandom();
			

		}
		void UpdateConstant()
		{
			if (IsDisabled()) return;

		}
		void UpdateConstant5()
		{
			if (IsDisabled()) return;
		

		}
		void OnDestroy()
		{
			CancelInvoke();
		}

		bool IsDisabled()
        {
			if (!enabledJSON.val)
			{
				return true;
			}
			return false;
		}

		void SetupForces()
        {
			SetupForce1();
			SetupForce2();
			SetupForce3();
		}
		void SetupForcesRandom()
		{
			if(SPQRUtils.checkChance_p67(25f))
				SetupForce1();
			if (SPQRUtils.checkChance_p67(25f))
				SetupForce2();
			if (SPQRUtils.checkChance_p67(25f))
				SetupForce3();
		}
		void SetupForceClean()
		{

			var name = "force1";

			var speedFactor = sliders["force1_factor"].val;
			var forceFactor = sliders["force1_force"].val;

			cycleForces[name].forceAxisJSON.val = SPQRUtils.RandomFromString_p67(new string[] { "Z" });
			cycleForces[name].torqueAxisJSON.val = SPQRUtils.RandomFromString_p67(new string[] { "X" });
			cycleForces[name].receiverChoiceJSON.val = "object";
			cycleForces[name].periodJSON.val = 0.5f / speedFactor;
			cycleForces[name].periodRatioJSON.val = 0.5f;
			cycleForces[name].forceDurationJSON.val = 1f / speedFactor;
			cycleForces[name].forceFactorJSON.val = 0f * forceFactor;
			cycleForces[name].forceQuicknessJSON.val = 10f * speedFactor;
			cycleForces[name].torqueFactorJSON.val = 5f * forceFactor;
			cycleForces[name].torqueQuicknessJSON.val = 10f * speedFactor;
			cycleForces[name].applyForceOnReturnJSON.val = true;
		}
		void SetupForce1()
		{

			var name = "force1";

			var speedFactor = sliders[name + "_factor"].val;
			var forceFactor = sliders[name + "_force"].val;

			cycleForces[name].forceAxisJSON.val = SPQRUtils.RandomFromString_p67(new string[] { "X", "Y", "Z" });
			cycleForces[name].torqueAxisJSON.val = SPQRUtils.RandomFromString_p67(new string[] { "X","Y","Z" });
			cycleForces[name].receiverChoiceJSON.val = "object";
			cycleForces[name].periodJSON.val = 1.82f / speedFactor;
			cycleForces[name].periodRatioJSON.val = Random.Range(0.33f, 0.66f);
			cycleForces[name].forceDurationJSON.val = 1f / speedFactor;
			cycleForces[name].forceFactorJSON.val = Random.Range(16f, 19f) * forceFactor;
			cycleForces[name].forceQuicknessJSON.val = Random.Range(2f, 3f) * speedFactor;
			cycleForces[name].torqueFactorJSON.val = Random.Range(0.14f, 0.20f) * forceFactor;
			cycleForces[name].torqueQuicknessJSON.val = Random.Range(0.41f, 0.65f) * speedFactor;
			cycleForces[name].applyForceOnReturnJSON.val = true;
		}
		void SetupForce2()
		{
			var name = "force2";

			var speedFactor = sliders[name+"_factor"].val;
			var forceFactor = sliders[name+"_force"].val;

			cycleForces[name].forceAxisJSON.val = SPQRUtils.RandomFromString_p67(new string[] { "X", "Y", "Z" });
			cycleForces[name].torqueAxisJSON.val = SPQRUtils.RandomFromString_p67(new string[] { "X", "Y", "Z" });
			cycleForces[name].receiverChoiceJSON.val = "object";
			cycleForces[name].periodJSON.val = 2.50f / speedFactor;
			cycleForces[name].periodRatioJSON.val = Random.Range(0.3f, 0.7f);
			cycleForces[name].forceDurationJSON.val = 1f / speedFactor;
			cycleForces[name].forceFactorJSON.val = Random.Range(14f, 22f) * forceFactor;
			cycleForces[name].forceQuicknessJSON.val = Random.Range(2.5f, 3.5f) * speedFactor;
			cycleForces[name].torqueFactorJSON.val = Random.Range(0.28f, 0.37f) * forceFactor;
			cycleForces[name].torqueQuicknessJSON.val = Random.Range(0.08f, 0.13f) * speedFactor;
			cycleForces[name].applyForceOnReturnJSON.val = true;
		}
		void SetupForce3()
		{
			var name = "force3";

			var speedFactor = sliders[name + "_factor"].val;
			var forceFactor = sliders[name + "_force"].val;

			cycleForces[name].forceAxisJSON.val = SPQRUtils.RandomFromString_p67(new string[] { "X", "Y", "Z" });
			cycleForces[name].torqueAxisJSON.val = SPQRUtils.RandomFromString_p67(new string[] { "X", "Y", "Z" });
			cycleForces[name].receiverChoiceJSON.val = "object";
			cycleForces[name].periodJSON.val = 0.32f / speedFactor;
			cycleForces[name].periodRatioJSON.val = Random.Range(0.4f, 0.6f);
			cycleForces[name].forceDurationJSON.val = 1f / speedFactor;
			cycleForces[name].forceFactorJSON.val = Random.Range(20f, 28f) * forceFactor;
			cycleForces[name].forceQuicknessJSON.val = Random.Range(1f, 1.4f) * speedFactor;
			cycleForces[name].torqueFactorJSON.val = Random.Range(0.03f, 0.08f) * forceFactor;
			cycleForces[name].torqueQuicknessJSON.val = Random.Range(0.5f, 0.69f) * speedFactor;
			cycleForces[name].applyForceOnReturnJSON.val = true;
		}

		void UpdateShakiness(float shakiness)
        {
			for (var i = 1; i <= forcesNumber; i++)
			{
				sliders["force" + i + "_force"].val = shakiness;
			}
			SetupForces();
		}


	}
}