﻿using System;
using System.Linq;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using Random = UnityEngine.Random;

namespace SPQR {
	public class ShakyWindow : MVRScript {
		public Dictionary<string, JSONStorableFloat> sliders = new Dictionary<string, JSONStorableFloat>();
		public Dictionary<string, JSONStorableBool> enableFeatures = new Dictionary<string, JSONStorableBool>();

		int forcesNumber = 3;

		float maxPosX = 0.005f;
		float maxPosY = 0.01f;
		float maxPosZ = 0.001f;
		float maxMove = 1f;
		float maxMoveMin = 0.1f;
		float maxMoveMax = 2f;

		float maxRotX = 0.3f;
		float maxRotY = 0.4f;
		float maxRotZ = 0.2f;
		float maxRotate = 1f;
		float maxRotateMin = 0.1f;
		float maxRotateMax = 0.35f;

		Dictionary<string, float> shaketimer = new Dictionary<string, float>() {
			{ "mainRotX", 123.45f },
			{ "mainRotY", 44.61f },
			{ "mainRotZ", 77.522f },
			{ "mainPosX", 44.125f },
			{ "mainPosY", 124.4551f },
			{ "mainPosZ", 34.566f },
		};

		public override void Init() {


			enableFeatures.Add("enabled", new JSONStorableBool("Enable ShakyWindow (F2)", true, Toggle));
			RegisterBool(enableFeatures["enabled"]);
			CreateToggle(enableFeatures["enabled"], false);


			sliders.Add("Shakiness", new JSONStorableFloat("Shakiness", 1f, UpdateShakiness, 0f, 10f, false));
			RegisterFloat(sliders["Shakiness"]);
			CreateSlider(sliders["Shakiness"], false);





			InvokeRepeating("UpdateSecond", 1f, 1f);
			//InvokeRepeating("UpdateConstant_p67", 1f, 1f / 30f);
			//InvokeRepeating("UpdateConstant5_p67", 1f, 1f / 5f);
		}

		
		void Start() {
		
		}


		float yRotation = 0f;
		bool yRotationPositive = true;
		float zRotation = 0f;
		bool zRotationPositive = true;

		float xRotationOffset = 0f;
		float yRotationOffset = 0f;
		float zRotationOffset = 0f;
		public float shakeDuration = 5f;
		public float decreaseFactor = 1f;


		float offsetPosMax = 0.1f;
		float offsetPosX = 0f;
		float offsetPosY = 0f;
		float offsetPosZ = 0f;

		float offsetRotMax = 3f;
		float offsetRotX = 0f;
		float offsetRotY = 0f;
		float offsetRotZ = 0f;

		float timeoutRotX = 0f;
		float timeoutRotY = 0f;
		float timeoutRotZ = 0f;

		float GetOffsetPosMax()
        {
			return offsetPosMax;
		}
		float GetOffsetRotMax()
		{
			return offsetRotMax;
		}
		void Randomize()
        {
			var deltaMove = 0.01f;

			
			var move = Random.Range(-deltaMove, deltaMove);
			offsetPosX += move * Time.deltaTime;
			if (offsetPosX > GetOffsetPosMax()) offsetPosX = GetOffsetPosMax();
			if (offsetPosX < -GetOffsetPosMax()) offsetPosX = -GetOffsetPosMax();


			move = Random.Range(-deltaMove, deltaMove);
			offsetPosY += move * Time.deltaTime;
			if (offsetPosY > GetOffsetPosMax()) offsetPosY = GetOffsetPosMax();
			if (offsetPosY < -GetOffsetPosMax()) offsetPosY = -GetOffsetPosMax();

			move = Random.Range(-deltaMove, deltaMove);
			offsetPosZ += move * Time.deltaTime;
			if (offsetPosZ > GetOffsetPosMax()) offsetPosZ = GetOffsetPosMax();
			if (offsetPosZ < -GetOffsetPosMax()) offsetPosZ = -GetOffsetPosMax();

			//List<string> keys = new List<string>(shaketimer.Keys);

			//foreach( var key in keys)
			//         {
			//	if(Random.Range(0f,100f) <= 25f)
			//             {
			//		shaketimer[key] /= 4f;
			//             }
			//         }
		}

		float GetMaxMove()
		{
			return maxMove * sliders["Shakiness"].val;

		}
		float GetMaxRotate()
        {
			return maxRotate *  sliders["Shakiness"].val;

		}
		float GetShakevalue()
        {
			return sliders["Shakiness"].val; 
		}

		float GetShake(string s, float factorTime, float factorShake)
        {
			float shakevalue;

			float asym = Random.Range(0.3f, 1f);
			float asymval = Random.Range(0.8f, 1f);
			shaketimer[s] += Time.deltaTime *  factorTime * asym;
			shakevalue = Mathf.Sin(shaketimer[s]) *  factorShake ;
			return shakevalue;
		}

		void Update() {
			if (Input.GetKeyDown(KeyCode.F2))
			{
				Toggle(!enableFeatures["enabled"].val);
			}
			if (GetShakevalue() == 0f) return;
			if (!enableFeatures["enabled"].val) return;
			SetupForcesRandom();

			Vector3 localEulerAngles = SuperController.singleton.MonitorCenterCamera.transform.localEulerAngles;
			Vector3 localPosition = SuperController.singleton.MonitorCenterCamera.transform.localPosition;

			//if (timeoutRotX <= 0f) localEulerAngles.x = offsetRotX + GetShake("mainRotX", GetMaxRotate(), maxRotX);
			if (timeoutRotY <= 0f) localEulerAngles.y = offsetRotY + GetShake("mainRotY", GetMaxRotate(), maxRotY);
			if (timeoutRotZ <= 0f) localEulerAngles.z = offsetRotZ + GetShake("mainRotZ", GetMaxRotate(), maxRotZ);

            localPosition.x = offsetPosX + GetShake("mainPosX", GetMaxMove(), maxPosX);
            localPosition.z = offsetPosZ + GetShake("mainPosZ", GetMaxMove(), maxPosZ);

            SuperController.singleton.MonitorCenterCamera.transform.localEulerAngles = localEulerAngles;
			SuperController.singleton.MonitorCenterCamera.transform.localPosition = localPosition;

			if (timeoutRotX > 0f) timeoutRotX -= Time.deltaTime;
			if (timeoutRotY > 0f) timeoutRotY -= Time.deltaTime;
			if (timeoutRotZ > 0f) timeoutRotZ -= Time.deltaTime;

			

		}

		void Toggle(bool val)
        {
			enableFeatures["enabled"].val = val;
			if (!val)
            {
				SuperController.singleton.MonitorCenterCamera.transform.localEulerAngles = Vector3.zero;
				SuperController.singleton.MonitorCenterCamera.transform.localPosition = Vector3.zero;
			}
		}
		void FixedUpdate() {
			
		}
		void UpdateSecond()
		{
			if (IsDisabled()) return;
			if (!enableFeatures["enabled"].val) return;
			if (GetShakevalue() == 0f) return;
			if (timeoutRotX <= 0f)
            {
				if (Random.Range(0,100) <= 15) timeoutRotX = Random.Range(0.5f, 2f);
            }
			if (timeoutRotY <= 0f)
			{
				if (Random.Range(0, 100) <= 15) timeoutRotY = Random.Range(0.5f, 2f);
			}
			if (timeoutRotZ <= 0f)
			{
				if (Random.Range(0, 100) <= 15) timeoutRotZ = Random.Range(0.5f, 2f);
			}


		}
		void UpdateConstant()
		{
			if (IsDisabled()) return;

		}
		void UpdateConstant5()
		{
			if (IsDisabled()) return;
		

		}
		void OnDestroy()
		{
			CancelInvoke();
		}

		void SetupForcesRandom()
        {
			Randomize();

		}

		bool IsDisabled()
        {
			if (!enabledJSON.val)
			{
				return true;
			}
			return false;
		}

	
		void UpdateShakiness(float shakiness)
        {
			
		}


	}
}