﻿using System;
using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace SPQR {
	public class OhMyButtons : MVRScript {
		protected Atom receivingAtom;
		private Button currentButton;
		JSONStorableStringChooser jchooser;
		
		protected void SyncAtomChoices() {
			List<string> atomChoices = new List<string>();
			atomChoices.Add("None");

			List<string> choices = new List<string>();
			foreach(var atom in  GetSceneAtoms()){
				if(atom.uid == containingAtom.uid) continue;
				if(atom.type == "UIButton"){
					atomChoices.Add(atom.uid);
				}
			}
			atomJSON.choices = atomChoices;
		}

		protected void SyncAtom(string atomUID) {
			receivingAtom = SuperController.singleton.GetAtomByUid(atomUID);
			
		}
		protected JSONStorableStringChooser atomJSON;



		public override void Init() {
			if(containingAtom.type != "UIButton") return;

			var trigger = this.containingAtom.GetStorableByID("Trigger");
			UIButtonTrigger t = trigger.GetComponent<UIButtonTrigger>();
			currentButton = t.button;

			// make atom selector
				List<string> choices = new List<string>();
				foreach(var atom in  GetSceneAtoms()){
					if(atom.uid == containingAtom.uid) continue;
					if(atom.type == "UIButton"){
						choices.Add(atom.uid);
					}
				}
				atomJSON = new JSONStorableStringChooser("atom", choices, null, "Button", SyncAtom);
				atomJSON.representsAtomUid = true;
				RegisterStringChooser(atomJSON);
				SyncAtomChoices();
				UIDynamicPopup dp = CreateFilterablePopup(atomJSON);
				dp.popupPanelHeight = 500f;
				dp.popup.onOpenPopupHandlers += SyncAtomChoices;

				List<string> choices2 = new List<string>();
				choices2.Add("None");
				choices2.Add("Trigger");
				choices2.Add("TriggerSubButton");
				jchooser = new JSONStorableStringChooser("Action", choices2, "None", "Action", DummyChooserCallback);
				RegisterStringChooser(jchooser);
				UIDynamicPopup udp = CreateFilterablePopup(jchooser,true);
				//CreateScrollablePopup(jchooser, true);
				udp.labelWidth = 300f;

				currentButton.onClick.AddListener(Trigger);
		}
		void Start() {
			
		}

		void Update() {
			
		}
	

		
		void FixedUpdate() {
			
		}

		void OnDisable(){
			currentButton.onClick.RemoveListener(Trigger);
		}
		void OnDestroy() {
			currentButton.onClick.RemoveListener(Trigger);
		}

		void ClickButton(){
			var trigger = containingAtom.GetStorableByID("Trigger");
			UIButtonTrigger t = trigger.GetComponent<UIButtonTrigger>();
			t.button.onClick.Invoke();
		}


		void ClickSubButton(){
			if(receivingAtom!=null){
				var trigger = receivingAtom.GetStorableByID("Trigger");
                UIButtonTrigger t = trigger.GetComponent<UIButtonTrigger>();
                t.button.onClick.Invoke();
			}
		}

		protected void DummyChooserCallback(string s) {
			switch(s){
				case "Trigger":
					ClickButton();
					break;
				case "TriggerSubButton":
					ClickSubButton();
					break;
			}
			jchooser.val = "None";

		}

		void Trigger(){
			if(receivingAtom!=null){
				var trigger = receivingAtom.GetStorableByID("Trigger");
                UIButtonTrigger t = trigger.GetComponent<UIButtonTrigger>();
                t.button.onClick.Invoke();
			}
			
		}

	}
}