﻿using System;
using System.Linq;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace SPQR {
	public class OptimizeClothes : Optimize{

        float defaultIterations = 1f;
        
        Dictionary<string, DAZCharacterSelector> cache;
        public OptimizeClothes(OldVam x) : base(x)
        {
            checkDuration = 5.03f;
            cache = new Dictionary<string, DAZCharacterSelector>() { };
        }

        public override bool CheckChange()
        {
            return true ;
        }

        public override void DoOptimize()
        {
            bool res = false;

            var allatoms_p67 = SuperController.singleton.GetAtoms();
            foreach (var atom in allatoms_p67)
            {
                if (atom.type != "Person") continue;
                DAZCharacterSelector dcs = null;
               
                if (cache.ContainsKey(atom.uid) && cache[atom.uid] != null)
                {
                    dcs = cache[atom.uid];
                }
                else
                {
                    dcs = atom.GetComponentInChildren<DAZCharacterSelector>();
                }
                if (dcs == null) continue;
                //cache[atom.uid] = dcs;

                var pjson = dcs.GetJSON(true, true);
                for (int i = 0; i < pjson["clothing"].Count; i++)
                {
                    var storable = pjson["clothing"][i]["internalId"].ToString();
                    if(storable == "" || storable == null)
                    {
                        storable = pjson["clothing"][i]["id"].ToString().Replace(" ", "");
                        storable = storable.Replace("\"", "");
                    }


                    JSONStorable t = atom.GetStorableByID(storable + "Sim");
                    if (t == null) continue;
                    // iterations
                    var s1 = t.GetFloatJSONParam("iterations");
                    if (s1.val != defaultIterations)
                    {
                        res = true;
                        s1.val = defaultIterations;
                    }


                }
            }

            if (res)
            {
                Log("Optimized clothing items");
            }
        }






    }
}