﻿using System;
using System.Linq;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace SPQR {
	public class OptimizeHair : Optimize{

        float defaultHairWidth = 0.00022f;
        int defaultCurveDensity = 13;
        int defaultHairMultiplier = 13;
        float defaultIterations = 1f;

        Dictionary<string, DAZCharacterSelector> cache;
        public OptimizeHair(OldVam x) : base(x)
        {
            checkDuration = 4.69f;
            cache = new Dictionary<string, DAZCharacterSelector>() { };
        }

        public override bool CheckChange()
        {
            return true ;
        }

        public override void DoOptimize()
        {
            bool res = false;

            var allatoms_p67 = SuperController.singleton.GetAtoms();
            foreach (var atom in allatoms_p67)
            {
                if (atom.type != "Person") continue;
                DAZCharacterSelector dcs = null;
               
                if (cache.ContainsKey(atom.uid) && cache[atom.uid] != null)
                {
                    dcs = cache[atom.uid];
                }
                else
                {
                    dcs = atom.GetComponentInChildren<DAZCharacterSelector>();
                }
                if (dcs == null) continue;
                //cache[atom.uid] = dcs;

                var pjson = dcs.GetJSON(true, true);
                for (int i = 0; i < pjson["hair"].Count; i++)
                {
                    var storable = pjson["hair"][i]["internalId"];


                    JSONStorable t = atom.GetStorableByID(storable + "Sim");

                    //hair width
                    var s1 = t.GetFloatJSONParam("width");
                    if (s1.val != defaultHairWidth)
                    {
                        res = true;
                        s1.val = defaultHairWidth;
                    }
                    //hair curveDensity
                    var s2 = t.GetFloatJSONParam("curveDensity");
                    if (s2.val != defaultCurveDensity)
                    {
                        res = true;
                        s2.val = defaultCurveDensity;
                    }
                    //hair hairMultiplier
                    var s3 = t.GetFloatJSONParam("hairMultiplier");
                    if (s3.val != defaultHairMultiplier)
                    {
                        res = true;
                        s3.val = defaultHairMultiplier;
                    }
                    //hair iterations
                    var s4 = t.GetFloatJSONParam("iterations");
                    if (s4.val != defaultIterations)
                    {
                        res = true;
                        s4.val = defaultIterations;
                    }


                }
            }

            if (res)
            {
                Log("Optimized hair items");
            }
        }






    }
}