﻿using System;
using System.Linq;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace SPQR {
	public class OptimizeLights : Optimize{
        Atom atomLight;
        bool initiatedAtomLight = false;
        Rigidbody centerEye;
        float cooldownAdd = 0f;
        public OptimizeLights(OldVam x) : base(x)
        {
            checkDuration = 5f;
            centerEye = SuperController.singleton.GetAtomByUid("[CameraRig]").rigidbodies.First(rb => rb.name == "CenterEye");
        }

        public override bool CheckChange()
        {
            atomLight = SPQRUtils.GetAtomById_p67("oldvamLight");
            if (atomLight == null)
            {
                if (Time.time - cooldownAdd < 3f) return false;
                cooldownAdd = Time.time;
                //SuperController.LogMessage("add Atom");
                X.StartCoroutine(X.CoroutineCreateAtom_p67("InvisibleLight", "oldvamLight"));
                initiatedAtomLight = false;
                return false;
            }
            if (!initiatedAtomLight) InitiateAtomLight();
            return true;
        }

        public override void DoOptimize()
        {
            bool res = TurnOffAllOtherLights();
            if (res)
            {
                Log("Optimized Lights.");
            }
        }

        private void InitiateAtomLight()
        {
            initiatedAtomLight = true;
            atomLight.hidden = true;
            atomLight.SetOn(true);
            atomLight.enabled = true;
            SPQRUtils.SetAtomGizmo_p67(atomLight, false);
            SPQRUtils.setAtomStorableBool_p67("oldvamLight", "Light", "shadowsOn", false);
            SPQRUtils.SetAtomStorableColor_p67("oldvamLight", "Light", "color", HSVColorPicker.RGBToHSV(1f, 1f, 1f));
            SPQRUtils.SetAtomStorableFloat_p67("oldvamLight", "Light", "range", 25f);
            SPQRUtils.SetAtomStorableFloat_p67("oldvamLight", "Light", "intensity", 1.1f);
            SPQRUtils.setAtomStorableStringChooser_p67("oldvamLight", "Light", "type", "Point");
        }

        public void Update()
        {
            if (atomLight == null) return;

            atomLight.mainController.transform.position =
                   centerEye.transform.position +
                   centerEye.transform.forward * -0.1f +
                   centerEye.transform.up * 0.05f;

            atomLight.mainController.transform.rotation = centerEye.transform.rotation;

        }


        public bool TurnOffAllOtherLights()
        {
            var res = false;
            var allatoms_p67 = SuperController.singleton.GetAtoms();
            foreach (var a in allatoms_p67)
            {
                if (a.type != "InvisibleLight") continue;
                if (a.uid.StartsWith("AEMainMenuLights")) continue;
                if (a.uid.StartsWith("oldvamLight")) continue;
                if (a.on)
                {
                    a.SetOn(false);
                    res = true;
                }
            }
            return res;
        }


    }
}