﻿using System;
using System.Linq;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using UnityEngine.UI;
using MeshVR;

namespace SPQR {
	public class MemoryHeapInfo : MVRScript {

		long lastCount, currentCount = 0;

        protected Canvas dynCanvas;
        protected Text dynText;

        GameObject background3;
        Image backgroundImage3;
        RectTransform bgRect3;

        GameObject mainGameObject;

        DateTime _lastTimeFPS;
        int _framesRenderedFPS;
        public int _fps, _fps10;
        private List<int> _fpsHistory = new List<int>();
        private int _averageFpsInterval = 10;

        Canvas testCanvas;


        public void printParent(GameObject a)
        {
            if (a.transform.parent == null) return;
            SuperController.LogMessage("parent: <" + a.transform.parent.name +"> ");
            printParent(a.transform.parent.gameObject);
        }

        public override void Init() {

			InvokeRepeating("UpdateSecond", 1f, 1f);
            lastCount = (long)Math.Round((double)System.GC.GetTotalMemory(false) / 1000);
            SuperController.LogMessage("SPQR.MemoryHeapInfo: Forcing garbage collector...");
            System.GC.Collect();
            SuperController.LogMessage("SPQR.MemoryHeapInfo: Initialized!");

            try
            {

                Initiate();



            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }
        void OnDestroy()
        {
            DestroyObjects();
        }

        void OnDisable()
        {
            Update();
        }


        void Initiate()
        {
            Canvas[] canvases = FindObjectsOfType<Canvas>();
            foreach (Canvas obj in canvases)
            {
                if (obj.name == "Scene Control Canvas")
                {
                    testCanvas = obj;
                }
                //else SuperController.LogMessage("name:" + obj.name);
            }


            //var allButtons = UnityEngine.Object.FindObjectsOfType<UnityEngine.Object>();
            //foreach (var button in allButtons)
            //{
            //    var text = button as Text;
            //    if (text != null)
            //    {
            //        if (text.text.StartsWith("Play Mode"))
            //        {
            //            SuperController.LogMessage(text.name + "  = " + text.text);
            //            printParent(text.gameObject);

            //        }
            //    }
            //}

            if (testCanvas == null) return;

            DestroyObjects();




            //GameObject rectangleObject = new GameObject("CustomMemoryHeapTestRect");
            //Image rectangleImage = rectangleObject.AddComponent<Image>();
            //rectangleImage.color = Color.red;
            //// Set the size of the rectangle to 5x5 pixels
            //RectTransform rectTransform = rectangleObject.GetComponent<RectTransform>();
            //rectTransform.sizeDelta = new Vector2(500, 500);
            //// Set the position of the rectangle to the center of the Canvas
            //RectTransform canvasRect = GetComponent<RectTransform>();
            //rectTransform.SetParent(testCanvas.transform, false);
            //rectTransform.anchoredPosition = Vector2.zero;



            mainGameObject = new GameObject("CustomMemoryHeapCanvas");

            GraphicRaycaster gr = mainGameObject.AddComponent<GraphicRaycaster>();
            RectTransform rt = mainGameObject.GetComponent<RectTransform>();
            rt.anchoredPosition = new Vector2(-340f, -27f);

            //g.transform.localPosition = new Vector3(10f, 0f, 0f);

            // anchor to head for HUD effect
            Transform headCenter = SuperController.singleton.centerCameraTarget.transform;
            rt.SetParent(testCanvas.transform, false);


            GameObject background = new GameObject();
            background.name = "CustomMemoryHeapCanvasBackground";
            background.transform.parent = mainGameObject.transform;
            background.transform.localScale = Vector3.one;
            background.transform.localPosition = Vector3.zero;
            background.transform.localRotation = Quaternion.identity;

            Image backgroundImage = background.AddComponent<Image>();
            RectTransform bgRect = background.GetComponent<RectTransform>();
            bgRect.SetSizeWithCurrentAnchors(RectTransform.Axis.Horizontal, 400f);
            bgRect.SetSizeWithCurrentAnchors(RectTransform.Axis.Vertical, 56f);
            backgroundImage.color = new Color(34f / 255f, 50f / 255f, 82f / 255f, 1f);


            GameObject background2 = new GameObject();
            background2.name = "CustomMemoryHeapCanvasLine";
            background2.transform.parent = mainGameObject.transform;
            background2.transform.localScale = Vector3.one * 2f;
            background2.transform.localPosition = new Vector3(0f, -12f, 1f);
            background2.transform.localRotation = Quaternion.identity;

            Image backgroundImage2 = background2.AddComponent<Image>();
            RectTransform bgRect2 = background2.GetComponent<RectTransform>();
            bgRect2.SetSizeWithCurrentAnchors(RectTransform.Axis.Horizontal, 150f);
            bgRect2.SetSizeWithCurrentAnchors(RectTransform.Axis.Vertical, 5f);
            backgroundImage2.color = new Color(0.33f, 0.33f, 0.33f);




            background3 = new GameObject();
            background3.name = "CustomMemoryHeapCanvasLine2";
            background3.transform.parent = background2.transform;
            background3.transform.localScale = Vector3.one;

            // Set local position to the left side (adjust the x-coordinate as needed)
            background3.transform.localPosition = new Vector3(0f, 0f, 0f);

            background3.transform.localRotation = Quaternion.identity;

            backgroundImage3 = background3.AddComponent<Image>();
            bgRect3 = background3.GetComponent<RectTransform>();

            // Set the size of the red bar (50% of the width of background2)
            bgRect3.SetSizeWithCurrentAnchors(RectTransform.Axis.Horizontal, 0.25f * 150f);
            bgRect3.SetSizeWithCurrentAnchors(RectTransform.Axis.Vertical, 5f);
            backgroundImage3.color = new Color(1f, 0.33f, 0.33f);

            // Set pivot and anchor to left to align the red bar to the left
            bgRect3.pivot = new Vector2(0f, 0.5f);
            bgRect3.anchorMin = new Vector2(0f, 0.5f);
            bgRect3.anchorMax = new Vector2(0f, 0.5f);



            GameObject g2 = new GameObject();
            g2.name = "CustomMemoryHeapCanvasText";
            g2.transform.parent = mainGameObject.transform;
            g2.transform.localScale = Vector3.one * 2f;
            g2.transform.localPosition = new Vector3(0f, 12f, 0f);
            g2.transform.localRotation = Quaternion.identity;
            Text t = g2.AddComponent<Text>();
            RectTransform rt2 = g2.GetComponent<RectTransform>();
            rt2.SetSizeWithCurrentAnchors(RectTransform.Axis.Horizontal, 250f);
            rt2.SetSizeWithCurrentAnchors(RectTransform.Axis.Vertical, 40f);
            t.alignment = TextAnchor.MiddleCenter;
            t.horizontalOverflow = HorizontalWrapMode.Overflow;
            t.verticalOverflow = VerticalWrapMode.Overflow;
            Font ArialFont = (Font)Resources.GetBuiltinResource(typeof(Font), "Arial.ttf");
            t.font = ArialFont;
            t.fontSize = 12;
            t.text = "";
            t.enabled = true;
            t.color = Color.white;
            dynText = t;

        }
        bool IsDisabled()
        {
            if (!enabledJSON.val) return true;
            if (SuperController.singleton.topWorldUI.gameObject.activeInHierarchy) return true;
            if (((SuperController.singleton.hiResScreenshotPreview.gameObject.activeInHierarchy && SuperController.singleton.hiResScreenshotCamera.enabled) || SuperController.singleton.screenshotCamera.enabled)) return true;
            return false;
        }

        float GetPercent(float val)
        {
            val = Mathf.Clamp(val, 0f, 1024f);
            return (val / 1024f);
        }
        void SetLinePercent(float percent = 0.5f)
        {
            Color green = Color.green;
            Color yellow = Color.yellow;
            Color red = Color.red;
            Color interpolatedColor = Color.Lerp(green, yellow, percent * 2f);

            if (percent > 0.5f)
            {
                interpolatedColor = Color.Lerp(yellow, red, (percent - 0.5f) * 2f);
            }
            bgRect3.SetSizeWithCurrentAnchors(RectTransform.Axis.Horizontal, percent * 150f);
            backgroundImage3.color = interpolatedColor;
        }
        void DestroyObjects()
        {
            GameObject[] allGameObjects = FindObjectsOfType<GameObject>();
            foreach (GameObject obj in allGameObjects)
            {
                if (obj.name.StartsWith("CustomMemoryHeap")) Destroy(obj);

               
                //var comp = obj.GetComponent<Text>();
                //if (comp != null)
                //{
                //    SuperController.LogMessage(" "+comp.text);

                //    if (comp.text == "Play Mode (P)") comp.text = "PLAY";
                //}

            }
        }

        void Start() {
		
		}

       

	
		void Update() {
            if (testCanvas == null)
            {
                Initiate();
            }
            if (mainGameObject == null) return;
           
            if (mainGameObject.activeSelf)
            {
                if (IsDisabled())
                {
                    mainGameObject.SetActive(false);
                    return;
                }
            } else
            {
                if (!IsDisabled()) mainGameObject.SetActive(true);
            }

            _framesRenderedFPS++;

            if ((DateTime.Now - _lastTimeFPS).TotalSeconds >= 1)
            {
                _fps = _framesRenderedFPS;
                _framesRenderedFPS = 0;
                _lastTimeFPS = DateTime.Now;
                _fpsHistory.Add(_fps);
                while (_fpsHistory.Count > _averageFpsInterval)
                {
                    _fpsHistory.RemoveAt(0);
                }
                _fps10 = Mathf.RoundToInt(CalculateAverageFPS(_fpsHistory));
             
            }

        }

        int CalculateAverageFPS(List<int> fpsHistory)
        {
            if (fpsHistory.Count == 0)
            {
                return 0;
            }

            int sum = 0;
            foreach (int fps in fpsHistory)
            {
                sum += fps;
            }

            return Mathf.RoundToInt(sum / fpsHistory.Count);
        }

        void FixedUpdate() {
		
		}
		int i = 0;
		void UpdateSecond()
		{
			i = (i+1) % 3;
			currentCount = (long)Math.Round((double)System.GC.GetTotalMemory(false) / 1000);
            var diffCount = currentCount - lastCount;
			lastCount = currentCount;
            //if (i == 0) 
            //SuperController.singleton.ClearMessages();
            //SuperController.LogMessage("Memory (F8): " + currentCount / 1000 + " MB ("+(diffCount>0f?"+":"")+diffCount+" KB)" ,false);

            var percent = GetPercent(diffCount);
            SetLinePercent(percent);
            dynText.text = "" + currentCount / 1000 + " MB |" + (diffCount > 0f ? "+" : "") + diffCount + " KB | "+ _fps10 + " FPS";

        }
	
	}
}