﻿using System;
using System.Linq;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace SPQR {
	public class PersonalSpace : MVRScript {

		bool isDebugMode = false;
        float deltaDistance = 1f;
		public override void Init() {
			SuperController.singleton.onAtomAddedHandlers += OnAtomAdded;
        }

	
		void OnDestroy()
		{
            SuperController.singleton.onAtomAddedHandlers -= OnAtomAdded;
        }

	
		bool IsDisabled()
        {
			if (!enabledJSON.val)
			{
				return true;
			}
			return false;
		}

		void OnAtomAdded(Atom atom)
		{
			if (IsDisabled()) return;
			if (atom.type != "Person") return;

			List<Atom> people = new List<Atom>{ };
			var sceneAtoms = SuperController.singleton.GetAtoms();
			foreach(var sceneAtom in sceneAtoms)
			{
				if (atom.GetInstanceID() == sceneAtom.GetInstanceID()) continue;
				if(sceneAtom.type == "Person")
				{
					people.Add(sceneAtom);
				}
			}
			var originalCollision = atom.collisionEnabled;
            atom.collisionEnabled = false;
			atom.enabled = false;
            PositionAtomPersonalSpace(atom, people, originalCollision);
		}
		

        void PositionAtomPersonalSpace(Atom atom, List<Atom> people, bool originalCollision, int iteration = 0, int offsetX = 0, int offsetZ = 0)
		{
			if(iteration > 500)
			{
                FinishPersonalSpace(atom, originalCollision);
                SuperController.LogMessage("PersonalSpace plugin: Couldn't position Person, limit reached, stopping");
				return;
			}
			var position = atom.mainController.transform.position + atom.mainController.transform.forward * offsetZ* deltaDistance + atom.mainController.transform.right * offsetX* deltaDistance;
			var clear = true;
			foreach(var person in people)
			{
				var distance = Vector3.Distance(person.mainController.transform.position, position);
				if (distance < deltaDistance)
				{
					clear = false;
					break;
				}
            }
			if (clear)
			{
				atom.mainController.transform.position = position;
				FinishPersonalSpace(atom, originalCollision);
			} else
			{
				int newOffsetX = offsetX + (UnityEngine.Random.Range(0, 2) == 0 ? -1 : 1);
                int newOffsetZ = offsetZ + (UnityEngine.Random.Range(0, 2) == 0 ? -1 : 1);
                PositionAtomPersonalSpace(atom, people, originalCollision,++iteration, newOffsetX, newOffsetZ);
            }

		}

		void FinishPersonalSpace(Atom atom, bool originalCollision)
		{
            atom.enabled = true;
            StartCoroutine(EnableCollisionPersonalSpaceDelayed(atom, originalCollision, 1f));
		}
        IEnumerator EnableCollisionPersonalSpaceDelayed(Atom a, bool originalCollision, float delay)
        {
            yield return new WaitForSeconds(delay);
			a.collisionEnabled = originalCollision;
			
        }

    }
}