﻿using System;
using UnityEngine;
using System.Text;
using System.Linq;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using MVR.FileManagementSecure;
using MacGruber;

namespace SPQR
{
    public class MorphBlend : MVRScript {

		Dictionary<string, float>  morphsList1;
		Dictionary<string, float>  morphsList2;
		Dictionary<string, int>  morphsIds;
		
		
		private const int countSliders = 10;
		JSONStorableFloat jfloat;
		JSONStorableString[] sliderName;
		JSONStorableFloat[] sliderValue;        
		JSONStorableFloat[] sliderStart;
		JSONStorableFloat[] sliderEnd;
		UIDynamicSlider[] dsliders;
		UIDynamicSlider[] dsliders2;
		UIDynamicSlider dslider;
		JSONStorableString jstring3;
		
		int titles = 0;

		public override void Init() {
			
		    if (containingAtom.type != "Person"){
				 CreateTitle("Must be added to a Person",28,false);
				return;
			}
			
			
			
			sliderName = new JSONStorableString[countSliders];
			sliderValue = new JSONStorableFloat[countSliders];
			sliderStart = new JSONStorableFloat[countSliders];
			sliderEnd = new JSONStorableFloat[countSliders];
			dsliders = new UIDynamicSlider[countSliders];
			dsliders2 = new UIDynamicSlider[countSliders];
			morphsList1 = new Dictionary<string, float>(){};
			morphsList2 = new Dictionary<string, float>(){};
			morphsIds = new Dictionary<string, int>(){};

			
			Utils.OnInitUI(CreateUIElement);
			
            CreateTitle("MorphBlend v1",50,false);
			Utils.SetupSpacer(this, 100, true);
						
			JSONStorableString jstring = new JSONStorableString("instructions2", "\n This plugin allows setting a group of morphs with start and end values and adds a blend slider that controls the blending of those morphs.");
            UIDynamicTextField dtext = CreateTextField(jstring);
            dtext.UItext.supportRichText = true;
            dtext.backgroundColor = new Color(0.42f, 0.03f, 0.51f, 0.7f);
            dtext.textColor = new Color(1f, 1f, 1f, 0.9f);
            dtext.height = 350f;

			
			
			
            JSONStorableString jstring2 = new JSONStorableString("instructions1", "\n Morph uid must be the internal name the morph. You can click Make csv to generate a csv in \\Saves\\PluginData\\MorphBlend with the available morphs, the first column is the uid. It's better to use default (not in a package) morphs if you want to share your own package to make sure it works for everyone. Otherwise if you use custom morphs, you will need to have them as dependencies. You can do that quickly automatically by making sure you include a scene where you have a model where those morphs are not 0.00. If they have a value, VAM will detect them as being used and will add them as dependencies. They will not be added just by setting the name (uid) alone in this plugin.");
            UIDynamicTextField dtext2 = CreateTextField(jstring2,true);
            dtext2.UItext.supportRichText = true;
            dtext2.backgroundColor = new Color(0.42f, 0.03f, 0.51f, 0.7f);
            dtext2.textColor = new Color(1f, 1f, 1f, 0.9f);
            dtext2.height = 350f;
			
			
			
		    Utils.SetupSpacer(this, 70, false);
            Utils.SetupSpacer(this, 70, true);
			
			
			CreateTitle("Morphs to Blend",50,false);
			
			for(var i = 0; i < countSliders; i++){
				
				CreateTitle("Morph "+(i+1),30,false);
				
				sliderName[i] = new JSONStorableString("Morph"+(i+1), "", UpdateVarValue);
				RegisterString(sliderName[i]);
				UIDynamicLabelInput myStringInput = Utils.SetupTextInput(this, "Morph uid "+(i+1), sliderName[i], false);
				
				sliderValue[i] = new JSONStorableFloat("Preview value:", 0f, UpdateSlider, -1f, 1f, false, false);
				dsliders[i] = CreateSlider(sliderValue[i],false);

				sliderStart[i] = new JSONStorableFloat("Start"+(i+1), 0f, UpdateSlider, -1f, 1f, false);
				RegisterFloat(sliderStart[i]);
				dsliders[i] = CreateSlider(sliderStart[i],false);
				
				sliderEnd[i] = new JSONStorableFloat("End"+(i+1), 1f, UpdateSlider, -1f, 1f, false);
				RegisterFloat(sliderEnd[i]);
				dsliders2[i] = CreateSlider(sliderEnd[i],false);
				
		
				
			
			}
			
			CreateTitle("Blend percent",50,true);
		
			jfloat = new JSONStorableFloat("Blend", 0f, BlendCallback, 0f, 1f, true);
			RegisterFloat(jfloat);
			UIDynamicSlider dslider = CreateSlider(jfloat,true);
			
			CreateTitle("Helpers",50,true);
		    Utils.SetupButton(this, "Make morphs .csv", WriteDebugList, true);
		
			jstring3 = new JSONStorableString("instructions3", "");
            UIDynamicTextField dtext3 = CreateTextField(jstring3,true);
            dtext3.height = 150f;
		
			Utils.SetupAction(this, "ActionClearAll", ActionClearAll);			

			
			
		}

		void Start() {
		}
		void Update() {
		
		}
		void FixedUpdate() {
			
		}
        void OnDestroy() {
		}


		//--------------------------------------//

		void PopulateMorphs(){  
			morphsList1 = new Dictionary<string, float>(){};
			morphsList2 = new Dictionary<string, float>(){};
			morphsIds = new Dictionary<string, int>(){};
			
			for (var i = 0; i < countSliders; i++){
				var name = sliderName[i].val.Replace("\n", "").Replace("\r", "").Replace(",", "");
				sliderName[i].val = name;
				if(sliderName[i].val.Length < 0 ) continue;
				if(morphsList1.ContainsKey(sliderName[i].val) && morphsList2.ContainsKey(sliderName[i].val)) continue;
				morphsIds.Add(sliderName[i].val,i);
				morphsList1.Add(sliderName[i].val, sliderStart[i].val);
				morphsList2.Add(sliderName[i].val, sliderEnd[i].val);
			}
		}

		void UpdateSlider(JSONStorableFloat jf){
			PopulateMorphs();
			BlendCallback(jfloat);
		}
		void BlendCallback(JSONStorableFloat jf){
			GenerateDAZMorphsControlUI morphControl = returnMorphs();
            morphControl.GetMorphs().ForEach((morph) =>{
                if(morphsList1.ContainsKey(morph.uid)){
					morph.morphValue = Mathf.Lerp(morphsList1[morph.uid], morphsList2[morph.uid],jf.val);
					sliderValue[morphsIds[morph.uid]].val = morph.morphValue;
				}
			});
		}
		
		void ActionClearAll(){
			for(var i = 0; i < countSliders; i++){
				sliderName[i].val = "";
				sliderValue[i].val = 0f;
				sliderStart[i].val = 0f;
				sliderEnd[i].val = 1f;
			}
		}
			
		protected void UpdateVarValue(string s){ PopulateMorphs();}
		protected void OperationUpdate(string s){PopulateMorphs();}
		

		GenerateDAZMorphsControlUI returnMorphs() {
			JSONStorable geometry = containingAtom.GetStorableByID("geometry");
			DAZCharacterSelector character = geometry as DAZCharacterSelector;
			return character.morphsControlUI;
		}

		void WriteDebugList(){
			jstring3.val = "Preparing csv...";
			GenerateDAZMorphsControlUI morphControl = returnMorphs();
			string debugList =
				"uid,"+
				"resolvedDisplayName,"+
				"resolvedRegionName,"+
				"morphName,"+
				"displayName,"+
				"region,"+
				"group,"+
				"active,"+
				"isInPackage,"+
				"license,"+
				"packageUid,"+
				"metaJSONFile,"+
				"deltasLoadPath,"+
				"version,"+
				"isTransient,"+
				"activeImmediate,"+
				"isPoseControl,"+
				"isRuntime"+
				"\r\n";
				
            morphControl.GetMorphs().ForEach((morph) =>{
				
				debugList+= 
				    morph.uid+","+
					morph.resolvedDisplayName+","+
					morph.resolvedRegionName+","+
					morph.morphName+","+
					morph.displayName+","+
					morph.region+","+
					morph.group+","+
					(morph.active ? "active" : "inactive" )+ ","+
					(morph.isInPackage ? "in package" : "no package" )+ ","+
					morph.packageLicense+","+
					morph.packageUid+","+
					morph.metaJSONFile+","+
					morph.deltasLoadPath+","+
					morph.version+","+
					morph.isTransient+","+
					morph.activeImmediate+","+
					morph.isPoseControl+","+
					morph.isRuntime+""+
					"\r\n";
			});
			var csvPath = @"Saves\PluginData\MorphBlend\";
			if (!FileManagerSecure.DirectoryExists(csvPath)) FileManagerSecure.CreateDirectory(csvPath);
			FileManagerSecure.WriteAllText(csvPath+"morphs.csv",debugList);
			
			jstring3.val+= "\r\nDone! Saved in: "+csvPath+"morphs.csv";
			
		}
		
		private void CreateTitle(string text, int fontsize, bool right,int spacing){
            CreateTitle(text, fontsize, right);
            Utils.SetupSpacer(this, spacing, right);
        }
        private void CreateTitle(string text, int fontsize, bool right){
                JSONStorableString title1 = new JSONStorableString("title"+(++titles), "\n<size="+fontsize+"><b>"+text+"</b></size>");
                UIDynamicTextField title1text = CreateTextField(title1, right);
                title1text.UItext.supportRichText = true;
                title1text.backgroundColor = new Color(0f, 0f, 0f, 0f);
                title1text.textColor = new Color(0.42f, 0.03f, 0.51f, 1f);
                title1text.height = 100f;
                title1text.UItext.alignment = right ? TextAnchor.MiddleRight : TextAnchor.MiddleLeft;
        }


	}
}