﻿using System;
using System.Linq;
using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using MVR.FileManagementSecure;


/*
 

CHANGELOG V2

FIX: prevent error messages appearing while files is locked for writing and vam fails to load it
NEW: added autoplay toggle, on by default; when it's off the plugin will just recheck and refresh the file in vam, not play it, to allow for custom play (maybe paired with APICheck plugin)
NEW: added queue toggle, allows for files to be queued; can be used with multiple files by changing the filename from a trigger, like "folder/reply2.wav", "folder/reply3.wav"



*/

namespace SPQR {
	public class SoundRefresh : MVRScript {
		
		bool sceneLoaded = false;
		DateTime lastTime;
		JSONStorableString filter;
        JSONStorableBool isAutoPlay;
        JSONStorableBool isQueue;

        public override void Init() {
			if(containingAtom.type != "Person")
            {
				SuperController.LogError("SoundRefresh plugin must be added to a Person atom");
				return;
            }
			InitUI();
			InvokeRepeating("UpdateSecond", 1f, 1f);
			InvokeRepeating("UpdateConstant", 1f, 1f / 30f);
			InvokeRepeating("UpdateConstant5", 1f, 1f / 5f);
			SuperController.singleton.onSceneLoadedHandlers += OnSceneLoaded;
		}
		void InitUI()
        {
			var path = "Custom\\Assets\\Audio\\RT_Lipsync\\latest.wav";
			//path = "SPQR.TextAudioTool\\files\\tts-local-latest.wav";
            filter = new JSONStorableString("Filter", path, UpdatedFilter);
            var t = CreateTextInput_p67(this, null, filter, false)[0];
            RegisterString(filter);


            isAutoPlay = new JSONStorableBool("Autoplay", true);
            isAutoPlay.storeType = JSONStorableParam.StoreType.Full;
            RegisterBool(isAutoPlay);
            CreateToggle(isAutoPlay, false);


            isQueue = new JSONStorableBool("Queue", false);
            isQueue.storeType = JSONStorableParam.StoreType.Full;
            RegisterBool(isQueue);
            CreateToggle(isQueue, false);


            var btn1 = CreateButton("Play", false);
			btn1.button.onClick.AddListener(RefreshSounds);

			var act1 = new JSONStorableAction("Play", RefreshSounds);
			RegisterAction(act1);
		}

		void Start() {
			UpdateDateTime();
		}

	
		void Update() {
		
		}

	
		void FixedUpdate() {
		
		}
		void UpdateSecond()
		{
			CheckFileChange();
			if (IsDisabled()) return;

		}
		void UpdateConstant()
		{
			if (IsDisabled()) return;

		}
		void UpdateConstant5()
		{
			if (IsDisabled()) return;

		}
		void OnDestroy()
		{
			CancelInvoke();
			SuperController.singleton.onSceneLoadedHandlers -= OnSceneLoaded;
		}

		void OnSceneLoaded()
		{
			sceneLoaded = true;
		}
		bool IsDisabled()
        {
			if (!enabledJSON.val)
			{
				return true;
			}
			return false;
		}
		void UpdatedFilter(string s)
        {
			UpdateDateTime();

		}
		bool FileIsOk()
        {
			if (filter.val == "") return false;
			return FileManagerSecure.FileExists(filter.val);

		}

		void CheckFileChange()
        {
			if (!FileIsOk()) return;
			try
			{
				var currentTime = FileManagerSecure.FileLastWriteTime(filter.val);
				//SuperController.LogMessage(lastTime.ToString() + " vs " + currentTime.ToString() + " = " + DateTime.Compare(lastTime, currentTime));
				if (DateTime.Compare(lastTime, currentTime) < 0)
				{
					//SuperController.LogMessage(" CHANGE at " + currentTime.ToString());
					RefreshSounds();
					UpdateDateTime();
				}
			} catch (Exception e)
			{

			}
		}
		void UpdateDateTime()
        {
			if (!FileIsOk()) return;
			lastTime = FileManagerSecure.FileLastWriteTime(filter.val);
        }
		public static UIDynamicTextField[] CreateTextInput_p67(MVRScript X, JSONStorableString labeljss, JSONStorableString s, bool rightSide = false)
		{
			var response = new UIDynamicTextField[2];
			UIDynamicTextField dtext;
			if (labeljss != null)
			{
				dtext = X.CreateTextField(labeljss, rightSide);
				dtext.height = 5f;
				dtext.backgroundColor = Color.clear;
				response[1] = dtext;
			}

			var textfield = X.CreateTextField(s, rightSide);
			textfield.backgroundColor = Color.white;
			var input = textfield.gameObject.AddComponent<InputField>();
			input.textComponent = textfield.UItext;
			s.inputField = input;


			response[0] = textfield;

			return response;
		}

		void RefreshSounds()
        {
			string localPath = SuperController.singleton.NormalizeLoadPath(filter.val);
			if (!FileManagerSecure.FileExists(localPath)) return;

			var namesplit = localPath.Split('/');
			var name = "\\"+namesplit[namesplit.Length - 1];

			var j = URLAudioClipManager.singleton.GetJSON();
			foreach (var k in j["clips"].AsArray)
			{
				var clip = (JSONNode)k;

				if(clip["url"].Value  == localPath)
                {
					var nac = URLAudioClipManager.singleton.GetClip(clip["displayName"]);
					URLAudioClipManager.singleton.RemoveClip(nac);
				}
			}
			if (!isAutoPlay.val) return;

			var raeAudioSource = containingAtom.GetStorableByID("HeadAudioSource") as AudioSourceControl;
			var qclip = URLAudioClipManager.singleton.QueueClip(localPath, name);
			var nacclip = URLAudioClipManager.singleton.GetClip(localPath);
			if(nacclip.sourceClip == null)
            {
				RaeSoundDelayed_p67(localPath,raeAudioSource,true,0);
			} else
            {
				if (!isQueue.val)
				{
                    raeAudioSource.audioSource.clip = nacclip.sourceClip;
                    raeAudioSource.audioSource.Play();
				} else
				{
					raeAudioSource.QueueClip(nacclip);
                }
			}
		

		}

		public void RaeSoundDelayed_p67(string localPath, AudioSourceControl raeAudioSource, bool force, int ite)
		{
			if (ite > 300)
			{
				SuperController.LogMessage("SoundRefresh expired - couldn't load "+filter.val);
				return;
			}

			var nac = URLAudioClipManager.singleton.GetClip(localPath);
			if (nac.sourceClip == null)
			{
				StartCoroutine(InvokeRoutine_p67(() => RaeSoundDelayed_p67(localPath, raeAudioSource, force, ite + 1), 0.05f));
				return;
			}

            if (!isQueue.val)
			{
                raeAudioSource.audioSource.clip = nac.sourceClip;
                if (force)
                {
                    raeAudioSource.StopAndClearQueue();
                }
                raeAudioSource.audioSource.Play();
            } else
			{
				raeAudioSource.QueueClip(nac);

            }
            
			
		}

		IEnumerator InvokeRoutine_p67(System.Action f, float delay)
		{
			yield return new WaitForSeconds(delay);
			f();
		}

	}
}