﻿using System;
using System.Linq;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace SPQR {
	public class Teleporter : MVRScript {
		
		bool sceneLoaded = false;

        protected JSONStorableStringChooser atomJSON;
		protected JSONStorableFloat distanceJSON;
		protected float previousDistance = 0f;
		public bool isTeleported = false;
        public override void Init() {


            atomJSON = new JSONStorableStringChooser("atom", SuperController.singleton.GetAtomUIDs(), null, "Destination", SyncAtom);
            atomJSON.representsAtomUid = true;
            RegisterStringChooser(atomJSON);
            SyncAtomChoices();
            UIDynamicPopup dp = CreateFilterablePopup(atomJSON);
            dp.popupPanelHeight = 1100f;
            dp.popup.onOpenPopupHandlers += SyncAtomChoices;


            distanceJSON = new JSONStorableFloat("Distance", 2f,  0f, 5f, false);
            RegisterFloat(distanceJSON);
            UIDynamicSlider dslider = CreateSlider(distanceJSON);


            InvokeRepeating("UpdateSecond", 1f, 1f);
			InvokeRepeating("UpdateConstant", 1f, 1f / 30f);
			InvokeRepeating("UpdateConstant5", 1f, 1f / 5f);
			SuperController.singleton.onSceneLoadedHandlers += OnSceneLoaded;
		}

	
		void Start() {
		
		}

	
		void Update() {
			try
			{
				var distance = GetTransformDistanceToCamera_p67(containingAtom.mainController.transform);
				if (previousDistance == 0f) previousDistance = distance;

				
            

                if (distance <= distanceJSON.val)
				{
                    if (Mathf.Abs(previousDistance - distance) > 4f)
                    {
                        isTeleported = true;
                    }

					if (!isTeleported)
					{
						DoTeleport();
					}
                } else
				{
					if(isTeleported == true)
					{
						isTeleported = false;
					}
				}

                previousDistance = distance; 



                //SuperController.LogMessage("distance at " + Time.time + ": " + distance);
            } catch(Exception e)
			{
				SuperController.LogError("err:" + e.ToString());
			}
		}

		void DoTeleport()
		{
            //SuperController.LogMessage("TELEPORT");
            var atom = SuperController.singleton.GetAtomByUid(atomJSON.val);
			if (atom == null) return;
			var delta = SuperController.singleton.playerHeightAdjust - containingAtom.mainController.transform.position.y;
			//SuperController.LogMessage("delta:" + delta);

            Vector3 destination = atom.mainController.transform.position + 0.25f * atom.mainController.transform.forward;
			Vector3 rotation = atom.mainController.transform.eulerAngles;
            MovePlayer_p67(destination, rotation);
			SuperController.singleton.playerHeightAdjust = destination.y - delta;



        }

        public void MovePlayer_p67(Vector3 position, Vector3 rotation)
        {
            SuperController.singleton.ResetNavigationRigPositionRotation();
			

            Vector3 positionCustom_p67 = new Vector3(position.x, 0f, position.z);
            Vector3 rotationCustom_p67 = new Vector3(0f, (rotation.y + 180f) % 360f, 0f);
            float height_p67 = position.y;


            var navigationRigParent = SuperController.singleton.navigationRigParent;

            navigationRigParent.transform.position = positionCustom_p67;
            navigationRigParent.transform.localEulerAngles = rotationCustom_p67;
			


        }

        void FixedUpdate() {
		
		}
		void UpdateSecond()
		{
			if (IsDisabled()) return;

		}
		void UpdateConstant()
		{
			if (IsDisabled()) return;

		}
		void UpdateConstant5()
		{
			if (IsDisabled()) return;

		}
		void OnDestroy()
		{
			CancelInvoke();
			SuperController.singleton.onSceneLoadedHandlers -= OnSceneLoaded;
		}

		void OnSceneLoaded()
		{
			sceneLoaded = true;
		}
		bool IsDisabled()
        {
			if (!enabledJSON.val)
			{
				return true;
			}
			return false;
		}

        protected void SyncAtomChoices()
        {
            List<string> atomChoices = new List<string>();
            atomChoices.Add("None");
            foreach (string atomUID in SuperController.singleton.GetAtomUIDs())
            {
                atomChoices.Add(atomUID);
            }
            atomJSON.choices = atomChoices;
        }

        protected void SyncAtom(string atomUID)
		{
		}

        public float GetTransformDistance_p67(Transform a, Transform b)
        {
            return Mathf.Abs(Vector3.Distance(a.position, b.position));
        }

        public float GetTransformDistanceToCamera_p67(Transform a)
        {
            return GetTransformDistance_p67(a, SuperController.singleton.lookCamera.transform);
        }
    }
}