﻿using SimpleJSON;
using System;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using ZenFulcrum.EmbeddedBrowser;
using System.Linq;
using System.Collections;


namespace SPQR
{
	public class VamWidgetController : MVRScript
	{

		//--------------------------- BROWSER CONTROLLER --------------------------
		void SPQRBrowserDebug(string s)
        {
			SuperController.LogMessage("received: " + s);
        }

		 void SPQRBrowserShowDevTools(Dictionary<string, string> input)
        {
			var atomid = input["atomid"];
			var enabled = input["enabled"] == "true";
			Atom browser = SuperController.singleton.GetAtomByUid(atomid);
			if (browser == null) return;
			Browser zfb4 = browser.reParentObject.GetComponentInChildren<Browser>();
			if (zfb4 == null) return;
			zfb4.ShowDevTools(enabled);
		}

		 void SPQRBrowserPressKey(Dictionary<string, string> input)
		{
			var atomid = input["atomid"];
			int keyint = 0;
			int.TryParse(input["key"], out keyint);
			KeyCode key = (KeyCode)keyint;
			Atom browser = SuperController.singleton.GetAtomByUid(atomid);
			if (browser == null) return;
			Browser zfb4 = browser.reParentObject.GetComponentInChildren<Browser>();
			if (zfb4 == null) return;
			zfb4.PressKey(key);
		}


		 void SPQRBrowserSend(Dictionary<string,string> input)
		{
			//SuperController.LogMessage("SPQRBrowserSend receieved");
			var atomid = input["atomid"];
			var data = input["data"];
			Atom browser = SuperController.singleton.GetAtomByUid(atomid);
			if (browser == null) return;


			Browser zfb4 = browser.reParentObject.GetComponentInChildren<Browser>();
			if(zfb4 != null)
            {
				if (!input.ContainsKey("response"))
				{
					zfb4.EvalJS(data).Done();
				} else
                {
					zfb4.EvalJS(data).Then(ret => SPQRBrowserRespond(ret)).Done();
				}
            }

        }

		void SPQRBrowserRespond(ZenFulcrum.EmbeddedBrowser.JSONNode ret)
        {
			var data = JSON.Parse(ret);
			if (data["callback"] == null) return;
			Atom cc = SuperController.singleton.GetAtomByUid("CoreControl");
			cc.gameObject.BroadcastMessage(data["callback"], data);
		}

		void SPQRRegisterCallback(Dictionary<string, string> input)
        {
			var atomid = input["atomid"];
			var callback = input["callback"];
			Atom browser = SuperController.singleton.GetAtomByUid(atomid);
			if (browser == null) return;
			Browser zfb4 = browser.reParentObject.GetComponentInChildren<Browser>();
			if (zfb4 == null) return;

			zfb4.RegisterFunction(callback, args => {
				Atom cc = SuperController.singleton.GetAtomByUid("CoreControl");
				cc.gameObject.BroadcastMessage(args[0].Value.ToString(), args[1].Value.ToString());
				}
			);
		}

		void SPQRBrowserLoadHtml(Dictionary<string, string> input)
		{
			var atomid = input["atomid"];
			var html = input["html"];
			Atom browser = SuperController.singleton.GetAtomByUid(atomid);
			if (browser == null) return;
			Browser zfb4 = browser.reParentObject.GetComponentInChildren<Browser>();
			if (zfb4 == null) return;
			zfb4.LoadHTML(html);
		}


		void SPQRBrowserPointer(Dictionary<string, string> input)
		{
			var atomid = input["atomid"];
			BrowserNative.CursorType pointer = (BrowserNative.CursorType)Enum.Parse(typeof(BrowserNative.CursorType),input["pointer"]);
			Atom browser = SuperController.singleton.GetAtomByUid(atomid);
			if (browser == null) return;
			Browser zfb4 = browser.reParentObject.GetComponentInChildren<Browser>();
			if (zfb4 == null) return;
			zfb4.UIHandler.BrowserCursor.SetActiveCursor(pointer);
		}


		void SPQRBrowserHasMouse(Dictionary<string, string> input)
		{
			var atomid = input["atomid"];
			bool hasmouse = input["hasmouse"] == "true";
			Atom browser = SuperController.singleton.GetAtomByUid(atomid);
			if (browser == null) return;
			Browser zfb4 = browser.reParentObject.GetComponentInChildren<Browser>();
			if (zfb4 == null) return;
			zfb4.UIHandler.BrowserCursor.HasMouse = hasmouse;
		}



		bool sceneLoaded = false;

		Dictionary<string, JSONStorableBool> enableFeatures_p67 = new Dictionary<string, JSONStorableBool>();
		Dictionary<string, JSONStorableString> strings_p67 = new Dictionary<string, JSONStorableString>();
		Dictionary<string, UIDynamicButton> buttons_p67 = new Dictionary<string, UIDynamicButton>();
		Dictionary<string, JSONStorableFloat> sliders_p67 = new Dictionary<string, JSONStorableFloat>();
		Dictionary<string, JSONStorableString> titles = new Dictionary<string, JSONStorableString>();
		UIDynamicTextField htmllengthField;


		public override void Init()
		{

			if (!AllowedAtom()) return;


			CreateTitle_p67("titlepluginname", "VAM WIDGETS v1", 40, false);
			CreateTitle_p67("patreon", "help me do more cool vam stuff if you like this! ❤️ patreon.com/SPQR_AETERNUM", 20, true);

			buttons_p67.Add("copy", CreateButton("Copy HTML code", false));
			buttons_p67["copy"].button.onClick.AddListener(ActionCopy);

			buttons_p67.Add("paste", CreateButton("Paste HTML code", false));
			buttons_p67["paste"].button.onClick.AddListener(ActionPaste);

			buttons_p67.Add("reload", CreateButton("Reload HTML code", false));
			buttons_p67["reload"].button.onClick.AddListener(ActionReload);

			buttons_p67.Add("clear", CreateButton("Clear Page", false));
			buttons_p67["clear"].button.onClick.AddListener(ActionClear);

			buttons_p67.Add("devtools", CreateButton("DevTools", false));
			buttons_p67["devtools"].button.onClick.AddListener(ActionDevtools);

			strings_p67.Add("htmltext", new JSONStorableString("HTML", GetDefaultEmpty(), CallbackUpdateHtml_p67));


			var htmllengthString = new JSONStorableString("Length", GetHtmllengthString());
			htmllengthField = CreateTextField(htmllengthString, true);
			htmllengthField.UItext.supportRichText = true;
			htmllengthField.backgroundColor = new Color(0.42f, 0.03f, 0.51f, 0.7f);
			htmllengthField.textColor = new Color(1f, 1f, 1f, 0.9f);
			htmllengthField.height = 700f;

			foreach (var str in strings_p67)
			{
				strings_p67[str.Key].storeType = JSONStorableParam.StoreType.Full;
				RegisterString(strings_p67[str.Key]);
			}

			InvokeRepeating("UpdateSecond", 1f, 1f);
			InvokeRepeating("UpdateConstant", 1f, 1f / 30f);
			InvokeRepeating("UpdateConstant5", 1f, 1f / 5f);
			SuperController.singleton.onSceneLoadedHandlers += OnSceneLoaded;
		}


		void Start()
		{
			if (!AllowedAtom()) return;
			ActionReload();
			Dictionary<string, string> data = new Dictionary<string, string>() {
				{ "atomid", containingAtom.name },
				{ "enabled", "true"}
			};


		}


		void Update()
		{

		}


		void FixedUpdate()
		{

		}
		void UpdateSecond()
		{
			if (IsDisabled()) return;

		}
		void UpdateConstant()
		{
			if (IsDisabled()) return;

		}
		void UpdateConstant5()
		{
			if (IsDisabled()) return;

		}
		void OnDestroy()
		{
			CancelInvoke();
			SuperController.singleton.onSceneLoadedHandlers -= OnSceneLoaded;
		}

		void OnSceneLoaded()
		{
			sceneLoaded = true;
		}
		bool IsDisabled()
		{
			if (!enabledJSON.val)
			{
				return true;
			}
			return false;
		}

		bool AllowedAtom()
		{
			if (containingAtom.type == "WebBrowser") return true;
			if (containingAtom.type == "WebPanel") return true;
			if (containingAtom.type == "WebPanelEmissive") return true;
			return false;
		}

		void CallbackUpdateHtml_p67(string html)
		{
			UpdateHtmllengthText();
			Dictionary<string, string> data = new Dictionary<string, string>() {
				{ "atomid", containingAtom.name },
				{ "html", html},
			};
			containingAtom.gameObject.BroadcastMessage("SPQRBrowserLoadHtml", data);
		}

		void UpdateHtmllengthText()
		{
			htmllengthField.text = GetHtmllengthString();

		}
		string GetHtmllengthString()
		{
			var htmllength = strings_p67["htmltext"].val.Length;

			var status = "";
			var color = "";
			if (htmllength < 1000)
			{
				status = "very quick loading";
				color = "#7CFC00";
			}
			else if (htmllength < 15000)
			{
				status = "quick loading";
				color = "#7CFC00";
			}
			else if (htmllength < 30000)
			{
				status = "slightly slow loading";
				color = "#FFFF33";
			}
			else if (htmllength < 100000)
			{
				status = "slow loading";
				color = "#FF8C00";
			}
			else if (htmllength < 200000)
			{
				status = "very slow loading";
				color = "#FF4500";
			}
			else
			{
				status = "extremely slow loading";
				color = "#DC143C";
			}


			return "\ncode length: <b><color=" + color + ">" + htmllength + "</color></b> characters \n" +
			"status: <b><color=" + color + ">" + status + "</color></b> \n\n---\n\n" +
			"I recommend keeping the code shorter than 100k characters, ideally around <color=#7CFC00>10k</color>. Long code will make saving & loading the atom very slow.  \n\n" +
			"If you edit code directly in the DevTools, make sure to copy it and hit <color=#7CFC00>PASTE</color> in the plugin for it to be saved in the plugin. Changes in DevTools <color=red>do not autosave!</color> \n\n" +
			"To copy the code from DevTools, right click on <html> in the Elements tab, and pick Copy > Copy Outer Html.";


		}
		void ActionPaste()
		{
			strings_p67["htmltext"].val = GUIUtility.systemCopyBuffer;
		}
		void ActionReload()
		{
			CallbackUpdateHtml_p67(strings_p67["htmltext"].val);
		}
		void ActionCopy()
		{
			GUIUtility.systemCopyBuffer = strings_p67["htmltext"].val;
		}
		void ActionClear()
		{
			strings_p67["htmltext"].val = GetDefaultEmpty();
		}

		void ActionDebug1()
		{
			Dictionary<string, string> data = new Dictionary<string, string>() {
					{ "atomid", containingAtom.name },
					{ "pointer", "None"}
				};
			containingAtom.gameObject.BroadcastMessage("SPQRBrowserPointer", data);

			//        Dictionary<string, string> data = new Dictionary<string, string>() {
			//                { "atomid", containingAtom.name },
			//{ "cursor", "url('data:image/svg+xml;utf8,<svg xmlns=&quot;http://www.w3.org/2000/svg&quot; fill=&quot;%23FF000001&quot; height=&quot;48&quot; viewBox=&quot;0 0 24 24&quot; width=&quot;48&quot;><path d=&quot;M0 0h24v24H0z&quot; fill=&quot;none&quot;/><path d=&quot;M20 11H7.83l5.59-5.59L12 4l-8 8 8 8 1.41-1.41L7.83 13H20v-2z&quot;/></svg>'), auto !important" }
			////{"cursor", "crosshair" }
			//            };
			//        containingAtom.gameObject.BroadcastMessage("SPQRBrowserCursor", data);
		}

		void ActionDevtools()
		{
			Dictionary<string, string> data = new Dictionary<string, string>() {
					{ "atomid", containingAtom.name },
					{ "enabled", "true"}
				};
			containingAtom.gameObject.BroadcastMessage("SPQRBrowserShowDevTools", data);
		}


		void CreateTitle_p67(string id, string text, int fontsize, bool right, int spacing)
		{
			CreateTitle_p67(id, text, fontsize, right);
			CreateSpacer_p67(this, spacing, right);
		}


		UIDynamic CreateSpacer_p67(MVRScript X, float height, bool rightSide)
		{
			UIDynamic spacer = X.CreateSpacer(rightSide);
			spacer.height = height;
			return spacer;
		}

		void CreateTitle_p67(string id, string text, int fontsize, bool right)
		{
			if (!titles.ContainsKey(id))
				titles.Add(id, new JSONStorableString("title " + id, "\n<size=" + fontsize + "><b>" + text + "</b></size>"));
			UIDynamicTextField title1text = CreateTextField(titles[id], right);
			title1text.UItext.supportRichText = true;
			title1text.backgroundColor = new Color(0f, 0f, 0f, 0f);
			title1text.textColor = new Color(0.42f, 0.03f, 0.51f, 1f);
			title1text.height = 100f;
			title1text.UItext.alignment = right ? TextAnchor.MiddleRight : TextAnchor.MiddleLeft;
		}


		string GetDefaultEmpty()
		{
			return "<html><head></head><body style=\"font-family:Trebuchet MS;width:100vw;height:100vh;overflow:hidden;background:#333;text-align:center;margin:0px;padding:0px;;\"> <div style=\"box-shadow: inset 0px 0px 0px 5vh #fff;box-sizing:border-box;position:absolute;width:100%;height:100%;left:0;top:0;\"> <h1 style=\";position:absolute;left:50%;top:50%;transform:translate(-50%,-50%);font-size:10vh;width:100%;;color:#fff;margin:0px;\">VAM Widgets</h1> <span style=\"font-weight:bold;position:absolute;bottom:6vh;width:100%;right:6vh;text-align:right;font-size:5vh;color:#CCC\">v1</span></div></body></html>";

		}





	}


}