using System;
using UnityEngine;

namespace SPQR
{
    public class EventType
    {   // using enums crashes vam
        public const byte NONE = 0;
        public const byte ENTERED = 1;
        public const byte HIT = 2;
        public const byte EXITED = 3;
        public const byte STAY = 4;
    }

    public class TriggerEventArgs : EventArgs
    {
        public Collision collision { get; set; }
        public Collider collider { get; set; }

        public Collider collider2 { get; set; }
        public byte evtType { get; set; }
        public bool insideTrigger { get; set; }
    }

    public class TriggerCollide : MonoBehaviour
    {
        TriggerEventArgs lastEvent;

        bool inside = false;

        public static float minTimeBetweenCollisions = 0.25f;
        float lastCollisionTime = 0;

        public event EventHandler<TriggerEventArgs> OnCollide;

        void Awake()
        {
            lastEvent = new TriggerEventArgs
            {
                evtType = EventType.NONE,
                collider = null,
                collision = null,
                insideTrigger = false,
                collider2 = null
            };
        }

        private void OnTriggerEnter(Collider other)
        {
            //other.isTrigger = false;
            inside = true;
            DoCollideEvent(EventType.ENTERED, other, null);
        }

        private void OnTriggerExit(Collider other)
        {
            inside = false;
            lastEvent.evtType = EventType.EXITED;
        }

        protected virtual void OnCollideEvent(TriggerEventArgs e)
        {
           // SuperController.LogMessage("sending event: " + e.collider.name + ", " + e.insideTrigger);
            EventHandler<TriggerEventArgs> handler = OnCollide;
            handler?.Invoke(this, e);
        }

        protected virtual void OnCollisionEnter(Collision collision)
        {
            DoCollideEvent(EventType.HIT, collision.collider, collision);
        }

        protected virtual void OnCollisionExit(Collision collision)
        {
            lastEvent.evtType = EventType.EXITED;
        }

        private void DoCollideEvent(byte evtType, Collider col, Collision c)
        {
            if (Time.timeSinceLevelLoad - lastCollisionTime > minTimeBetweenCollisions)
            {
                if (evtType == lastEvent.evtType || (evtType == EventType.HIT && c == null))
                {
                    return;
                }
                else
                {
                    //SuperController.LogMessage("collide: " + col.name + "time: " + Time.timeSinceLevelLoad);

                    lastCollisionTime = Time.timeSinceLevelLoad;


                    Collider currentColl = GetComponent<Collider>();
                    if (currentColl == null) currentColl = GetComponentInChildren<Collider>();
                    if (currentColl == null) currentColl = gameObject.transform.parent.GetComponentInChildren<Collider>();
                    if (currentColl == null) return;

                    TriggerEventArgs tempEvent = new TriggerEventArgs
                    {
                        evtType = evtType,
                        collider = currentColl,
                        collision = c,
                        insideTrigger = inside,
                        collider2 = col

                    };

                    OnCollideEvent(tempEvent);
                    lastEvent = tempEvent;
                }
            }
        }
    }


}
